<?php
/**
 * Miscellaneous functions
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.7G
 */

define('MF_LOADED',1);     // used to replace require_once

define('READ_BUFFER_SIZE',32384);     // Default amount of data read from file and put into read buffer used for mp3 parsing
define('MAX_READ_BUFFER_LENGTH',3238400); // Max amount of data in read buffer
define('MAX_PARSED_MP3FILE_SIZE',32000000); // Max amount of data in read buffer


/**
 * @desc Download favicons for a list of URLs
 *
 * @param array $urlsList: array of urls
 * @param array $options
 * 				maxRequests: max simultaneous HTTP requests (30)
 * 				toFile=>dir: save result icons to file into dir, using base64encoded(url).ico.format as filename
 * 				HTMLParsing=>true: true(default) to parse HTML for <link rel...
 * @return array (k=>array('format'=>'png'||'gif'||'jpg', 'data'=>image bin content)
 */
function mfGetFavicons($urlsList,$options=array()){
	$favicons=array();
	if(!defined('W_EXPLORERFUNCTIONS_LOADED')) require(INCLUDE_DIR.'explorerFunctions.php');

	// If synchronous monitoring set, add callback to cfCURLMultiRequests
	if(isset($_ENV['outSyncProgress'])) $_ENV['mfGetFaviconsCB']=array('callback'=>'syncProgressCallback'); else $_ENV['mfGetFaviconsCB']=array();

//$urlsList=array('http://www.meteofrance.com/FR/mameteo/prevReg.jsp?LIEUID=REG12&ECHEANCEID=200506041200');

	// Sync update progress
	function syncProgressCallback($nb,$time){outSyncProgressUpdate($nb+$_ENV['mfGetFaviconsOffset']+$_ENV['mfGetFaviconsOffsetSL']);}

	// Get favicon for 'maxRequests' urls
	function mfGetFaviconsSL($urls, $options){
		$result=array(); $base=array();
		$_ENV['mfGetFaviconsOffsetSL']=0;

		// Get /favicon.ico's
		foreach ($urls as $v){
			if(strtolower(substr($v,0,4))!='http') $url=parse_url('http://'.$v); else $url=parse_url($v);
			$uris[]=$url['scheme'].'://'.$url['host'].((isset($url['port']))?':'.$url['port']:'').'/favicon.ico';
		}

		// Download icons
		$res=cfCURLMultiRequests($uris,array(CURLOPT_FAILONERROR=>0,CURLOPT_FOLLOWLOCATION=>1,CURLOPT_HEADER=>1,CURLOPT_MAXREDIRS=>1)+$_ENV['mfGetFaviconsCB'],$redirections);

		// Get favicons from meta tags
		$metas=array();
		foreach ($res as $k=>$v) {
			if($v!==null){
				cfParseHTTPResponse($v,$status,$data,$headers);

				// Icon found
				if($status==200 && isset($headers['content-type']) && cfCmpLeft($headers['content-type'],'image')){
					$ct=substr($headers['content-type'],6);
					if($ct=='gif'||$ct=='png'||$ct=='jpg'||$ct=='jpeg') $icons[$k]=array('format'=>$ct,'data'=>$data); else $icons[$k]=array('format'=>'ico','raw'=>$data);
				}
				// Else, choose between follow redirection or parse html
				else {
					if(isset($redirections[$k])) $metas[$k]=$redirections[$k]; else $metas[$k]=$urls[$k];
				}
			}
		}

		// Some icons couldn't be found, but do no HTML parsing (used for RSS feeds)
		if(count($metas) && !$options['HTMLParsing']){}

		// If some icons couldn't be retreived, parse HTML pages
		elseif(count($metas)){
			$uris2=array();$redirections=array();

			// Retreive HTML pages
			$_ENV['mfGetFaviconsOffsetSL']=count($urls)-count($metas);
			$res2=cfCURLMultiRequests($metas,array(CURLOPT_FOLLOWLOCATION=>1,CURLOPT_HEADER=>1,CURLOPT_MAXREDIRS=>1)+$_ENV['mfGetFaviconsCB'],$redirections);

			// Parse HTML to find meta
			foreach ($res2 as $k=>$v){
				if(($p=stripos($v,'<link rel="shortcut icon"')) || ($p=stripos($v,'<link rel="icon"'))){
					$txt=substr($v,stripos($v,'href="',$p)+6); $txt=substr($txt,0,stripos($txt,'"'));

					if(substr(strtolower($txt),0,4)!=='http'){
						$base=(isset($redirections[$k]))?$redirections[$k]:$uris[$k];
						if($txt[0]=='/') $txt=substr($base,0,strlen($base)-12).$txt;
						else $txt=dirname($base).'/'.$txt;
					}
					$uris2[$k]=$txt;
				}
			}
			// Download found icons
			if(count($uris2)){
				$_ENV['mfGetFaviconsOffsetSL']=count($urls)-count($uris2);
				$res2=cfCURLMultiRequests($uris2,array(CURLOPT_FOLLOWLOCATION=>1,CURLOPT_HEADER=>1,CURLOPT_MAXREDIRS=>1)+$_ENV['mfGetFaviconsCB']);

				foreach ($res2 as $k=>$v) {
					cfParseHTTPResponse($v,$status,$data,$headers);
					if($status==200 /*&& isset($headers['content-type']) && cfCmpLeft($headers['content-type'],'image')*/){
						if(isset($headers['content-type']) && ($ct=substr($headers['content-type'],6)) &&
							($ct=='gif'||$ct=='png'||$ct=='jpg'||$ct=='jpeg'))
							$icons[$k]=array('format'=>$ct,'data'=>$data);
						else $icons[$k]=array('format'=>'ico','raw'=>$data);
					}
					else $metas[$k]=$urls[$k];
				}
			}
		}

		// Convert .ico to .png
		if(isset($icons)) foreach ($icons as $k=>$v) if ($v['format']=='ico') {
			$tmpF=cfAppTempDir().'/ico_tmp'.$k.'.ico';
			$prev=strlen((isset($v['raw']))?$v['raw']:$v['data']);
			file_put_contents($tmpF,isset($v['raw'])?$v['raw']:$v['data']);
			$icons[$k]['data']=efExtractIcon($tmpF,false,16,array('icoPngCheck'=>1));
			$icons[$k]['format']='png';
			//cfVarDump($k.' '.'>'.$prev.'>'.strlen($icons[$k]['data']));
			unset($icons[$k]['raw']);
			if(isset($tmpF)) @unlink($tmpF);
		}

		foreach ($urls as $k=>$v) if(!isset($icons[$k])) $icons[$k]=array('format'=>'no','data'=>false);

		foreach ($urls as $k=>$v) $icons[$k]['url']=$urls[$k];

		return $icons;
	}


	$options=$options+array('maxRequests'=>30,'HTMLParsing'=>true);
	$maxRequest=$options['maxRequests'];
	$nb=0;
	$tmpUrls=array();
	$_ENV['mfGetFaviconsOffset']=0;

	// Split requests into groups of $maxRequest
	while(count($urlsList)){
		$tmpUrls[]=array_shift($urlsList);
		if((++$nb)==$maxRequest || !count($urlsList)) {
			foreach (mfGetFaviconsSL($tmpUrls,$options) as $v) $favicons[]=$v;
			$nb=0;	$tmpUrls=array();
			$_ENV['mfGetFaviconsOffset']+=$maxRequest;
		}
	}

	// Save result to files
	if(isset($options['toFile'])){
		$dir=$options['toFile'];
		foreach ($favicons as $k=>$v){
			//echo $v['url'].(($v['format']=='no')?'NO TN':':<img src="data:image/'.$v['format'].';base64,'.base64_encode($v['data']).'">').'<br>';
			$cfn=$dir.'/'.md5($v['url']).'.ico.'.$v['format'];
			file_put_contents($cfn,$v['data']);
		}
		return count($favicons);
	}

	return $favicons;
}

/**
 * @desc Return url's favicon filename, false if not generated, 'noIcon' if favicon has not been found for this url
 *
 * @param string $url
 * @return mixed string / false
 */
function mfFaviconFile($url){
	$cfn=cfAppResourceDir().'/'.md5($url);
	foreach (array('png','gif','jpg','jpeg','no') as $v)
		if(file_exists($cfn.'.ico.'.$v)) return (($v=='no')?'noIcon':$cfn.'.ico.'.$v);
	return false;
}

/**
 * @desc Return url's favicon filename, false if not generated, 'noIcon' if favicon has not been found for this url
 *
 * @param string $url
 * @return mixed string / false
 */
function mfFaviconLink($url){
	$ico=mfFaviconFile($url);
	if($ico=='noIcon') return outIcon('blankFile'); else return cfExtImage($ico);
}

/**
 * @desc Download a file, and synchronously display progress bar
 *
 * @param string $url: URL of file to download
 * @param string $destFile: destination file
 * @param array $options: array of progress page display options (caption,icon,standalone,display,suspendSession, appendHTML (HTML appened after progress bar), cancelHref (href of cancel button, if not set, no cancel button)...)
 */
function mfMonitoredDownload($url,$destFile,$options=array()){
	require_once(INCLUDE_DIR.'outputFunctions.php');

	// Insert progress monitoring HTML stuff
	outSyncProgress(@$options['caption'],@$options['icon'],100,((isset($options['standalone']))?$options['standalone']:2),$options);
	if (isset($options['cancelHref'])) echo '<br>'.outButton(cfCaption('genCancel'),$options['cancelHref'],outIcon('cancel')).'<br>';
	if(isset($options['appendHTML'])) echo $options['appendHTML'];
	outSyncProgressUpdate(0,false);
	flush();

	// Max 3 redirections
	$proceed=4;
	while($proceed){
		if(!($handle=cfSocketHTTPRequest($url,7,false,$foo1,$foo2,false,true))) return false;
		$response='';
		while(strlen($response)<1023 && !feof($handle)) $response .= fread($handle, 1024);

		$pos=strpos($response,"\r\n\r\n");
		if($pos === false){
			fclose($handle);
			outSyncProgressUpdate('done');
			return false;
		}
		$header=substr($response,0,$pos);
		$response=substr($response,$pos+4);

		$headers = array();
		$lines = explode("\r\n", $header);
		$fSize=0;
		if(count($lines)==0) {outSyncProgressUpdate('done');return false;}

		// HTTP response code
		$responseCode=substr($lines[0],9,3);

		// Seek file size or redirection
		foreach($lines as $value){
			if(strtolower(substr($value,0,15))=='content-length:') $fSize=trim(substr($value,15));
			if(strtolower(substr($value,0,9))=='location:') $redir=trim(substr($value,9));
		}
		// If redirection
		if(isset($redir)) {
			@fclose($handle);
			$proceed--;
			if($proceed<=0) {
				outSyncProgressUpdate('done');
				return false;
			}
			$updateUri=$redir;
			if(substr($redir,0,1)=='/') {
				$url=parse_url($url);
				$url=$url['scheme'].'://'.$url['host'].((isset($url['port']))?':'.$url['port']:'').$redir;
			}
			else $url=$redir;
			unset($redir);
		}
		// ! 200 OK
		elseif($responseCode!='200') {outSyncProgressUpdate('done');return false;}
		// No redirection and response code 200 OK
		else $proceed=0;
	}

	// Proceed Download
	$handlew = fopen ($destFile, "w");
	if(!$handlew) {fclose($handle);outSyncProgressUpdate('done');return false;}
	$i=-1;
	fwrite($handlew,$response);
	$dlSize=strlen($response);

	// Ignore user abort to detect clicks on cancel button
	ignore_user_abort(true);

	while (!feof($handle)) {
		$response=fread($handle, 8192);
		fwrite($handlew,$response);
		$i++;$dlSize+=strlen($response);

		if($i%4==0) outSyncProgressUpdate($dlSize,$fSize);

		// If user aborted, delete destination file and exit
		if(connection_aborted()){
			fclose($handle);
			fclose($handlew);
			@unlink($destFile);
			exit;
		}
	}
	fclose($handlew);
	fclose($handle);

	outSyncProgressUpdate('done');

	return $dlSize;
}


/**
 * @Desc Class used to retreive frames of ID3V2 tagged mp3.
 * Check http://www.mp3-tech.org/ or http://www.datavoyage.com/mpgscript/mpeghdr.htm for info about format
 *
 */
class id3V2Ext{
	public $version; // ID3 version
	public $tag=array(); // Contained tags array
	private $data; // (start of file content)
	private $dataLength; // Length of data
	private $framesNb=0; // Number of mp3 frames (used for vbr calculations)
	public $br; // Bitrate (b/s)
	public $vbr=0; // Boolean: true if vbr
	private $fp; // File pointer
	private $fileSize; // mp3 file size
	private $firstFramePos=false; // position of 1st mp3 frame
	private $offset=0; // Total length of data removed from the begining of buffer

	static $frames=array('CRA'=>'audioEncr','AENC'=>'audioEncr','PIC'=>'attPict','APIC'=>'attPict','ASPI'=>'audioSeekPntIdx',
						'COM'=>'comment','COMM'=>'comment','COMR'=>'commercial','ENCR'=>'encrMethodReg','EQU'=>'equalisation',
						'EQUA'=>'equalisation','EQU2'=>'equalisation2','ETC'=>'eventTimingCodes','ETCO'=>'eventTimingCodes',
						'GEO'=>'encObj','GEOB'=>'encObj','GRID'=>'grpIdReg','LNK'=>'lnkInf','LINK'=>'lnkInf','MCI'=>'cdId',
						'MCDI'=>'cdId','MLL'=>'mpgLocLookupTbl','MLLT'=>'mpgLocLookupTbl','OWNE'=>'ownership','PRIV'=>'private',
						'CNT'=>'playCnt','PCNT'=>'playCnt','POP'=>'popularimeter','POPM'=>'popularimeter','POSS'=>'posSynch',
						'BUF'=>'recmdBufSize','RBUF'=>'recmdBufSize','RVA'=>'relVolAdj','RVAD'=>'relVolAdj','RVA2'=>'relVolAdj2',
						'REV'=>'reverb','RVRB'=>'reverb','SEEK'=>'seek','SIGN'=>'signature','SLT'=>'synchLyric','SYLT'=>'synchLyric',
						'STC'=>'synchTempoCode','SYTC'=>'synchTempoCode','TAL'=>'album','TALB'=>'album','TBP'=>'bpm','TBPM'=>'bpm',
						'TCM'=>'composer','TCOM'=>'composer','TCO'=>'genre','TCON'=>'genre','TCR'=>'copyright','TCOP'=>'copyright',
						'TDEN'=>'encodingTime','TDLY'=>'playlistDelay','TDOR'=>'orgReleaseTime','TDRC'=>'recTime',
						'TDRL'=>'releaseTime','TDTG'=>'taggingTime','TEN'=>'encodedBy','TENC'=>'encodedBy','TXT'=>'lyricist',
						'TEXT'=>'lyricist','TFLT'=>'filetype','TIPL'=>'involvedPeopleList','TT1'=>'description',
						'TIT1'=>'description','TT2'=>'title','TIT2'=>'title','TT3'=>'subtitle','TIT3'=>'subtitle',
						'TKEY'=>'initialKey','TLA'=>'language','TLAN'=>'language','TLE'=>'length','TLEN'=>'length',
						'TMCL'=>'musicianCreditsList','TMED'=>'mediaType','TMOO'=>'mood','TOT'=>'originalAlbum',
						'TOAL'=>'originalAlbum','TOF'=>'originalFilename','TOFN'=>'originalFilename','TOL'=>'originalLyricist',
						'TOLY'=>'originalLyricist','TOA'=>'originalArtist','TOPE'=>'originalArtist','TOWN'=>'fileOwner',
						'TP1'=>'artist','TPE1'=>'artist','TP2'=>'band','TPE2'=>'band','TP3'=>'conductor','TPE3'=>'conductor', // prod
						'TP4'=>'remixer','TPE4'=>'remixer','TPOS'=>'partOfASet','TPRO'=>'producedNotice','TPB'=>'publisher',
						'TPUB'=>'publisher','TRK'=>'track','TRCK'=>'track','TRSN'=>'IRSName','TRSO'=>'IRSOwner','TSI'=>'size',
						'TSIZ'=>'size','TSOA'=>'albumSortOrder','TSOP'=>'performerSortOrder','TSOT'=>'titleSortOrder','TRC'=>'isrc',
						'TSRC'=>'isrc','TSS'=>'encoderSettings','TSSE'=>'encoderSettings','TSST'=>'setSubtitle','TXX'=>'text',
						'TXXX'=>'text','TYE'=>'year','TYER'=>'year','UFI'=>'uniqueFileId','UFID'=>'uniqueFileId','USER'=>'termsOfUse',
						'ULT'=>'lyrics','USLT'=>'lyrics','WCM'=>'commInfo','WCOM'=>'commInfo',
						'WCP'=>'copyrightInfo','WCOP'=>'copyrightInfo','WAF'=>'webOffAudioFile','WOAF'=>'webOffAudioFile',
						'WAR'=>'webOffArtist','WOAR'=>'webOffArtist','WAS'=>'webOffAudioSrc','WOAS'=>'webOffAudioSrc',
						'WORS'=>'webOffIRS','WPAY'=>'payment','WPB'=>'webOffPubl','WPUB'=>'webOffPubl','WXX'=>'webUserdef',
						'WXXX'=>'webUserdef'); // prod

	// Add data to buffer
	private function readData($size=READ_BUFFER_SIZE){
		if($size=='all') $size=$this->fileSize;
		if($size>$_ENV['memory_limit']) return false;
		$this->data.=fread($this->fp,$size);
		$this->dataLength=strlen($this->data);
		return true;
	}

	/**
	 * Read buffer, and automatically read and flush unused data (RUF)
	 * @param $from: absolute position from begining of file
	 * @param $length: length of requested data
	 */
	private function getData($from,$length){
		// Not enough data into buffer: refill
		if($this->dataLength<$from+$length) {
			$this->readData(max(READ_BUFFER_SIZE,$from+$length-$this->dataLength));

			// Destroy the begining of buffer if overall size is too large
			if($this->dataLength-$this->offset>MAX_READ_BUFFER_LENGTH){
				$this->data=substr($this->data,$from-$this->offset);
				$this->offset=$from;
			}
		}

		// Return requested data
		return substr($this->data,$from-$this->offset,$length);
	}

	/**
	 * Parse mp3 frame header
	 *
	 * @param string $header4B: 4 bytes header
	 * @param integer $pos
	 * @return array of properties if OK, false if not an header
	 */
	private function parseHeader($header4B,$pos){
		static $freqs=array(2.5=>array('00'=>11025,'01'=>12000,'10'=>8000),
					2=>array('00'=>22050,'01'=>24000,'10'=>16000),
					1=>array('00'=>44100,'01'=>48000,'10'=>32000));
		static $bitrates=array(
					'11'=>array(-1,32,64,96,128,164,192,224,256,288,320,352,384,416,448,-1),
					'12'=>array(-1,32,48,56,64,80,96,112,128,160,192,224,256,320,384,-1),
					'13'=>array(-1,32,40,48,56,64,80,96,112,128,160,192,224,256,320,-1),
					'21'=>array(-1,32,48,56,64,80,96,112,128,144,160,176,192,224,256,-1),
					'22'=>array(-1,8,16,24,32,40,48,56,64,80,96,112,128,144,160,-1),
					'23'=>array(-1,8,16,24,32,40,48,56,64,80,96,112,128,144,160,-1)
					);
		static $fields;
		static $header;

		// Convert header into bits string
		$header='';	for($i=0;$i<4;$i++)	$header.=str_pad(decbin(ord($header4B[$i])),8,'0',STR_PAD_LEFT);

		// Frame sync
		if(substr($header,0,11)!='11111111111') return false;
		$fields=array();

		// MPEG Audio version ID
		$fields['vid']=((substr($header,11,2)=='00')?2.5:((substr($header,11,2)==10)?2:1));

		// Layer description
		$fields['layer']=((substr($header,13,2)=='01')?3:((substr($header,11,2)==10)?2:1));

		// 16bits CRC after header ?
		$fields['crc']=1-(int)$header[15];

		// Sampling rate frequency
		$fields['freq']=substr($header,20,2);
		if(!isset($freqs[$fields['vid']][$fields['freq']])) return false;
		$fields['freq']=$freqs[$fields['vid']][$fields['freq']];

		// Padding
		$fields['pad']=$header[22];

		// Bitrate
		$fields['br']=bindec(substr($header,16,4));
		$index=floor($fields['vid']).$fields['layer'];
		if(!isset($bitrates[$index][$fields['br']])) return false;
		$fields['br']=1000*$bitrates[$index][$fields['br']];
		if($fields['br']<=0) return false;

		// Compute frame length
		if($fields['layer']==1) $fields['length']=(12*$fields['br']/$fields['freq']+(int)$fields['pad'])*4;
		else $fields['length']=144*$fields['br']/$fields['freq']+(int)$fields['pad'];
		$fields['length']=floor($fields['length']);

		if($fields['crc']) $fields['length']+=1;

		// Check for another header after frame length (just check 2 first bytes)
		if($pos+$fields['length']+4<$this->fileSize){
			// Read data from file if needed
			//while($pos+$fields['length']+4>$this->dataLength && $this->dataLength<$this->fileSize) $this->readData();
			if(substr($this->data,$pos+$fields['length'],2)!=substr($header4B,0,2)) return false;
			$fields['nextPos']=$pos+$fields['length'];
		}
		else{
			$fields['nextPos']=0;
		}
		return $fields;
	}

	/**
	 * @desc Generate tags array
	 *
	 * @param string $completeFilename: path and filname
	 * @param string $getOnlyTag: (optional) discard all tags not named (tag ID) like this
	 * @param boolean $process: (optional) decode tags, replace keys by key names
	 * @param boolean $vbrParsing: (optional) true to read all frames to compute track length and bitrate
	 * @return boolean: true on success
	 */
	function __construct($completeFilename, $getOnlyTag=false,$process=false,$vbrParsing=false){
		$this->tag=array();

		if(!($this->fp=@fopen($completeFilename,'rb'))) return false;

		// Set max read size, to prevent memory problems
		$_ENV['memory_limit']=min(((int)str_replace('M','000000',ini_get('memory_limit')))/2,MAX_PARSED_MP3FILE_SIZE);

		// Read 1st block of data 65536 B
		$this->readData();

		/**
		 * Parse file header
		 */
		$header=substr($this->data,0,10);

		// Check for "ID3" tag
		if(substr($header,0,3)!='ID3') {
			// If ID3 tag not found, check for mp3 frame header - removed as some mp3 with no ID3 data may start by something else...
			if(false && $this->parseHeader(substr($header,0,4),0)==false){
				fclose($this->fp);
				return false;
			}
			$this->fileSize=filesize($completeFilename);
			$pos=-1;
			$totalLength=0;
		}
		// "ID3" tag found, proceed
		else{
			$this->fileSize=filesize($completeFilename);
			$flag = 0;
			$this->version = ($majorVersion=ord($header[3])).'.'.ord($header[4]);
			$flag = ord($header[5]);

			// Compute tag data length
			$totalLength=($flag & 64)?-10:0;
			$nb=0;

			for($i=0;$i<4;$i++)	$bytes[$i] = ord($header[6+$i]);
			for($i=3;$i>=0;$i--) for($o=0;$o<7;$o++){
				if(($bytes[$i] & 1<<$o)==1<<$o) $totalLength+=1<<$nb;
				$nb++;
			}
			// Incorrect length, abort
			if($totalLength<=0) {fclose($this->fp);$this->data='';return false;}

			// Put at least tag data length of data into buffer and extract tag
			while ($totalLength>$this->dataLength) $this->readData();
			$tagData = substr($this->data,10,$totalLength+4);

			$pos=0;

			/**
			 * Browse tag frames
			 */
			while($pos<$totalLength){
				// V2.2
				if($majorVersion<3){
					// Get frame ID
					if(!($id=trim(substr($tagData,$pos,3)))) break;
					// Frame size
					$unpacked=@unpack('N',"\0".substr($tagData,$pos+3,3));
					if(!isset($unpacked[1])) break;
					$size=$unpacked[1];
					$pos+=6;
				}
				// V2.3 & V2.4
				else{
					// Get frame ID
					if(!($id=trim(substr($tagData,$pos,4)))) break;
					// Frame size
					$unpacked=@unpack('Nv',substr($tagData,$pos+4,4));
					if(!isset($unpacked['v'])) break;
					$size=$unpacked['v'];
					// Skip flag
					$pos+=10;
				}

				// If not filtered tags, skip
				if($getOnlyTag && $getOnlyTag!=$id) {
					$pos+=$size;
					continue;
				}

				// Get tag data
				if($process){
					$pid=$id;

					// Skip binary frames
					if($id=='APIC'||$id=='GEOB'||$id=='MCDI'||$id=='PRIV') $this->tag[$id]='TRUE';
					elseif($size>1){
						if(isset(id3V2Ext::$frames[$id])) $id=id3V2Ext::$frames[$id];
						$tmpTag=substr($tagData,$pos+1,$size-1);
						// Corrects comment encoding
						if(($id=='comment'||$id=='COMM') && ($tmpTag[3]=="\0"||substr($tmpTag,0,3)=='eng')){
							if(substr($tmpTag,4,1)=='^') $tmpTag=substr($tmpTag,5);
							elseif(substr($tmpTag,3,1)=="\0") $tmpTag=substr($tmpTag,4);
							else $tmpTag=substr($tmpTag,3);
						}

						// De-UTF16 encode
						if(substr($tmpTag,0,2)=='') $tmpTag=cfCodePageDecode($tmpTag,'UTF-16');
						else if(substr($tmpTag,0,2)=='') $tmpTag=cfUTF8Decode($tmpTag,true,false,false);
						// Special case: ill? encoded unicode indicator
						elseif(strpos($tmpTag,"\0")!==false) {
							$tmpTag=cfCodePageDecode("".substr($tmpTag,strpos($tmpTag,"\0")+3),'UTF-16');
						}
						// Remove remaining nulls
						$tmpTag=str_replace("\0",'',$tmpTag,$nb);
						if(strlen($tmpTag)) $this->tag[$id]=$tmpTag;
					}
				}
				else $this->tag[$id]=substr($tagData,$pos,$size);
				$pos+=$size;

				if($getOnlyTag) break;
			}
		}

		/**
		 * Track Length
		 * If not set in tags, compute track length by dividing file size by bitrate
		 */
		if(!isset($this->tag['length'])){
			$dcc=chr(255);
			if($pos>$this->fileSize) $pos=0;
			if($pos+14>=$this->dataLength) $this->readData(max(READ_BUFFER_SIZE,$pos+4-$this->dataLength));
			$pos=10+$totalLength;
			if(!$this->parseHeader(substr($this->data,$pos,4),$pos)) $pos=0; else $pos--;

			// Locate 1st mp3 frame from $pos, in order to find bitrate
			$iter=0; // Max number of iterations, used to prevent hanging on incorrect files
			$nf=0;

			while (($vbrParsing || $iter++<2500) && $this->dataLength /* Check data has been read ie. file size not too big */) {
				// Long chr(255) sequences : skip all tests to find 1st non 255
				if($nf>100){
					$nf=0;
					if(!$this->readData('all')) break; // potential memory problem: abort
					while ($pos+$nf<$this->fileSize && $this->data[$pos+$nf]==$dcc)	$nf++;
					if($pos+$nf==$this->fileSize) break(2); else $pos+=$nf-2;
				}
				// Over 1e5 attempts, fail
				if($iter>100000) break;

				// Search for 11111111 sync sequence
				$pos=strpos($this->data,$dcc,$pos+1);

				if($pos===false	){
					// sync sequence not found: read extra data and restart
					if($this->dataLength < $this->fileSize){
						$pos=$this->dataLength;
						if(!$this->readData()) break; // Refill buffer
						continue;
					}
					// EOF reached, frame not found !
					break;
				}

				// Refill buffer if needed
				if($pos+4>$this->dataLength){
					if(!$this->readData()) break; // Refill buffer
					if($pos+4>$this->dataLength) break;
				}


				// Verify it's actually a frame header, and retreive info
				if(!($h=$this->parseHeader(substr($this->data,$pos,4),$pos))) {$nf++;continue;}

				// Below this point, correct frame found !
				$nf=0;
				if($this->firstFramePos===false) $this->firstFramePos=$pos;
				$this->framesNb++;
				$this->br+=$h['br'];

				// If not parsing all frames, stop here
				if(!$vbrParsing) break;

				// Parsing all frames: put whole file in buffer
				if(!$this->readData('all')) break;

				$firstBr=$h['br'];

				// Parsing all frames: directly jump to next frame
				while ($pos=$h['nextPos']){

					if(!($h=$this->parseHeader(substr($this->data,$pos,4),$pos))) break;
					$this->framesNb++;
					$this->br+=$h['br'];

					if($this->framesNb<=500){
						if($h['br']!==$firstBr) $this->vbr=1;
						// If 500 frames parsed and no vbr detected, abort
						if($this->framesNb==500 && !$this->vbr){
							$this->framesNb=1;
							$this->br=$firstBr;
							break(2);
						}
					}
				}
				$this->tag['vbr']=$this->vbr;
				break;
			}

			// If at least one frame found
			if($this->framesNb>0){
				//if($this->framesNb>1) cfVarDump($this->framesNb.' frames');
				// Average bitrage (bps)
				$this->tag['br']=$this->br/$this->framesNb;
				// Track length
				//cfVarDump('frame found at '.$this->firstFramePos);
				$this->tag['length']=ceil(8000*($this->fileSize - $this->firstFramePos)/$this->tag['br']); // Set length (ms)
			}
		}

		fclose($this->fp);
		$this->data='';
	}

	/**
	 * @desc See __construct
	 *
	 * @param string $completeFilename: path and filname
	 * @param string $getOnlyTag: (optional) discard all tags not named (tag ID) like this
	 * @param boolean $process: (optional) decode tags, replace keys by key names
	 * @param boolean $vbrParsing: (optional) true to read all frames to compute track length and bitrate
	 * @return boolean: true on success
	 */
	function parseFile($completeFilename, $getOnlyTag=false,$process=false,$vbrParsing=false){
		$this->__construct($completeFilename, $getOnlyTag,$process,$vbrParsing);
	}

	/**
	 * @desc Generate image from embeded data (send headers and image)
	 *
	 * @param int $width: (optional) output width (if not set, use actual image width)
	 * @param int $height: (optional) output height (if not set, use actual image height)
	 * @return boolean false if image couldn't be generated
	 */
	public function generate($width=0,$height=0){
		global $debugRemoveContentHeader;

		// Search for APIC tag
		foreach ($this->tag as $k=>$v) if($k=='APIC'){
			$pos=0;

			if($v[$pos++]!="\0") return false;
			while($v[$pos]!="\0" && $pos<strlen($v))	$pos++;
			$mimeType=substr($v,1,$pos-1);
			$pos+=2;
			while($v[$pos]!="\0" && $pos<strlen($v))	$pos++;
			$pos++;

			// Resized image
			if($width && $height){
				if(!$in=@imagecreatefromstring(substr($v,$pos))){
					// BMP
					if(substr($v,$pos,2)=='BM'){
						require_once(INCLUDE_DIR.'bmp.php');
						$in=ImageCreateFromBMP(substr($v,$pos),false);
					}
				}

				if($in){
					if(!@$debugRemoveContentHeader) header('Content-Type: image/jpeg');
					$out=imagecreatetruecolor($width,$height);
					fastimagecopyresampled($out,$in,0,0,0,0,$width,$height,imagesx($in),imagesy($in),3);
					imagejpeg($out,null,70);
					imagedestroy($in);
					imagedestroy($out);
					exit;
				}
			}

			// Original image
			if(!@$debugRemoveContentHeader) header('Content-Type: '.$mimeType);
			echo substr($v,$pos);
			exit;
		}
	}

	/**
	 * @desc Tell if an mp3 has an attached picture
	 *
	 * @return boolean
	 */
	public 	function hasImage(){return isset($this->tag['APIC']);}

	private function get_int32($str){
		$unpacked = unpack('Nv',$str);
		return $unpacked['v'];
	}
}


/**
 * @desc Extract metadata from FLAC files
 *
 */
class FLACMetaData{
	public $tag;

	static $frames=array('title'=>'title','album'=>'album','tracknumber'=>'track','artist'=>'artist','albumartist'=>'band','album artist'=>'band','copyright'=>'copyright','organization'=>'commercial','genre'=>'genre','genre'=>'genre','date'=>'year','isrc'=>'isrc');

	function __construct($completeFilename){
		if(!($fp=@fopen($completeFilename,'rb'))) return false;
		// FLAC Header
		if(fread($fp,4)!='fLaC') return false;
		while (1) {
			// Read block header
			$blockHeader=fread($fp,4);
			if(strlen($blockHeader)!=4) break;
			// Last block
			if(ord($blockHeader[0])>127){
				$blockHeader[0]=chr(ord($blockHeader[0]-128));
				$lastBlock=true;
			}
			// Block type
			$blockType=ord($blockHeader[0]);

			// Block size
			$size=@unpack('N',"\0".substr($blockHeader,1));
			if(!isset($size[1])) break;
			$size=$size[1];

			if($blockType==0 || $blockType==4) {
				$data=fread($fp,$size);

				// METADATA_BLOCK_STREAMINFO
				if($blockType==0){
					/* TODO
					$sr=substr($data,8,2);
					$sr=unpack('n',$sr);
					$srl=substr($data,9,1);
					$srl=$srl>>4;
					*/
				}
				// METADATA_BLOCK_VORBIS_COMMENT
				if($blockType==4){
					// Skip vendor
					$subsize=@unpack('V',substr($data,0,4));
					if(!isset($subsize[1])) break;
					$pos=4+$subsize[1];
					$nbComments=unpack('V',substr($data,$pos,4));
					$pos+=4;
					// Read comments
					for($i=0;$i<$nbComments;$i++){
						$subsize=@unpack('V',substr($data,$pos,4));
						if(!isset($subsize[1])) break;
						$comment=substr($data,$pos+4,$subsize[1]);
						$pos+=4+$subsize[1];
						@list($k,$v)=explode('=',$comment,2);
						$k=strtolower($k);
						if(isset(FLACMetaData::$frames[$k])) $this->tag[FLACMetaData::$frames[$k]]=cfUTF8Decode($v);
					}
					// No usefull info out of comments block and streaminfo blocks (and streaminfo is 1st)
					break;
				}
			}
			else fseek($fp,$size,SEEK_CUR);

			if(isset($lastBlock)) break;
		}
		fclose($fp);
	}
}


/**
 * @desc Extract metadata from OGG files
 *
 */
class OGGMetaData{
	public $tag;

	static $frames=array('title'=>'title','album'=>'album','tracknumber'=>'track','artist'=>'artist','albumartist'=>'band','album artist'=>'band','copyright'=>'copyright','organization'=>'commercial','genre'=>'genre','genre'=>'genre','date'=>'year','isrc'=>'isrc');

	function __construct($completeFilename){
		if(!($fp=@fopen($completeFilename,'rb'))) return false;
		$header=fread($fp,4096);
		$pos=0;
		// OGG Header
		if(substr($header,0,4)!='OggS') {fclose($fp);return false;}
		$serial = substr($header, 14, 4);

		// Directly search for metadata frame
		if(!($pos=strpos($header,"\x03vorbis"))) {fclose($fp);return false;}

		// Read stream description frame
		if($pos=strpos($header,"\x01vorbis")) {
			$this->tag['audioChannels']=ord(substr($header,$pos+7+4,1));
			$this->tag['sampleRate']=unpack('V',substr($header,$pos+7+5,4));
			$this->tag['sampleRate']=$this->tag['sampleRate'][1];
		}

		// Directly search for metadata frame
		if(!($pos=strpos($header,"\x03vorbis"))) {fclose($fp);return false;}

		// Skip vendor
		$pos+=7;
		$pos+=ord(substr($header,$pos,1))+4;

		// Get number of comment fields
		$nbComments=unpack('V',substr($header,$pos,4));
		$pos+=4;

		// Read comments
		for($i=0;$i<$nbComments[1];$i++){
			$subsize=@unpack('V',substr($header,$pos,4));
			if(!isset($subsize[1])) break;
			$comment=substr($header,$pos+4,$subsize[1]);
			$pos+=4+$subsize[1];
			@list($k,$v)=explode('=',$comment,2);
			$k=strtolower($k);
			if(isset(FLACMetaData::$frames[$k])) $this->tag[FLACMetaData::$frames[$k]]=cfUTF8Decode($v);
		}

		// Get track length (http://people.xiph.org/~giles/2006/meta.php.txt)
		if(isset($this->tag['sampleRate'])){
			fseek($fp, -6000, SEEK_END);
			$end = fread($fp, 6000);
			$tail = strstr($end, "OggS");
			if ($tail) {
			  $serial = substr($tail, 14, 4);
			  if ($serial == substr($tail, 14, 4)) {
			    $granulepos = ord($tail[6]);
			    $granulepos = $granulepos | (ord($tail[7]) << 8);
			    $granulepos = $granulepos | (ord($tail[8]) << 16);
			    $granulepos = $granulepos | (ord($tail[9]) << 24);
			    $granulepos = $granulepos | (ord($tail[10]) << 32);
			    $granulepos = $granulepos | (ord($tail[11]) << 40);
			    $granulepos = $granulepos | (ord($tail[12]) << 48);
			    $granulepos = $granulepos | (ord($tail[13]) << 56);
			    $this->tag['length'] = ceil(1000*$granulepos/$this->tag['sampleRate']);
			  }
			}
		}

		fclose($fp);
		return true;
	}
}
?>