<?php
/**
 * Common explorer functions used by scripts displaying mobile devices pages
 *
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2008 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.3.0
 */

require_once(INCLUDE_DIR.'mobileFunctions.php');
require_once(INCLUDE_DIR.'outputFunctions.php');

/**
 * @desc Initialize mobile explorer specific settings
 *
 */
function mefExplorerInit(){
	if(cfRGetVar('resourceMobileConfigured')) return ;

	cfGSetVar("generalHideUnallowedFiles",true); // Hide unallowed files as so far they would be displayed as accessible files. TODO: remove this and display correctly unaccessible files

	cfRInitVar('itemsPerPageMobile',50); // Items per page - 0: unlimited, 10, 20, 50
	if(cfBGetVar('name')=='iPad') cfRSetVar('itemsPerPageMobile',150);
	cfRSetVar('itemsPerPage',cfRGetVar('itemsPerPageMobile')); // Items per page. Use itemsPerPageMobile for initiatlisation
	cfRSetVar('itemsPerPagePos',0); // Current position (offset) in files

	// Default shrinked / expanded image view (0: shrinked, 1: expanded, 2:fullsize)
	cfRInitVar('viewImageZoom',0);

	// Update directory content to see if some files are in it, in order to automatically select file/folder view
	efDirUpdate(array('noThumbnails'=>1));

	// If no files in, display folder view, else display files view
	cfRInitVar('view','concat');

	// Adjust audio player to browser capabilities
	if(!cfBGetVar(cfRGetVar('audioPlayer'))) cfRSetVar('audioPlayer',((cfBGetVar('flash'))?'flash':((cfBGetVar('wmp'))?'wmp':((cfBGetVar('qt'))?'qt':'qtReenc'))));

	// Set a default bitrate for audio streaming
	if(is_array(cfRGetVar('fileTypeFilter')) && in_array('audio',cfRGetVar('fileTypeFilter'))) cfRInitVar('outputBitRate','original');

	// Set default video quality (flash)
	cfRSetVar('flashOutputSize',(cfRGetVar('flashOutputSizeMax')==160)?160:320);
	cfRSetVar('flashOutputQuality',(cfRGetVar('flashOutputQualityMax')==='low')?'low':'medium');

	cfRSetVar('resourceMobileConfigured',1);
}

/**
 * Entry point for mobile explorer scripts
 *
 */
function mefExplorer(){
	// Init explorer mobile-specific data
	mefExplorerInit();

	/*
	 ***************************************************************************************************************************
	 * Process POST commands
	 ***************************************************************************************************************************
	 */
	efProcessPOST();
	mfProcessPOST();
	if(isset($_POST['uploadGetId'])) require(INCLUDE_DIR.'uploadWindow.php');

	/*
	 ***************************************************************************************************************************
	 * Get files and subfolders list, generate thumbnails if needed
	 ***************************************************************************************************************************
	 */
	$thumbnailsNeeded=cfRGetVar('fileTypeFilter') && (in_array('image',cfRGetVar('fileTypeFilter')) || in_array('video',cfRGetVar('fileTypeFilter')));
	efDirUpdate(array('includeRights'=>1,'noExtra'=>1)+(($thumbnailsNeeded)?array():array('noThumbnails'=>1)));

	// Correct view type according to actual content
	$view=cfRGetVar('view');

	//if(!isset($_POST['data3']) || $_POST['data3']!='setView'){
		if($view=='folders' && $_ENV['weezoResEfFiles']['nbSubDirs']==0 && $_ENV['weezoResEfFiles']['nbFiles']>0) cfRSetVar('view','folders');
		elseif($view=='files' && $_ENV['weezoResEfFiles']['nbFiles']==0) cfRSetVar('view','folders');
	//}
	$view=cfRGetVar('view');

	/*
	 ***************************************************************************************************************************
	 * Insert HEAD, JS functions and form
	 ***************************************************************************************************************************
	 */
	mfInsertHead(array('topControlsExtra'=>mefTopControls()));
	mefInsertScripts();

	// Images & video thumbnails generation & monitoring
	if(isset($_ENV['thumbnailPhotoNeeded']) && is_array($_ENV['thumbnailPhotoNeeded'])) efMakePhotoThumbnails($_ENV['thumbnailPhotoNeeded']); unset($_ENV['thumbnailPhotoNeeded']);
	if(isset($_ENV['thumbnailVideoNeeded']) && is_array($_ENV['thumbnailVideoNeeded'])) efMakeVideoThumbnails($_ENV['thumbnailVideoNeeded']); unset($_ENV['thumbnailVideoNeeded']);

	if(!cfIsAsync()) echo '<div id="foldersDiv"></div>'; // Folders container DIV
	if(!cfIsAsync() && $view!='tools') echo '<div id="mainDiv"></div>'; // Files and tools container DIV (mefDisplayTools() inserts it by itself)

	// If user is browsing folders (or no file in this directory)
	if($view=='folders' || $view=='concat' || $_ENV['weezoResEfFiles']['nbFiles']==0)	mefDisplayFolders();

	// If user is browsing files
	if($view=='files' || $view=='concat')	mefDisplayFiles();

	// If user is viewing tools (mdl, upload, remote dl...)
	if($view=='tools') mefDisplayTools();

	if(cfIsAsync()) die(cfAsyncFooter());
	die('</body></html>');
}

/**
 * @desc Insert scripts needed by explorers
 *
 */
function mefInsertScripts(){
	if(cfIsAsync()) return ;
	//outInsertStandardComForm($_SERVER['PHP_SELF'],cfUTF8Encode(cfResourceRelativePath(cfRGetVar('efCurrentDirectory'))));
	outInsertStandardComForm($_SERVER['PHP_SELF'],cfUTF8Encode(cfResourceRelativePath(cfRGetVar('efCurrentDirectory'))));

	// Common explorer js functions
	if(!cfBGetVar('inlineJS')) echo cfScriptLink('mExplorer.js');

	// Javascript
?>
<script type="text/javascript">
var eV="<?php echo cfRGetVar('view');?>",
icoSdown='<?php echo outIcon('sdown');?>',
icoUp='<?php echo outIcon('sup');?>',
icoFold='<?php echo outIcon('fi/med/folder'); ?>',
icoFoldUp='<?php echo outIcon('fi/med/folderUp'); ?>',
icoBack='<?php echo outIcon('back'); ?>',
capBack="<?php echo cfCaptionJS('genBack');?>",
capRen="<?php echo cfCaptionJS('explorerEnterNewName');?>",
capSup="<?php echo cfCaptionJS('genConfirmSuppress');?>",
capExe="<?php echo cfCaptionJS('explorerEnterParameters');?>";
<?php
echo cfJSRVar('shareAllowed')."\n";
echo cfJSRVar('mailoutAllowed')."\n";

// iPhone orientation management
	if(cfBGetVar('OS')=='iOS'){?>
function updateOrientation(){
	if(W.orientation==undefined) return;
	if(Math.abs(orientation)==90) {
		dgn('viewport').setAttribute("content", "width=<?php echo (cfBGetVar('name')=='iPad')?1024:480;?>, height=<?php echo (cfBGetVar('name')=='iPad')?768:268;?>, user-scalable=yes,initial-scale=1.0, maximum-scale=1.0, minimum-scale=1.0");
		D.body.style.height='<?php echo (cfBGetVar('name')=='iPad')?768:268;?>px';
	}
	else {
		dgn('viewport').setAttribute("content", "width=<?php echo (cfBGetVar('name')=='iPad')?768:320;?>, height=<?php echo (cfBGetVar('name')=='iPad')?1024:416;?>, user-scalable=yes,initial-scale=1.0, maximum-scale=1.0, minimum-scale=1.0");
		D.body.style.height='<?php echo (cfBGetVar('name')=='iPad')?1024:416;?>px';
	}
}
	<?php
	}

	// Inline explorer scripts
	if(cfBGetVar('inlineJS')) echo file_get_contents(cfAppDocRoot().'/js/mExplorer.js');

	/***************************************************************************************************************
	 * All files (explorer)
	 **************************************************************************************************************/
	if(cfRGetVar('fileTypeFilter')==false){
?>
function updtFiles(fl,prev,next){
	files=fl;
	fS=-1;
	if(eV!='concat') eV='files';
	setViewIco();
	var out='<table class="liTable" cellspacing=0 cellpadding=0><colgroup><col width="1%"><col width="98%"><col width="1%"></colgroup>';
	if(prev) out+='<tr class="li" onclick="setPos(this,\'prev\')"><td class="lil">&nbsp;</td><td class="lic" style="text-align:center"><img src="<?php echo outIcon('sup');?>" class="prevNextImg"></td><td class="lir">&nbsp;</td></tr>';

	for(i=0;i<fl.length;i++) {
		out+='<tr id="tr'+i+'" class="li"><td class="lil" onclick="view(dgi(\'tr'+i+'\'))"><p class="frLi16"><img src="'+fl[i][2]+'"></p></td><td class="lic" onclick="sel(this.parentNode,0)"><div class="lits">'+fl[i][1]+'</div>'+fl[i][5]+'</td><td class="lir" onclick="sel(this.parentNode,0)"><b class="goIco"></b></td></tr>';
	}
	if(next) out+='<tr class="li" onclick="setPos(this,\'next\')"><td class="lil">&nbsp;</td><td class="lic" style="text-align:center"><img src="<?php echo outIcon('sdown');?>" class="prevNextImg"></td><td class="lir">&nbsp;</td></tr>';
	updtView(out+'</table>');
}
<?php
}

	/***************************************************************************************************************
	 * Audio files
	 **************************************************************************************************************/
	if(cfRGetVar('fileTypeFilter')==array('audio')){
	?>
function updtFiles(fl,prev,next){
	files=fl;
	fS=-1;
	if(eV!='concat') eV='files';
	setViewIco();
	var out='<table class="liTable" cellspacing=0 cellpadding=0><colgroup><col width="1%"><col width="98%"><col width="1%"></colgroup>';

<?php
	// If browser / OS supports playlists, add playlist-related control
	if(cfBGetVar('playlistFormat')) {?>
if(fl.length) out+='<tr class="li" id="mAddTr"><td onclick="mAddAll(this)" style="width:70px" class="lil"><img src="<?php echo outIcon('addMult')?>" class="ico"></td><td class="lic lit"><?php echo addslashes(cfCaption('explorerAudioPlaylistAdd'));?></td><td class="lir" onclick="plPlayAllIfAny(this)"><?php echo outImage(outIcon('play'),false,'id="plPlayAllBt2"');?></td></tr><?php echo mfListSpacer();?>';
<?php
	} // eo playlist

	// Previous files control
	?>
	if(prev) out+='<tr class="li" onclick="setPos(this,\'prev\')"><td class="lil">&nbsp;</td><td class="lic" style="text-align:center"><img src="<?php echo outIcon('sup');?>" class="prevNextImg"></td><td class="lir">&nbsp;</td></tr>';

<?php // Files ?>
	for(i=0;i<fl.length;i++) {
	out+='<tr id="tr'+i+'" class="li"><?php
	if(cfBGetVar('name')=='IEMobile'){
	?>
<td class="lil"><a href="javascript:view(dgi(\'tr'+i+'\'))"><img src="<?php echo outIcon('play')?>" class="nfrLi32"></a><?php } else { ?>
<td class="lil"><img src="<?php echo outIcon('play')?>" class="nfrLi32" onclick="view(dgi(\'tr'+i+'\'))"><?php }
	if(cfBGetVar('playlistFormat')) {
		if(cfBGetVar('name')=='IEMobile')
			echo '<a href="javascript:mAdd(dgi(\\\'tr\'+i+\'\\\'),\\\'pl\\\')"><img src="'.outIcon('add').'" class="nfrLi32"></a>';
		else
			echo '<img src="'.outIcon('add').'" class="nfrLi32" onclick="mAdd(dgi(\\\'tr\'+i+\'\\\'),\\\'pl\\\')">';
	}
		?></td><td class="lic" onclick="sel(this.parentNode,0)"><div class="lits">'+fl[i][1]+'</div>'+fl[i][6]+'</td><td class="lir" onclick="sel(this.parentNode,0)"><b class="goIco"></b></td></tr>';
	}
	if(next) out+='<tr class="li" onclick="setPos(this,\'next\')"><td class="lil">&nbsp;</td><td class="lic" style="text-align:center"><img src="<?php echo outIcon('sdown');?>" class="prevNextImg"></td><td class="lir">&nbsp;</td></tr>';

	updtView(out+'</table>');
	plUpdtNB();
}
<?php // Number of files in playlist ?>;
var plNB=<?php echo (int)@count(cfRGetVar('playlist')); ?>;
function plUpdtNB(nb){nb=(nb!=undefined)?nb:plNB;plNB=nb;var n=dgi('plPlayAllBt2'); if(n) n.style.display=(nb)?'block':'none'}
function plPlayAllIfAny(n){
	if(!plNB) return;
	wl.liSel(n);
	plPlayAll();
}
	<?php
	}

	/***************************************************************************************************************
	 * Video or multimedia files
	 **************************************************************************************************************/
	if(is_array(cfRGetVar('fileTypeFilter')) && in_array('video',cfRGetVar('fileTypeFilter'))){
?>
function updtFiles(fl,prev,next){
	files=fl; fS=-1;

	if(eV!='concat') eV='files';

	setViewIco();
	var out='<table class="liTable" cellspacing=0 cellpadding=0><colgroup><col width="1%"><col width="98%"><col width="1%"></colgroup>';
	if(prev) out+='<tr class="li" onclick="setPos(this,\'prev\')"><td class="lil">&nbsp;</td><td class="lic" style="text-align:center"><img src="<?php echo outIcon('sup');?>" class="prevNextImg"></td><td class="lir">&nbsp;</td></tr>';

	for(i=0;i<fl.length;i++) {
		out+='<tr id="tr'+i+'" class="li"><td class="lil" onclick="view(this.parentNode)">';
		out+='<div style="width:'+imgw+'px;height:'+imgh+'px" class="tnOuter"><img height="'+imgh+'" width="0" src="/gfx/v.gif"><img src="'+fl[i][2]+'" style="vertical-align:middle"></div>';
		out+='</td><td class="lic" onclick="sel(this.parentNode,0)"><div class="lits">'+fl[i][1]+'</div></td><td class="lir" onclick="sel(this.parentNode,0)"><b class="goIco"></b></td></tr>';
	}
	if(next) out+='<tr class="li" onclick="setPos(this,\'next\')"><td class="lil">&nbsp;</td><td class="lic" style="text-align:center"><img src="<?php echo outIcon('sdown');?>" class="prevNextImg"></td><td class="lir">&nbsp;</td></tr>';
	updtView(out+'</table>');
}
	<?php
	}


	/***************************************************************************************************************
	 * Images files
	 **************************************************************************************************************/
	if(cfRGetVar('fileTypeFilter')==array('image')){
?>
var cT,margin=0;
function sT(){cT=new Date().getTime()}
function iS(n){
	var cn,i=n.id.substr(1);
	if(fS==i) return;
	if(fS!=-1 && (cn=dgi('td'+fS))){
		cn=cn.childNodes;
		cn[0].style.display='';
		while(cn.length>1) dgi('td'+fS).removeChild(cn[1]);
	}
	fS=i;
	return view();
}
function updtFiles(fl,prev,next){
	var out='';
	files=fl; fS=-1;
	if(eV!='concat') eV='files';
	setViewIco();

	if(prev) out+='<table class="liTable" cellspacing=0 cellpadding=0><colgroup><col width="1%"><col width="89%"><col width="10%"></colgroup><tr class="li" onclick="setPos(this,\'prev\')"><td class="lil">&nbsp;</td><td class="lic" style="text-align:center"><img src="'+icoUp+'" class="prevNextImg"></td><td class="lir">&nbsp;</td></tr></table><br/>';
	out+='<div class="imagesGrid">';
	for(i=0;i<fl.length;i++) {
		out+='<img <?php
		if(cfRGetVar('fileViewAllowed')) {
?>
id="f'+i+'" onmousedown="$(this).e.longClick([sel,this],[iS,this])" <?php
		}
		?> src="'+fl[i][2]+'" style="width:'+imgw+'px;height:'+imgh+'px;margin:'+margin+'px">';
	}
	out+='</div>';

	if(next) out+='<br/><table class="liTable" cellspacing=0 cellpadding=0><colgroup><col width="1%"><col width="89%"><col width="10%"></colgroup><tr class="li" onclick="setPos(this,\'next\')"><td class="lil">&nbsp;</td><td class="lic" style="text-align:center"><img src="'+icoSdown+'" class="prevNextImg"></td><td class="lir">&nbsp;</td></tr></table>';
	updtView('<div class="frame1">'+out+'</div>');
}
<?php
	}
?>
var vA={};
vA['dlBt']=["dl(this)","<?php echo outIcon('dl');?>","<?php echo cfCaption('genDownload');?>"];
<?php

// Multiple download / playlist add
if(is_array(cfRGetVar('fileTypeFilter')) && (in_array('audio',cfRGetVar('fileTypeFilter'))||in_array('video',cfRGetVar('fileTypeFilter')))){
?>
vA['mdlBt']=["mAdd(this,'pl')","<?php echo outIcon('mdl');?>","<?php echo cfCaption('explorerAudioPlaylistAdd');?>"];
<?php
	}
	else{
?>
vA['mdlBt']=["mAdd(this)","<?php echo outIcon('addMult32');?>","<?php echo cfCaption('explorerMultipleDownloadAdd');?>"];
<?php
	}
	// View
	if(is_array(cfRGetVar('fileTypeFilter')) && (in_array('audio',cfRGetVar('fileTypeFilter'))||in_array('video',cfRGetVar('fileTypeFilter')))){?>
vA['plBt']=["view(this)","<?php echo outIcon('play');?>","<?php echo cfCaption('explorerAudioPlay');?>"];
<?php } else{ ?>
vA['plBt']=["view(this)","<?php echo outIcon('search32');?>","<?php echo cfCaption('genView');?>"];
<?php }?>
vA['renBt']=["ren(this)","<?php echo outIcon('edit');?>","<?php echo cfCaption('explorerRename');?>"];
vA['exeBt']=["exe(this)","<?php echo outIcon('exec32');?>","<?php echo cfCaption('explorerExecute');?>"];
vA['supBt']=["sup(this)","<?php echo outIcon('cancel32');?>","<?php echo cfCaption('genDelete');?>"];
vA['shareBt']=["share(this)","<?php echo outIcon('share32');?>","<?php echo str_replace('...','',cfCaption('doShare'));?>"];
vA['mailoutBt']=["mailout(this)","<?php echo outIcon('mailOut32');?>","<?php echo str_replace('...','',cfCaption('sendByMail'));?>"];
</script>
<?php
}

/**
 * @desc Top right explorer-specific controls
 *
 * @return string: HTML code
 */
function mefTopControls(){
	$output='';
	// Audio quality controls
	if(cfRGetVar('fileTypeFilter') && in_array('audio',cfRGetVar('fileTypeFilter'))) $output.=mfAudioQualityControls(cfRGetVar('audioQuality'));

	if(cfRGetVar('subFoldersIncluded')){
		$output.=mfDropDownMenu(outIcon('fi/med/folderFiles'),false,array(
			array('selected'=>((cfRGetVar('view')=='concat')?1:''),'icon'=>outIcon('fi/med/folderFiles'),'caption'=>cfCaption('filesAndFolders'),'onclick'=>"setView('concat')"),
			array('selected'=>((cfRGetVar('view')=='folders')?1:''),'icon'=>outIcon('fi/med/folder'),'caption'=>cfCaption('genDirectories'),'onclick'=>"setView('folders')"),
			array('selected'=>((cfRGetVar('view')=='files')?1:''),'icon'=>outIcon('fi/med/icoBlank'),'caption'=>cfCaption('genFiles'),'onclick'=>"setView('files')")
		));

	}

/*
	// Browse files+folders (concatenate) button (do not display if no folder browsing)
	if(cfRGetVar('subFoldersIncluded')) $output.=mfButton(false,'javascript:setView(\'concat\')',outIcon('fi/med/folderFiles'),'id="setViewconcat"',(cfRGetVar('view')=='concat')?'on':'');

	// Browse folders button (do not display if no folder browsing or files)
	if(cfRGetVar('subFoldersIncluded')) $output.=mfButton(false,'javascript:setView(\'folders\')',outIcon('fi/med/folder'),'id="setViewfolders"',(cfRGetVar('view')=='folders')?'on':'');

	// Browse files button
	$output.=mfButton(false,'javascript:setView(\'files\')',outIcon('fi/med/icoBlank'),'id="setViewfiles"',(cfRGetVar('view')=='files')?'on':'');
*/
	// Tools button
	if(mefHasTools()) {
		if(is_array(cfRGetVar('fileTypeFilter')) && (in_array('audio',cfRGetVar('fileTypeFilter'))||in_array('video',cfRGetVar('fileTypeFilter'))))
			$output.=mfButton(false,'javascript:setView(\'tools\')',outIcon('fi/med/icoM3U'),'id="setViewtools"',(cfRGetVar('view')=='tools')?'on':'');
		else
			$output.='&nbsp;&nbsp;'.mfButton(false,'javascript:setView(\'tools\')',outIcon('mTools'),'id="setViewtools"',(cfRGetVar('view')=='tools')?'on':'');
	}

	return $output;
}

/**
 * @desc Return true if resource has multiple downloads and/or upload
 *
 * @return boolean
 */
function mefHasTools(){
	$filter=cfRGetVar('fileTypeFilter');
	if(cfRGetVar('multipleDownloadAllowed') || (cfBGetVar('playlistFormat') && is_array($filter) && in_array('audio',$filter))) return true;
	if(cfRGetVar('uploadAllowed')) return true;
	return false;
}

/**
 * @desc return JS array item corresponding to a file description
 *
 * @param string $id: id of item
 * @param array $value: properties of file
 * @param array $options: tnw: if set, add thumbnail url instead of icon, with tnw width & tnh height
 * @return string
 */
function mefFilePropertiesToJSArray($id,$value,$options=false){
	// JS Array format :
	//
	// 00 - id,
	// 01 - name,
	// 02 - icon/thumbnail path
	// 03 - downloadable (0 no, or 1 downloadable &  viewable or 2 viewable only)
	// 04 - mdownloadable (0 or 1)
	// 05 - size text,

	// Explorer
	// 06 - unlinkable (0 or 1),
	// 07 - renamable (0 or 1),
	// 08 - excutable (0 or 1)

	// Images
	// 06 - size

	// Audio
	// 06 - title

	static $filter=null;
	if($filter===null) $filter=cfRGetVar('fileTypeFilter');

	// ID, name
	$output='['.$id.',"'.$filename=str_replace('&nbsp;',' ',cfProtectFilename($value['name'])).'",';

	// Icon / thumbnail
	if(isset($options['tnw'])) {
		// If a thumbnail exists, base smaller thumbnail on it
		if(isset($value['thumbnail'])) {
			// Squared image (images)
			if($options['tnw']==$options['tnh'])
				$output.='"'.cfExtImage(isset($value['thumbnail'])?$value['thumbnail']:$value['completeFileName'],$options['tnw'],$options['tnh'],true,0,array('sq'=>1)).'",';
			// Regular image (video & multimedia)
			else{
				$output.='"'.cfExtImage(isset($value['thumbnail'])?$value['thumbnail']:$value['completeFileName'],$options['tnw'],$options['tnh'],true,0).'",';
			}
		}
		else $output.='"'.cfExtImage($value['completeFileName'],$options['tnw'],$options['tnh'],true,0,array('sq'=>1)).'",';
	}
	elseif($filter==array('audio')) $output.='"",'; // Hard-coded icon in JS
	else $output.='"'.$value['icon'].'",';

	// Download / view file
	if($value['rights']['download'] && cfRGetVar('downloadShortcut')) $output.='1,';
	elseif($value['rights']['download']) $output.='2,';
	else $output.='0,';
	// Multiple download
	if($value['rights']['download'] && cfRGetVar('multipleDownloadAllowed')) $output.='1,'; else $output.='0,';
	// Size
	$output.="'".addslashes(str_replace(' ','&nbsp;',efFileSizeFormat($value['size'])))."',";

	if(!$filter || in_array('video',$filter)){
		// Unlink
		if($value['rights']['modify']) $output.='1,'; else $output.='0,';
		// Rename
		if($value['rights']['modify']) $output.='1,'; else $output.='0,';
		// Execute
		if($value['rights']['execute']) $output.='1'; else $output.='0';
	}
	elseif($filter==array('image')){
		// Dimensions
		$output.='"'.((isset($value['extra1']))?$value['extra1']:'').'"';
	}
	elseif($filter==array('audio')){
		// Dimensions
		$ai=efGetAudioInfo($value['completeFileName'],false);
 		$output.='"'.str_replace('"','&quot;',cfUTF8Encode(((isset($ai['title']))?$ai['title']:$value['name']))).'"';
	}
	return $output.']';
}

/**
 * @desc Return HTML code for multiple downloads list
 *
 * @return string
 */
function mefMultipleDownloadsInnerHTML(){
	$output='';

	if(cfRGetVar('playlist')) foreach (cfRGetVar('playlist') as $k=>$v) $output.='<div class="sli">'.outImage(outIcon('audioBitrate32'),false,false,'float:left;margin-right:0.5em').cfUTF8Encode($v['label']).'</div>';
	elseif(cfRGetVar('mDownloadList')) foreach (cfRGetVar('mDownloadList') as $k=>$v) $output.='<div class="sli"><img src="'.efIcon($k).'" class="frW ico16">'.cfUTF8Encode($k).'</div>';
	return $output;
}

/**
 * @desc output script needed for sync/async update of tools view
 * @param array $options: options:
 * 			'noMAddAll': no mAddAll button (used by mobile music DB scripts)
 */
function mefDisplayTools($options=false){
	$output='';
	$filter=cfRGetVar('fileTypeFilter');

	cfDebugTag('mefToolsDebugTag1');

	// Uploads
	if(cfRGetVar('uploadAllowed') && cfBGetVar('upload')){
		$output.='<div class="frame2"><div class="frame2Header">'.cfCaption('genFileUpload').'</div>';
		$output.=mfUploadForm(true);
		$output.='</div>';
	}

	// Remote downloads
	if(cfRGetVar('remoteDownloadAllowed')){
		$output.='<div class="frame2"><div class="frame2Header">'.cfCaption('genDirectDownload').'</div>';
		$output.='<form name="ddBox" method="POST" onsubmit="return false">';
		$output.='<input class="textInput" type="text" id="rdlInput" name="rdName" style="width:80%" value="http://"> ';
		$output.='<input type="hidden" name="resId" value="'.$_SESSION['activeResourceId'].'">';
		$output.=mfButton(false,'javascript:rdl();',outIcon('ok32'),'id="rdlBt" style="float:right"');
		$output.='</form></div>';
	}

	// Multiple downloads
	if(cfRGetVar('multipleDownloadAllowed') || (cfBGetVar('playlistFormat') && is_array($filter) && in_array('audio',$filter))){
		$output.='<div class="frame2">';
		// Audio: call it playlist
		if(is_array($filter) && in_array('audio',$filter) && count($filter)==1)
			$output.='<div class="frame2Header">'.cfCaption('explorerAudioPlaylist').'</div>';
		else
			$output.='<div class="frame2Header">'.cfCaption('explorerMultipleDownload').'</div>';

		// Multiple download & download buttons
		$output.=mfButton(false,'javascript:mDownload()',outIcon('dl'))
				.((isset($options['noMAddAll']))?'':mfButton(false,'javascript:mAddAll()',outIcon('addMult')))
				.mfButton(false,'javascript:mSuppAll()',outIcon('clear'));
		// Playlist play all button
		if(cfBGetVar('playlistFormat') && is_array($filter) && in_array('audio',$filter))
			$output.=mfButton(false,'javascript:plPlayAll()',outIcon('play'));
		// File's div
		$output.='<div class="frame3" id="mdlF" style="height:80%;overflow:auto">';
		$output.=mefMultipleDownloadsInnerHTML();
		$output.='</div></div>';
	}

	// Insert generated HTML code into mainDiv
	if(cfIsAsync()) {
		echo cfAsyncXMLInnerHTMLbyId($output,'mainDiv');
		echo cfAsyncJSaction("if(dgi('foldersDiv')) dgi('foldersDiv').innerHTML=''"); // Hide folders
	}
	else echo '<div id="mainDiv">'.$output.'</div>';
	// Send javascript response
	mefSendJSResponse($output);
}

/**
 * @Desc output script needed for sync/async update of folders view
 *
 */
function mefDisplayFolders(){
	/**
	 * Nb files in directory
	 */
	$output='';
	if(cfRGetVar('fileTypeFilter')==array('image'))	$icon='icoImage';
	elseif(cfRGetVar('fileTypeFilter')==array('audio'))	$icon='icoMus';
	elseif(cfRGetVar('fileTypeFilter')==array('video'))	$icon='icoVid';
	elseif(cfRGetVar('fileTypeFilter')==array('image','audio','video'))	$icon='icoMultimedia';
	else $icon='icoBlank';
	if($_ENV['weezoResEfFiles']['nbFiles']==0) $icon='icoBlankNo';

	// Switch to files view if empty folder
	if($_ENV['weezoResEfFiles']['nbSubDirs']==0 && $_ENV['weezoResEfFiles']['nbFiles']>0 && (!isset($_POST['data3']) || $_POST['data3']!='setView') && cfRGetVar('view')!=='concat')
		return mefDisplayFiles();

	// Update script
	$filesDesc=(cfRGetVar('view')=='concat' && $_ENV['weezoResEfFiles']['nbFiles']>0)?'':cfCaption('explorerFiles',$_ENV['weezoResEfFiles']['nbFiles']);

	$output='updtFolders("'.cfProtectFilename(cfResourceRelativePath(cfRGetVar('efCurrentDirectory'))).'","'.outIcon('fi/med/'.$icon).'","'.$filesDesc.'",Array(';


	$i=0; $nb=0;

	/**
	 * Upper directory
	 */
	if((cfFileRights(cfRGetVar('efCurrentDirectory')."/..",'state')=='authorized')){
		$output.='".."';
		$nb=1;
	}

	/**
	 * Range
	 */
	if(cfRGetVar('view')=='concat'){
		$itemsPerPagePos=$itemsPerPage=0; // Concatenated view: always display all directories
	}
	else{
		$itemsPerPagePos=cfRGetVar('itemsPerPagePos');
		$itemsPerPage=cfRGetVar('itemsPerPage');
	}

	/**
	 * Subdirectories
	 */
	foreach ($_ENV['weezoResEfFiles'] as $file) if(is_array($file)){
		if($file['type']=='file') continue;
		if($i++<$itemsPerPagePos) continue;
		if($itemsPerPage && $nb>$itemsPerPage) break;

		if($nb>0) $output.=',';
		if($file['type']=='dir'){
			$output.='"'.cfUTF8Encode($file['completeFileName'],false,false).'"';
		}
		elseif($file['type']=='drive'){
			$output.='["'.cfUTF8Encode($file['completeFileName']).'","'.$file['icon'].'"]';
		}
		$nb++;
	}

	$output.='),'.(($itemsPerPagePos)?1:0).','.(($itemsPerPage && $nb>$itemsPerPage)?1:0).')';

	// Send javascript response
	mefSendJSResponse($output);
}

/**
 * @Desc output script needed for sync/async update of files view
 *
 */
function mefDisplayFiles(){
	$out='';

	if(cfRGetVar('view')!='concat') $out.='updtFolders("'.cfProtectFilename(cfResourceRelativePath(cfRGetVar('efCurrentDirectory'))).'");';

	$i=0;$nb=0;	$options=false;

 	$filter=cfRGetVar('fileTypeFilter');
	$out.='updtFiles([';

	// Compute thumbnails dimensions for video & multimedia
	if(is_array($filter)&&in_array('video',$filter)){
		// iPhone
		if(cfBGetVar('name')=='iPhone'){$w=64;$h=48;}
		// iPad
		else if(cfBGetVar('name')=='iPad'){$w=128;$h=96;}
		// Other
		else{$w=max(64,min(cfGGetVar('userScreenWidth'),cfGGetVar('userScreenHeight'))/6);$h=floor($w*3/4);}

		$options=array('tnw'=>$w,'tnh'=>$h);
		$nbx=1;
	}

	// Compute thumbnails dimensions for images
	elseif(is_array($filter)&&in_array('image',$filter)){
		$sw=max(cfGGetVar('userScreenWidth')-10,80);
		$nbx=ceil($sw/100);	$w=floor($sw/$nbx);
		if($w<70) {$nbx--;$w=floor($sw/$nbx);}

		// iPhone
		if(cfBGetVar('name')=='iPhone'){$w=100;$nbx=3;}
		// iPad
		if(cfBGetVar('name')=='iPad'){$w=100;$nbx=7;}

		// Compute margin around images
		$margin=max(1,($sw-($nbx*$w))/(2*$nbx));
$options=array('tnw'=>$w*1.5,'tnh'=>$w*1.5);
		$out='margin='.floor($margin).';'.$out;
		$h=$w;
	}
	elseif(is_array($filter)&&in_array('audio',$filter)){require_once(INCLUDE_DIR.'explorerFunctions.php');}

	// Get file's mime type (used for multimedia browser, not yet used)
	// if(cfRGetVar('mobileGetFileMimeType')) $options['mimeType']=1;

	// Number of thumbnails per page (round to above value to fill table)
	if(isset($nbx)) {
		$nbPP=$nbx*ceil(cfRGetVar('itemsPerPagePos')/$nbx);
		$iPP=$nbx*ceil(cfRGetVar('itemsPerPage')/$nbx);
	}
	else {
		$nbPP=cfRGetVar('itemsPerPagePos');
		$iPP=cfRGetVar('itemsPerPage');
	}

	// Generate array with complete files information, that will be used for page generation
	foreach ($_ENV['weezoResEfFiles'] as $file) if(is_array($file) && $file['type']=='file'){
		// Verify file is in displayed interval
		if($i++<$nbPP) continue;
		if(cfRGetVar('itemsPerPage') && $nb>=$iPP) break;

		if($nb>0) $out.=',';
		$out.=mefFilePropertiesToJSArray($i-1,$file,$options);
		$nb+=1;
	}

	$out.='],'.((cfRGetVar('itemsPerPagePos'))?1:0).','.(($iPP && $nb>=$iPP)?1:0).')';

	// Set thumbnails width/height and nb thumbnails / rows for images-only thumbnails
	if(is_array($filter)&&(in_array('video',$filter)|| in_array('image',$filter))) $out='imgw='.$w.';imgh='.$h.';imgx='.$nbx.';'.$out;

	// Send javascript
	mefSendJSResponse($out);
}

/**
 * @desc Sync or Async send output generated by mefDisplayFiles, mefDisplayFolders or mefDisplayTools
 * @param string $output
 *
 */
function mefSendJSResponse($output){
	/**
	 * Async Request: processed by efProcessPOST
	 */
	if(cfIsAsync()){
		if($output) echo cfAsyncXMLJSaction($output);
		// Update client-side playlist counter
		if($_POST['data3']=='plAddAll'||$_POST['data3']=='mAddAll'||$_POST['data3']=='plAdd'||$_POST['data3']=='plSupp'||$_POST['data3']=='mSuppAll'){
			$nbFiles=@count(cfRGetVar('playlist'));
			if($nbFiles!==false) echo cfAsyncJSaction('if(W.plUpdtNB) plUpdtNB('.$nbFiles.')');
		}
	}
	/**
	 * Sync Response
	 */
	else{
		if($output) echo '<script type="text/javascript">'.$output.'</script>';
	}
}

/**
 * @desc Insert form used to enter an email address for sending file by mail
 *
 * @return string
 */
function mefMailoutForm($cfn=false){
	$html ='<div class="frame1Header">'.outImageIcon('mailOut32').cfCaption('sendByMail').'</div>';
	$html.='<br>';

	$html.='<div class="frame2" id="mailoutOuter"><br>';
	$html.='<div class="frame2Header">'.cfCaption('genEMail').'</div>';
	$html.='<input class="textInput" type="email" id="mailoutEmail" style="width:100%" value="'.cfGGetVar('mailoutDefaultRecipient').'" onkeyup="mailoutInputValid(this.value)"><br>';
	$html.='<div class="frame2Header">'.cfCaption('genTitle').'</div>';
	$html.='<input class="textInput" type="email" id="mailoutTitle" style="width:100%" value="'.basename($cfn).'"><br>';
	$html.='<div class="frame2Header">'.cfCaption('explorerFileSendMessage').'</div>';
	$html.='<input class="textInput" type="email" id="mailoutMessage" style="width:100%;height:3em" value="">';
	$html.='<br><br><br><br></div><center><br>';
	// Validate password input button
	$html.=mfButton(cfCaption('genSend'),'javascript:mailoutProceed()',false,'id="mailoutOkBt"',false,array('extraClass'=>'wide green'));
	$html.='<br>';
	// Cancel password input button
	$html.=mfButton(cfCaption('genCancel'),'javascript:selBack()',false,'id="mailoutCancelBt"',false,array('extraClass'=>'wide red'));
	$html.='</center>';




	$script='';
	if(!cfGGetVar('mailoutDefaultRecipient')) $script.='dgi("mailoutEmail").focus();';

	//$script.='$("mailoutOuter").e.alignBottom(10);';
	$script.='W.mailoutInputValid=function(mail){';
	$script.="var r=(new RegExp('^[a-z0-9]+([_|\.|-]{1}[a-z0-9]+)*@[a-z0-9]+([_|\.|-]{1}[a-z0-9]+)*[\.]{1}[a-z]{2,6}$', 'i')).test(mail);";
	$script.="dgi('mailoutOkBt').className=((r)?'bt wide green':'btDisabled wide green')";
	$script.='};W.mailoutInputValid();';

	$html=cfAsyncXMLInnerHTMLbyId($html,'mainDiv');
	$html.=cfAsyncJSaction($script);
	return $html;
}
?>