<?php
/**
 * Transfers (uploads, remote downloads) visualisation scripts
 * Includes :
 * - transfer class
 * - transfer misc functions
 * - a page display script divided into 2 parts :
 * 	- display in a frame (with head and body stuff), self-refreshed with async requests, for inclusion by explorer pages
 * 	- display in div, for inclusion by administration / transfers page
 *
 * Images : view, slideshow, next, prev, fullSize
 * HTML : view webpage (and not source code)
 * text : inline view
 * other formats : download (Opera) or inline view if possible (IE)
 *
* parameters : POST data1 (file directory),  data2 (file name),  data3 (="view"),  data4 (action (optional)),  data5 (parameters (optional))
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

/*
 ***************************************************************************************************************************
 * Transfer Class
 ***************************************************************************************************************************
 */
class transfer{
	// Common variables
	public $id; // (unique) Id of transfer
	public $sessionId; // Session ID of user requesting transfer
	public $sessionIP; // IP adress of user who initiated the transfer
	public $type; // 'remoteDownload', 'upload', 'download', or 'bittorrentDownload'
	public $status; // Transfer status
	public $size; // Transfer total size

	/*
	 * status values :
	 * 		 created = 0
   	 *		 waitConfirm = 1
   	 *		 waitConnection = 2
   	 *		 running = 3
   	 *		 finished = 4
	 *		 paused = 5
     *		 canceled = 6
     *		 connecting = 7
     *		 failed = 8
     *		 removed = 9
     */
	public $updateInterval; // Transfer update-to-file interval, in seconds (used for download monitoring)

	// Common
	public $progress; // transfer progress (0-1)
	public $destinationFile; // Destination file (all transfers)

	// Remote download and upload specific
	public $destinationDir; // Destination directory (remoteDownload & updload)

	// Remote download & download & bittorrent download specific
	public $sourceFile;

	// Remote download specific
	public $URL;
	public $host;
	public $path;

	// bittorrent download specific
	public $userId; // (unique) id of user used to create torrent file. Used for speed limitation purposes
	public $infoHash; // Url encoded 20 bytes SHA1 hash of bencoded torrent's info dictionnary
	public $trackerIdSet; // True if "tracker id" sent to client
	public $filesOffsets; // list of files and offsets. Format : file_path:eof_offset|file_path:eof_offset|...
	public $pieces; // Number of pieces in torrent
	public $pieceLength; // Piece size, in bytes
	public $sentDataLength; // Nb of bytes sent
	public $linkedTransferToken;// id of linked directLink or publishDownload token

	// Constructor
	function __construct($transferType){
		if($transferType=='remoteDownload') $this->id="RDL";
		elseif($transferType=='upload')	$this->id="UL";
		elseif($transferType=='download') $this->id="DL";
		elseif($transferType=='multipleDownload') $this->id="MDL";
		elseif($transferType=='bittorrentDownload') $this->id="BTDL";
		else return false;

		$this->id.=rand(0,999999999);
		$this->type=$transferType;
		$this->sessionId=wSession_id();
		$this->sessionIP=$_SERVER['REMOTE_ADDR'];
		$this->status=0;
		$this->progress=0;
		if($transferType=='upload') $this->updateInterval=0.1; else $this->updateInterval=3;
		$this->startTime=time();
		$this->destinationDir=false;
		$this->destinationFile=false;
		$this->sourceFile=false;
		$this->URL=false;
		$this->host=false;
		$this->path=false;
		$this->userId=false;
		$this->infoHash=false;
		$this->pieces=false;
		$this->pieceLength=false;
		$this->trackerIdSet=false;
		$this->sentDataLength=0;
		$this->linkedTransferToken=false;
		$this->lastUpdated=time();
		return $this->id;
	}

	// Destructor
	function __destruct(){}

	// Return a formated date from a time
	function formatedDate($time){
		static $today;
		if(!isset($today)) {$today=getdate();$today=$today['year'].$today['yday'];}
		$d=getdate($time);

		if($today==$d['year'].$d['yday'])
			return date('H:i:s',$time);
		else
			return date(cfCaption('_FULL_DATE_FORMAT'),$time);
	}

	// Transfer "detail" string
	public function detail($useBR=false){
		$output='';
		$date=$this->formatedDate($this->startTime);

		if($this->type=='upload') {
			$output .= cfCaption('genFileUpload')."<br>\n";
			$output .= cfCaption('genStartedTime').cfCaption('genSeparator').$date."<br>\n";
			$output .= cfCaption('transferCfgSource').cfCaption('genSeparator').$this->sessionIP.'://?/'.cfUTF8Encode($this->sourceFile)."<br>\n";
			$output .= cfCaption('transferCfgDest').cfCaption('genSeparator').cfUTF8Encode($this->destinationDir.'/'.$this->destinationFile."<br>\n");
			$output .= cfCaption('genState').cfCaption('genSeparator').$this->captionStatus().' ('.floor(100*$this->progress).'%)'."<br>\n";
		}
		elseif($this->type=='download') {
			$output .= cfCaption('transfersDownload')."<br>\n";
			$output .= cfCaption('genStartedTime').cfCaption('genSeparator').$date."<br>\n";
			$output .= cfCaption('transferCfgSource').cfCaption('genSeparator').cfUTF8Encode($this->sourceFile."<br>\n");
			$output .= cfCaption('transferCfgDest').cfCaption('genSeparator').$this->sessionIP.'://?/'.cfUTF8Encode(basename($this->sourceFile))."<br>\n";
			$output .= cfCaption('genState').cfCaption('genSeparator').($this->captionStatus()).' ('.floor(100*$this->progress).'%)'."<br>\n";
		}
		elseif($this->type=='remoteDownload') {
			$output .= cfCaption('genDirectDownload')."<br>\n";
			$output .= cfCaption('genStartedTime').cfCaption('genSeparator').$date."<br>\n";
			$output .= cfCaption('transferCfgSource').cfCaption('genSeparator').cfUTF8Encode($this->URL)."<br>\n";
			$output .= cfCaption('transferCfgDest').cfCaption('genSeparator').cfUTF8Encode($this->destinationFile)."<br>\n";
			$output .= cfCaption('genState').cfCaption('genSeparator').$this->captionStatus().' ('.floor(100*$this->progress).'%)'."<br>\n";
		}
		elseif($this->type=='multipleDownload') {
			$output .= cfCaption('explorerMultipleDownload')."<br>\n";
			$output .= cfCaption('genStartedTime').cfCaption('genSeparator').$date."<br>\n";
			$output .= cfCaption('transferCfgSource').cfCaption('genSeparator').cfUTF8Encode($this->sourceFile)."<br>\n";
			$output .= cfCaption('transferCfgDest').cfCaption('genSeparator').cfUTF8Encode($this->destinationFile)."<br>\n";
			$output .= cfCaption('genState').cfCaption('genSeparator').$this->captionStatus().' ('.floor(100*$this->progress).'%)'."<br>\n";
		}
		elseif($this->type=='bittorrentDownload') {
			$output .= 'Bittorrent'."<br>\n";
			$output .= cfCaption('genStartedTime').cfCaption('genSeparator').$date."<br>\n";
			$output .= cfCaption('genState').cfCaption('genSeparator').$this->captionStatus().' ('.floor(100*$this->progress).'%)'."<br>\n";

			$output .= cfCaption('transferCfgSource').cfCaption('genSeparator')."<br>\n";
			foreach (explode('|',$this->sourceFile) as $file) if($file!='') $output.=' - '.cfUTF8Encode($file)."<br>\n";
			require_once(INCLUDE_DIR.'explorerFunctions.php');
			$output .= cfCaption('genSize').cfCaption('genSeparator').efFileSizeFormat($this->size);
		}
		if($useBR)	return  ($output); else return str_replace("\n",'',$output);
	}

	/**
	 * save upload in uploads.txt file or to memory
	 * @param array $transfersArray: whole array of transfers. If omitted, reload from memory or file
	 * @param boolean $commitToFile: true to force commit to file, false to save to memory only
	 */
	public function commitToFile($transfersArray=false,$forceCommitToFile=true){
		if(!$transfersArray) $transfersArray=tReadTransfersFiles();
		$transfersArray[$this->id]=$this;
		$result=tWritePHPTransfersFile($transfersArray,$forceCommitToFile);
		unset($transfersArray);
		return $result;
	}

	/**
	 * @desc Check if transfer is still in transfers array or if it has been removed
	 *
	 * @return boolean true if still in transfers array
	 */
	public function exists(){
		$transfers=tReadTransfersFiles(false);
		return isset($transfers[$this->id]);
	}

	/**
	 * @desc Return status label (non-utf8 encoded)
	 *
	 * @return string status
	 */
	public function captionStatus(){
		switch ($this->status){
			case 1: return cfCaption('transfersWaitConfirm');
			case 2: return cfCaption('transfersWaitConnection');
			case 3: return cfCaption('transfersWorking');
			case 4: return cfCaption('transfersFinished');
			case 5: return cfCaption('transfersPaused');
			case 6: return cfCaption('transfersCanceled');
			case 7: return cfCaption('transfersConnecting');
			case 8: return cfCaption('transfersFailed');
		}
	}

	// Return true if transfer display needs to be consistently refreshed (depending on status)
	public function needRefresh(){
		if($this->status==0 || $this->status==2 || $this->status==3 || $this->status==7 || $this->status==9) return true;
		return false;
	}

	// Stop transfer
	public function stop(){
		// Ask application to stop transfer
		if($this->type=='remoteDownload' || $this->type=='fileTransfer') cfServerSendCommand('transfer action="cancel" transferId="'.$this->id.'"');
		else $this->updateStatus(9);

		return true;
	}

	// Remove finished transfer from list (or even if not finished if $forceRemove)
	public function clear($forceRemove=false){
		if($forceRemove || ($this->status==4 || $this->status==6 || $this->status==8)) {
			// Ask application to remove transfer from list
			if($this->type=='remoteDownload') cfServerSendCommand('transfer action="clear" transferId="'.$this->id.'"');
			else $this->updateStatus(9);
			return true;
		}
		return false;
	}

	/**
	 * @desc Update transfer progress
	 * @param float $newProgress: new progress (0-1)
	 * @param boolean $commitToFile: set to true to save transfers to file
	 */
	public function updateProgress($newProgress, $commitToFile=false){
		if(!is_numeric($newProgress)) return false;
		$this->progress=max(0,min(1,$newProgress));
		$this->lastUpdated=time();
		if($commitToFile) $this->commitToFile();
		return $this->progress;
	}

	/**
	 * Indicate data has been sent. Used to update progress from sent size / total size
	 *
	 * @param int $extraDataSize: number of bytes sent since last call to this function
	 * @param bool $commitToFile: set to true to save transfers to file
	 */
	public function updateSentData($extraDataSize, $commitToFile=false){
		$this->sentDataLength+=$extraDataSize;
		if($this->size>0) $this->progress=max(0,min(1,$this->sentDataLength/$this->size));
		$this->lastUpdated=time();
		if($commitToFile) $this->commitToFile();
	}

	/**
	 * @desc Update transfer status
	 * @param integer $newStatus : new status
	 * status values :
	 * 		 created = 0
   	 *		 waitConfirm = 1
   	 *		 waitConnection = 2
   	 *		 running = 3
   	 *		 finished = 4
	 *		 paused = 5
     *		 canceled = 6
     *		 connecting = 7
     *		 failed = 8
     *		 removed = 9
     * @param boolean $commitToFile : set to true to save transfers to file
	 */
	public function updateStatus($newStatus, $commitToFile=false){
		if(!is_numeric($newStatus)) return false;
		if($this->status!=$newStatus) $commitToFile=true; // WA: force commit to file whenever status changes
		$this->status=$newStatus;
		// If status is set to finished, force progress to 100%
		if($this->status==4) $this->progress=1;
		$this->lastUpdated=time();
		if($commitToFile) $this->commitToFile();
	}

	/**
 * @desc display a progress bar with title and text % progress
 *
 * @param mixed $progress : progress value (0-1) or 'U' if unknown (for uploads)
 * @param integer $width : width of progressbar in px
 * @param integer $height : height of progressbar in px
 */
	public function tProgressBar($width){
		// If transfer is upload, update progress from file... when uploadprogress extension will work.
		if(false && $this->type=='upload'){
			$ul_info = uploadprogress_get_info($this->id);
			if(!$ul_info || !isset($ul_info['bytes_total']) || $ul_info['bytes_total']==0) $progress='U';
			else $progress=$ul_info['bytes_total']/$ul_info['bytes_total'];
		}
		$progress=$this->progress;
		$title=$this->captionStatus();
		if(!is_numeric($progress)) {
			$progress='U';
			//$output= '<span title="'.$title.'">';
		}
		else{
			if($progress<0) $progress=0;
			if($progress>1) $progress=1;
			//$output='<span title="'.$title.'">';
		}

		// Adjust number of decimals depending on total transfer size
		if($this->size < 1000000) $dec=1;			// < 1MB
		elseif($this->size < 10000000) $dec=10;		// < 10 MB
		else $dec=100;								// > 10 MB

		if($progress!=0 && $progress!=1) $title=(round(100*$dec*$progress)/$dec).'%';
		// Error progress bar
		if($this->status==6 || $this->status==8 || $this->status==9) return outProgressBar(-1,$width,($title),false,false);
		// Normal progress barr
		else return outProgressBar($progress,$width,$title,false,false);
	}
}

/**
 * @desc cross-session transfer speed limitation
 * 		Used to set a user (group) level download speed limit
 *
 */
class transferLimitation{
	private $userId; // Transfer user id
	private $maxTransferRate; // Maximum transfer rate, in KB/sec
	private $totalTransfered;

	/**
	 * @desc Constructor
	 *
	 * @param integer $speedLimit : speed limit, bytes per second
	 */
	function __construct($speedLimit=-1){
		$totalTransfered=0;
		// User Id
		if(isset($_SESSION['user']['id'])) $this->userId=$_SESSION['user']['id'];
		else $this->userId=$_SERVER['REMOTE_ADDR'];

		// Max transfer rate
		if($speedLimit!=-1 && is_numeric($speedLimit)) $this->maxTransferRate=$speedLimit;
		else{
			$this->maxTransferRate=0;
			if(isset($_SESSION['user']['downloadSpeedLimit']) && is_numeric($_SESSION['user']['downloadSpeedLimit']) && $_SESSION['user']['downloadSpeedLimit']!=0) $this->maxTransferRate=$_SESSION['user']['downloadSpeedLimit'];
		}
		if($this->maxTransferRate) $this->tTLCreate($this->userId, $this->maxTransferRate);
	}


	// Destructor
	function __destruct(){$this->tTLDestroy($this->userId);}

	/**
	 * @desc Update speed limit
	 *
	 * @param unknown_type $speedLimit
	 */
	function setSpeedLimit($speedLimit){
		if($this->maxTransferRate==$speedLimit) return;
		$this->maxTransferRate=$speedLimit;
		$this->tTLUpdate($this->userId, $speedLimit);
	}

	/**
	 * @desc Indicate new data has been transfered, and wait until average transfer rate is below max transfer rate
	 * @param integer $nbBytes : number of bytes sent
	 * @param boolean $isTotal : true if $nbBytes is total sent, false if $nbBytes is new data sent
	 * @param boolean $blocking : true if this function should include tempo
	 */
	public function addData($nbBytes, $isTotal=false, $blocking=true) {
		if(!$this->maxTransferRate) return;
		if($isTotal){
			$this->tTLAddData($this->userId,$nbBytes - $this->totalTransfered);
			$this->totalTransfered=$nbBytes;
		}
		else{
			$this->tTLAddData($this->userId,$nbBytes);
			$this->totalTransfered+=$nbBytes;
		}
		// Wait until average transfer rate is below max transfer rate
		if($blocking) {while (!$this->tTLIsReady($this->userId)) usleep(300000);}
	}
	/**
	 * @desc Return true if data can be sent, false if over limit
	 *
	 * @return boolean
	 */
	public function isReady(){return $this->tTLIsReady($this->userId);}
	public function debugView(){
		customTest($this->userId);
	}

	/**
	 * @desc Create a Transfer Limitation object
	 *
	 * @param string $userId: id of group
	 * @param integer $maxTransferRate
	 * @return bool
	 */
	private function tTLCreate($userId,$maxTransferRate){
		// Legacy Windows C function, faster, works
		if(function_exists('customTLCreate')) return customTLCreate($userId,time(),$maxTransferRate);

		// Replacement: use memory functions to store TL object
		$tl=cfMGetVar('TL.'.$userId);
		if(!$tl) $tl=array('id'=>$userId,'nb'=>0);
		$tl['time']=time();
		$tl['rate']=$maxTransferRate;
		$tl['total']=0;
		$tl['nb']++;
		cfMSetVar('TL'.$userId,$tl);
		return true;
	}

	/**
	 * @desc Update values for a Transfer Limitation object
	 *
	 * @param string $userId: id of group
	 * @param integer $maxTransferRate
	 */
	private function tTLUpdate($userId,$maxTransferRate){
		// Legacy Windows C function, faster, works
		if(function_exists('customTLUpdate')) return customTLUpdate($userId,time(),$maxTransferRate);

		// Replacement: use memory functions to store TL object
		$tl=cfMGetVar('TL.'.$userId);
		if(!$tl) return;
		$tl['time']=time();
		$tl['rate']=$maxTransferRate;
		$tl['total']=0;
		cfMSetVar('TL'.$userId,$tl);
	}

	/**
	 * @desc Indicate a transfer has sent / received data
	 *
	 * @param string $userId: id of group
	 * @param integer $bytes: volume of sent/received data
	 */
	private function tTLAddData($userId,$bytes){
		// Legacy Windows C function, faster, works
		if(function_exists('customTLAddData')) return customTLAddData($userId,time(),$bytes);

		// Replacement: use memory functions to store TL object
		$tl=cfMGetVar('TL.'.$userId);
		if(!$tl) return;

		$tl['total']+=$bytes;
		cfMSetVar('TL'.$userId,$tl);
	}


	/**
	 * @desc Update values for a Transfer Limitation object
	 *
	 * @param string $userId: id of group
	 * @return bool true if transfer is ready to send/receive more data
	 */
	private function tTLIsReady($userId){
		// Legacy Windows C function, faster, works
		if(function_exists('customTLIsReady')) return customTLIsReady($userId,time());

		// Replacement: use memory functions to store TL object
		$tl=cfMGetVar('TL.'.$userId);
		if(!$tl) return true;
		if(($tl['total'] / (max(1,time() - $tl['time']))) > $tl['rate']) return false; else return true;
	}

	/**
	 * @desc Remove a Transfer Limitation object
	 *
	 * @param string $userId: id of group
	 * @return bool true if transfer is ready to send/receive more data
	 */
	private function tTLDestroy($userId){
		// Legacy Windows C function, faster, works
		if(function_exists('customTLDestroy')) return customTLDestroy($userId,time());

		// Replacement: use memory functions to store TL object
		$tl=cfMGetVar('TL.'.$userId);
		if(!$tl) return;
		$tl['nb']--;
		if($tl['nb']<=0) cfMUnsetVar('TL.'.$userId);
		$tl['total']=0;
		$tl['time']=time();
		cfMSetVar('TL'.$userId,$tl);
	}


}

/**
 * @desc Count transfers and update active transfers count in UI if needed
 *
 */
function tUpdateActiveTransfersCount(&$transfersArray){
	$nb=(int)cfMGetVar('activeTransfersNb');
	$count=0;
	foreach ($transfersArray as $key=>$value) if($value->status==3) $count++;
	if($count!=$nb){
		cfMSetVar('activeTransfersNb',$count);
		cfServerSendCmd('updateActiveTransfersNb',array('activeTransfersNb'=>$count));
	}
}

/**
 * @desc write uploads and downloads into transfer file (data dir /uploads.txt)
 				overwrite all transfers with $transfersArray
 *
 * @param array $transfersArray: array of transfers to write
 * @param mixed $commitToFile: set to true to save to disk & memory, false to save to memory only, 'time' to commit only if last commit is old enough
 * @return string : time.microtime of save
 */
function tWritePHPTransfersFile($transfersArray, $commitToFile=true){

	// Clear finished transfers and detect aborted downloads
	foreach ($transfersArray as $key=>$value){
		if($value->status==9) unset($transfersArray[$key]);
		// Detect aborted downloads
		if($value->type=='download' && $value->status==3 && (time()-$value->lastUpdated)>(2*$value->updateInterval)) {
			$transfersArray[$key]->status=6;
		}
	}

	// Transfer saved time
	$time=time().'.'.microtime();

	// Save to memory
	cfMSetVar('weezoTransfers',$transfersArray);

	$lastSave=(int)cfMGetVar('weezoTransfersLastUpdate');
	cfMSetVar('weezoTransfersLastUpdate',$time);

	// Update number of running transfers into UI
	tUpdateActiveTransfersCount($transfersArray);

	// Check if upload.txt file must be updated
	if($commitToFile===false || ($commitToFile=='time' && time()-$lastSave<cfHGetVar('transfersFileUpdateInterval',3))) return $time;

	if(!$handle=fopen(cfAppDataDir().'/uploads.txt','w')) return false;

	// Write uploads, downloads and bittorrent downloads (if status is not 'cleared' (9))
	foreach ($transfersArray as $value){
		if(($value->type=='upload' || $value->type=='download' || $value->type=='bittorrentDownload' || $value->type=='multipleDownload') && $value->status!=9){
			fwrite($handle, '['.$value->id."]\n");
			fwrite($handle, 'type='.$value->type."\n");
			fwrite($handle, 'sessionId='.$value->sessionId."\n");
			fwrite($handle, 'sessionIP='.$value->sessionIP."\n");

			// Detect aborted downloads and multiple downloads
			if(($value->type=='download' ||$value->type=='multipleDownload') && $value->status==3 && (time()-$value->lastUpdated)>(2*$value->updateInterval)) $value->status=6;

			fwrite($handle, 'status='.$value->status."\n");
			fwrite($handle, 'progress='.$value->progress."\n");
			fwrite($handle, 'size='.$value->size."\n");
			fwrite($handle, 'startTime='.$value->startTime."\n");
			fwrite($handle, 'lastUpdated='.$value->lastUpdated."\n");
			if($value->sourceFile) fwrite($handle, 'sourceFile='.$value->sourceFile."\n");
			fwrite($handle, 'destinationFile='.$value->destinationFile."\n");
			if($value->type=='upload') fwrite($handle, 'destinationDir='.$value->destinationDir."\n");

			// Bittorrent download specific
			if($value->infoHash) fwrite($handle, 'infoHash='.$value->infoHash."\n");
			if($value->trackerIdSet) fwrite($handle, 'trackerIdSet=true'."\n");
			if($value->userId) fwrite($handle, 'userId='.$value->userId."\n");
			if($value->filesOffsets) fwrite($handle, 'filesOffsets='.$value->filesOffsets."\n");
			if($value->pieces) fwrite($handle, 'pieces='.$value->pieces."\n");
			if($value->pieceLength) fwrite($handle, 'pieceLength='.$value->pieceLength."\n");
			if($value->linkedTransferToken) fwrite($handle, 'linkedTransferToken='.$value->linkedTransferToken."\n");
			if($value->sentDataLength) fwrite($handle, 'sentDataLength='.$value->sentDataLength."\n");

			fwrite($handle,"\n");
		}
	}
	fclose($handle);

	return $time;
}

/**
 * @desc parse transfers file and return an array of transfers
 * @param boolean $forceReadFromFile : set to true to ignore memory cache
 * @return array : read transfers
 */
function tReadTransfersFiles($forceReadFromFile=false){
	$transfersArray=array();

	// If already loaded, read from memory
	if(cfMIssetVar('weezoTransfers') && !$forceReadFromFile){
		$parsedFile2=cfMGetVar('weezoTransfers');
		foreach ($parsedFile2 as $id=>$value) if($value && $value->type=='remoteDownload') unset($parsedFile2[$id]);
	}
	// Else read uploads and downloads file
	else {if(!$parsedFile2=cfParse_ini_file(cfAppDataDir().'/uploads.txt',true)) $parsedFile2=array();}

	// Read remote downloads file
	if(!$parsedFile=cfParse_ini_file(cfAppDataDir().'/transfers.txt',true)) $parsedFile=array();

	// Initialize last update if not set
	if(!cfMIssetVar('weezoTransfersLastUpdate')) cfMSetVar('weezoTransfersLastUpdate',time().'.'.microtime());


	// Browse merged files
	foreach (array_merge($parsedFile, $parsedFile2) as $id=>$value){
		if(!is_array($value)) $transfersArray[$id]=$value;
		elseif(isset($value['type'])){
			$transfersArray[$id]=new transfer($value['type']);
			$transfersArray[$id]->id=$id;
			$transfersArray[$id]->type=$value['type'];
			if(isset($value['sessionId'])) $transfersArray[$id]->sessionId=$value['sessionId'];
			if(isset($value['sessionIP'])) $transfersArray[$id]->sessionIP=$value['sessionIP'];
			if(isset($value['status'])) $transfersArray[$id]->status=$value['status'];
			if(isset($value['progress'])) $transfersArray[$id]->progress=str_replace(',','.',$value['progress']);
			if(isset($value['lastUpdated'])) $transfersArray[$id]->lastUpdated=$value['lastUpdated'];
			if(isset($value['startTime'])) $transfersArray[$id]->startTime=$value['startTime'];
			if(isset($value['infoHash'])) $transfersArray[$id]->infoHash=$value['infoHash'];
			if(isset($value['filesOffsets'])) $transfersArray[$id]->filesOffsets=$value['filesOffsets'];
			if(isset($value['pieces'])) $transfersArray[$id]->pieces=$value['pieces'];
			if(isset($value['pieceLength'])) $transfersArray[$id]->pieceLength=$value['pieceLength'];
			if(isset($value['sentDataLength'])) $transfersArray[$id]->sentDataLength=$value['sentDataLength'];
			if(isset($value['linkedTransferToken'])) $transfersArray[$id]->linkedTransferToken=$value['linkedTransferToken'];
			if(isset($value['size'])) $transfersArray[$id]->size=$value['size'];
			if(isset($value['trackerIdSet'])) $transfersArray[$id]->trackerIdSet=$value['trackerIdSet'];
			if(isset($value['userId'])) $transfersArray[$id]->userId=$value['userId'];
			// Detect aborted downloads
			if($transfersArray[$id]->type=='download' && $transfersArray[$id]->status==3 && (time()-$transfersArray[$id]->lastUpdated)>(2*$transfersArray[$id]->updateInterval)) {
				$transfersArray[$id]->status=6;
				$transfersArray[$id]->progress=0;
			}
			if(isset($value['destinationDir'])) $transfersArray[$id]->destinationDir=str_replace('\\','/',$value['destinationDir']);
			if(isset($value['destinationFile'])) {$transfersArray[$id]->destinationFile=str_replace('\\','/',$value['destinationFile']);$transfersArray[$id]->sourceFile=basename($value['destinationFile']); }
			if(isset($value['sourceFile'])) $transfersArray[$id]->sourceFile=$value['sourceFile'];
			if(isset($value['url'])) $transfersArray[$id]->URL=$value['url'];
			if(isset($value['host'])) $transfersArray[$id]->host=$value['host'];
			if(isset($value['path'])) $transfersArray[$id]->path=$value['path'];

			// Validate loaded data
			if(!isset($value['sessionId']) || !isset($value['sessionIP']) || !isset($value['status']) || !isset($value['progress'])) unset($transfersArray[$id]);
			switch ($value['type']){
				case 'upload':
					if(!isset($value['destinationFile']) || !isset($value['sourceFile']) || !isset($value['destinationDir'])) unset($transfersArray[$id]);
					break;
				case 'download':
					if(!isset($value['destinationFile']) || !isset($value['sourceFile']) || !isset($value['lastUpdated'])) unset($transfersArray[$id]);
					break;
				case 'multipleDownload':
					if(!isset($value['destinationFile']) || !isset($value['sourceFile']) || !isset($value['lastUpdated'])) unset($transfersArray[$id]);
					break;
				case 'remoteDownload':
					if(!isset($value['destinationFile']) || !isset($value['destinationDir']) || !isset($value['host']) || !isset($value['url']) || !isset($value['path'])) unset($transfersArray[$id]);
					break;
				case 'bittorrentDownload':
					if(!isset($value['sourceFile']) || !isset($value['infoHash']) || !isset($value['filesOffsets']) || !isset($value['pieces']) || !isset($value['pieceLength'])) unset($transfersArray[$id]);
					break;
				default:
					unset($transfersArray[$id]);
			}
		}
	}
	unset($parsedFile); unset($parsedFile2);

	// Save loaded transfers into memory
	cfMSetVar('weezoTransfers',$transfersArray);
	return $transfersArray;
}

/**
 * @desc Inform that a new file upload is about to start
 		generate an ID for this upload
 		add it to transfers list
 *
 * @param string $directory : directory where file should be uploaded
 * @param string $fileName : uploaded file name
 * @return transfer : created transfer
 */
function tAddUpload($directory, $fileName){
	//if(!cfFileRights($directory=cfCleanPathName($directory),'write')) return false;
	$tr=new transfer('upload');
	$tr->sourceFile=$fileName;
	$tr->destinationDir=$directory;
	$tr->destinationFile=$fileName;
	$tr->status=3;
	$tr->progress='U';
	$tr->commitToFile();
	//register_shutdown_function('tConnectionInterrupted',$tr->id);
	return $tr;
}

/**
 * @desc Inform that a new file download is starting
 		generate an ID for this download
 		add it to transfers list
 *
 * @param string $fileName : downloaded file name
 * @return transfer : created transfer
 */
function tAddDownload($fileName){
	$tr=new transfer('download');
	$tr->sourceFile=$fileName;
	$tr->destinationFile=basename($fileName);
	$tr->status=3;
	$tr->progress=0;
	$tr->size=cfFileSize($fileName);
	$tr->commitToFile();
	register_shutdown_function('tConnectionInterrupted',$tr->id);
	return $tr;
}

/**
 * @desc Inform that a new file upload is about to start
 		generate an ID for this upload
 		add it to transfers list
 *
 * @param string $directory : directory where file should be uploaded
 * @param string $fileName : uploaded file name
 * @return transfer : created transfer
 */
function tAddRemoteDownload($dir, $url){
	//if(!cfFileRights($directory=cfCleanPathName($directory),'write')) return false;

	// Check passed URL
	$parsed=@parse_url($url);
	if(!(isset($parsed['host'])) || !isset($parsed['path'])) return false;

	// Create transfer object
	$tr=new transfer('remoteDownload');
	$tr->sourceFile=basename($parsed['path']);
	$tr->destinationDir=cfCleanPathName($dir);
	$tr->destinationFile=rawurldecode(basename($parsed['path']));
	$tr->URL=$url;
	$tr->host=$parsed['host'];
	$tr->path=$parsed['path'];
	$tr->status=2;
	$tr->lastUpdated=time();
	$tr->progress=0;
	//$tr->commitToFile();

	// Generate command string
	$command='transfer transferId="'.$tr->id.'" type="remoteDownload" action="start" destinationDir="'.$dir.'" url="'.$url.'" host="'.$parsed['host'].'" path="'.$parsed['path'].'" startTime="'.$tr->startTime.'" lastUpdated="'.$tr->lastUpdated.'" destinationFile="'.$tr->destinationFile.'"';
	if(isset($parsed['user'])) $command=$command.' user="'.$parsed['user'].'"';
	if(isset($parsed['pass'])) $command=$command.' pass="'.$parsed['pass'].'"';
	if(isset($parsed['query'])) $command=$command.' query="'.$parsed['query'].'"';

	// Send command to application to generate and start transfer
	if(!cfServerSendCommand($command)) return false;
	return $tr;
}

/**
 * @desc Create new bittorrent download
 *
 * @param array $files : array of files and folders to include in torrent
 * @param string $metainfo: binary content of .torrent file
 * @param array $linkedTransferToken: id of linked directLink or publishDownload token
 * @param array $options: jsSyncMonitorFunction=>javascript function name: async update progress passing progress percentage (0-100)
 * @return transfer: created transfer, false if failure
 */
function tAddBittorrentDownload($files, &$metainfo, $linkedTransferToken=false,$options=array()){

	// Create File_Bittorrent_MakeTorrent object

	$version=2; // fileBittorrent library version
	if ($version==1){
		require_once(INCLUDE_DIR.'fileBittorrent/MakeTorrent.php');
		$MakeTorrent = new File_Bittorrent_MakeTorrent($files);
	}
	elseif($version==2){
		require_once(INCLUDE_DIR.'File/Bittorrent2/MakeTorrent.php');
		$MakeTorrent = new File_Bittorrent2_MakeTorrent($files);
	}

	// Set the announce URL

	// Registered users : use weezo.net as tracker
	if($regInfo=cfRegistrationData())
		$trackerURL='http://weezo.net:80/tracker.php?userName='.$regInfo['regName'].'&lng='.cfGGetVar('language');
	// Else, use server as tracker
	else
		$trackerURL=cfGGetVar('hostName').'/bittorrentTracker.php?lng='.cfGGetVar('language');

	$trackerURL.='&bittorrentPort='.cfGGetVar('bittorrentPort');

	$MakeTorrent->setAnnounce($trackerURL);

	// Set the comment
	$MakeTorrent->setComment('Generated by Weezo on '.date(cfCaption('_FULL_DATE_FORMAT')));

	// Set the piece length (in KB)
	$MakeTorrent->setPieceLength(1024);

	// Set the piece length (in KB)
	//$MakeTorrent->setIsPrivate();

	// Set Javascript monitoring function
	if(isset($options['jsSyncMonitorFunction'])) $MakeTorrent->setJsSyncMonitorFunction($options['jsSyncMonitorFunction']);

	// Build the torrent
	$metainfo = $MakeTorrent->buildTorrent($files);

	if(!$metainfo) return false;

	// Create transfer
	$tr=new transfer('bittorrentDownload');
	$tr->destinationFile=$MakeTorrent->torrentName;
	$tr->infoHash=$MakeTorrent->infoHash;

	// Generate source files list
	$str=''; foreach ($files as $file) $str.=$file.'|';
	$tr->sourceFile=$str;

	// Generate files/offset data
	$str=''; foreach ($MakeTorrent->files as $file) $str.=$file['name'].'?'.$file['offset'].'|';

	$tr->filesOffsets=$str;
	$tr->pieces=(strlen($MakeTorrent->pieces)/20);
	$tr->pieceLength=$MakeTorrent->piece_length;
	$tr->size=$MakeTorrent->dataLength;
	$tr->status=2;
	$tr->progress=0;
	$tr->sentDataLength=0;
	$tr->userId=cfUGetVar('id');
	$tr->linkedTransferToken=$linkedTransferToken;
	$tr->commitToFile();
	return $tr;
}

/**
 * @desc Inform that a new multiple (zip) download is starting
 		generate an ID for this download
 		add it to transfers list
 *
 * @param string $outputFilename : zip file name
 * @return transfer : created transfer
 */
function tAddMultipleDownload($outputFilename){
	$tr=new transfer('multipleDownload');

	// Generate source files list
	$str='';
	$sfl=cfRGetVar('mDownloadList');
	if(is_array($sfl) && @count($sfl)) foreach (array_values($sfl) as $file) $str.=$file.'|';
	$tr->sourceFile=$str;

	// Get total unziped size
	if(cfHGetVar('mDownloadPrecomputeSize')) list($nbFiles,$tr->size)=cfRGetVar('mDownloadTotals'); else $tr->size=0;

	$tr->destinationFile=($outputFilename);
	$tr->status=3;
	$tr->progress=0;
	$tr->commitToFile();

	register_shutdown_function('tConnectionInterrupted',$tr->id);

	return $tr;
}

/**
 * @desc shutdown function called on user connection end, used to set a transfer (upload or download) as stopped if running
 *
 * @param string $tid
 */
function tConnectionInterrupted($tid){
	$transfers=tReadTransfersFiles(false);
	if(isset($transfers[$tid]) && $transfers[$tid]->status==3) $transfers[$tid]->updateStatus(8,true);
}

/**
 * @desc Initialize transfers on a fresh server start: all running transfers are set as stopped
 *
 */
function tInitialize(){
	$updated=0;
	if(0) $transfer=new transfer(false); // Zend IDE WA

	$transfers=tReadTransfersFiles(true);

	foreach ($transfers as $id=>$transfer){
		// If transfer is a running download or upload, set as failed
		if($transfer->status==3 && ($transfer->type=='download' || $transfer->type=='upload')) {
			$transfer->updateStatus(8);
			$updated++;
		}
	}
	if($updated) tWritePHPTransfersFile($transfers,true);
}

/*
 ***************************************************************************************************************************
 * Transfer pages
 ***************************************************************************************************************************
 */


/**
 * @desc display transfers page
 *
 * @param boolean $standalone : true if page should include html head and body
 * @param string $bodyClass optional : class of body
 * @param boolean $autoRefresh: set to true to embed javascript that will periodically refresh transfers, false to let external js do the stuff (activity tab)
 * @param boolean $forceAsyncUpdate: set to true to force display even if transfers have not been updated since last display (used to update display when displayOnlyActiveTransfersTime is modified)
 */
function tDisplayTransfers($standalone, $bodyClass=false, $autoRefresh=true, $forceAsyncUpdate=false){
	if(!isset($_ENV['configurationEnvironment'])) $_ENV['configurationEnvironment']='browser';
	/*
	 ***************************************************************************************************************************
	 * Process POST Commands
	 ***************************************************************************************************************************
	 */
	$tId=cfUTF8Decode(@$_POST['data2']);
	if(isset($_POST['transfersRefresh'])) $action='transfersRefresh';
	else if(isset($_POST['transfersClear'])) {$action='transfersClear';$tId=cfUTF8Decode($_POST['transfersClear']);}
	else if(isset($_POST['transfersStop'])) {$action='transfersStop';$tId=cfUTF8Decode($_POST['transfersStop']);}
	else if(isset($_POST['data3'])) $action=cfUTF8Decode($_POST['data3']);
	else $action='';

	// Async update: 1st check if anything has changed since last display
	if(cfIsAsync() && $action!='transfersClear' && $action!='transfersStop'){
		$lastDisplay=(cfIsResource())?cfRGetVar('transfersLastDisplayTime'):cfGGetVar('transfersLastDisplayTime');
		if(!$forceAsyncUpdate && cfMGetVar('weezoTransfersLastUpdate')<=$lastDisplay) {
			if(cfIsInApp()) return 0;
			cfAsyncHeader();
			echo cfAsyncJSaction('transfersRefresh()');
			die(cfAsyncFooter());
		}
	}

	require_once(INCLUDE_DIR.'outputFunctions.php');
	// read transfers from files
	$transfersArray=tReadTransfersFiles();

	//if($action && $action!=='transfersRefresh') cfDbg($action.' => '.$tId);


	// Global commands (clear or kill all transfers)
	if($tId==='all' && ($action==='transfersClear' || $action==='transfersStop')) {
		foreach ($transfersArray as $key=>$value){
			if(cfUGetVar('administrator') || $value->sessionId==wSession_id() || cfIsInApp()) $value->clear($action=='transfersStop');
		}
		tWritePHPTransfersFile($transfersArray);
		if(cfIsAsync()){
			if($autoRefresh){
				if($standalone) cfAsyncHeader();
				echo cfAsyncXMLJSaction('if(dgi("transfersIFrame")) setTimeout(\'frames["transfersIFrame"].transfersRefresh(1)\',100)');
				if($standalone) die(cfAsyncFooter());
			}
			else return 0; // UI:
		}
	}
	// Check if transfer exists and user is allowed to interact
	elseif($action && $tId && isset($transfersArray[$tId]) && (cfUGetVar('administrator') || $transfersArray[$tId]->sessionId==wSession_id() || cfIsInApp())){
		$transfer=$transfersArray[$tId];
		// STOP (unfinished) transfer
		if($action=='transfersStop') {$transfer->stop(); $transfer->commitToFile();}
		elseif($action=='transfersClear') {$transfer->clear(); $transfer->commitToFile();}
		// If transfer is not an upload, let 1 sec to server to remove transfer from file
		if($transfer->type=='remoteDownload') sleep(1);
		// Re-read transfers file
		unset($transfersArray); $transfersArray=tReadTransfersFiles();
	}

	$displayOnlyActiveTransfersTime=cfGGetVar('displayOnlyActiveTransfersTime');

	/*
	 ***************************************************************************************************************************
	 * IFRAME page
	 ***************************************************************************************************************************
	 */
	if($standalone){
	/*
	 * HEAD
	 */

		// Normal (non-async) page head
		if(!cfIsAsync()){
			$_ENV['winNoBar']=true;
			cfInsertHEAD(false);

			// Javascript stuff...
?>
<script language="javascript" type="text/javascript">
var tIntLength=2000, transfersRefreshTO=0;

function transfersRefresh(immediate){
	if(transfersRefreshTO) clearTimeout(transfersRefreshTO);
	transfersRefreshTO=setTimeout('wl.asr.send({"transfersRefresh":1});transfersRefreshTO=0',(immediate)?1:tIntLength);
}

function updateInterval(newInterval){
	tIntLength=newInterval;
	transfersRefresh();
}
function transfersStop(id){fAS(id,'transfersStop');}
function transfersClear(id){fAS(id,'transfersClear');}
function jsConv(str){return str.replace(/<br>/gi,String.fromCharCode(10));}
</script>
</head>
<?php
			// Body
			if($bodyClass) echo '<body class="'.$bodyClass.'" style="margin:0px;border:0px;padding:0px;overflow-x:hidden">'; else echo '<body>';
			// Inser standard com form
			outInsertStandardComForm($_SERVER['PHP_SELF'],false,false,false,false,false,true);
			echo '<div id="content" style="height:95%"'.outEHoverFrameAttr().'>';
		}

		// Async response header
		else cfAsyncHeader();


	/*
	 * BODY
	 */
		// Display transfers
		$displayedTransfers=0; $needRefresh=false;$output='';

		if(is_array($transfersArray)){
			$output.= outTableTransparent(false,'table-layout:fixed');
			$output.= '<colgroup><col width="40"><col width="20"><col width="180"><col width="*"><col width="20"></colgroup>';
			// Display transfer if it belongs to user or if user is administrator
			foreach ($transfersArray as $id=>$transfer) if($transfer->status!=9){
				// If one should display only running tranfers and this one is not, don't show
				//if($displayOnlyActiveTransfersTime && $transfer->status>3 && (time()-$transfer->lastUpdated>$displayOnlyActiveTransfersTime)) continue;

				// If user is not admin and transfer belongs to another user, don't show
				if(!cfUGetVar('administrator') && $transfer->sessionId!==wSession_id()) continue;

				$output.='<tr id="tr'.$displayedTransfers.'" class="eTC" eH=1 style="padding-left:0.5em;cursor:default">'."\n";
				// Transfer Icon
				if($transfer->type=='upload') $output.='<td>'.outImage(outIcon('upload'),false,'title="'.cfCaption('genFileUpload').'"').'</td>';
				elseif($transfer->type=='remoteDownload') $output.= '<td>'.outImage(outIcon('ddl'),false,'title="'.cfCaption('genDirectDownload').'"').'</td>';
				elseif($transfer->type=='download') $output.= '<td>'.outImage(outIcon('tdl'),false,'title="'.cfCaption('transfersDownload').'"').'</td>';
				elseif($transfer->type=='multipleDownload') $output.= '<td>'.outImage(outIcon('tmdl'),false,'title="'.cfCaption('explorerMultipleDownload').'"').'</td>';
				elseif($transfer->type=='bittorrentDownload') $output.= '<td>'.outImage(outIcon('tbtdl'),false,'title="'.cfCaption('bittorrent').'"').'</td>';
				// Detail button
				$output.= '<td>'.outButtonSmall('', 'javascript:alert(jsConv(\''.addslashes($transfer->detail(false)).'\'))', outIcon('info'), cfCaption('genDetails')).'</td>';
				// Progress bar
				$output.= '<td>'.$transfer->tProgressBar('180px').'</td>';
				// File name
				$output.= '<td style="width:100%;padding-left:0.5em; position:relative;;overflow:hidden;white-space:nowrap;text-overflow:ellipsis">'.cfUTF8Encode(basename($transfer->destinationFile)).'</td>';
				// Clear finished transfers
				if($transfer->status==4 || $transfer->status==6 || $transfer->status==8) $output.= '<td>'.outButtonSmall('','javascript:transfersClear(\''.$transfer->id.'\')',outIcon('eraser'),cfCaption('transferRemove')).'</td>';
				// Stop transfer
				elseif($transfer->progress!=1 || $transfer->type=='bittorrentDownload') $output.= '<td>'.outButtonSmall('','javascript:transfersStop(\''.$transfer->id.'\')',outIcon('cancel'),cfCaption('genCancel')).'</td>';
				else $output.= '<td></td>';

				$output.= "</tr>\n";
				if($transfer->needRefresh()) $needRefresh=true;
				$displayedTransfers++;
			}
			$output.="</table>\n";
		}
		// If no transfer, display "no transfer" message
		if(!$displayedTransfers){
				$output=''; // reset output
				$output.= outTableTransparent($bodyClass,false, 'height="95%" width="100%", id="noTransferTable"');
				$output.= '<tr><td height="100%" valign="middle" align="center">'.cfCaption('activityNoTransfer')."</td></tr></table>\n";
		}

		// Flush page to output (XML response or HTML page)
		cfAsyncOutputFlush($output,cfIsAsync(),false,'<innerHTML nodeId="content">',false);

		// if at least one tranfer is active, indicate that page need to be (async) reloaded in a few seconds, else set to transferIdleRefreshRate
		if($needRefresh) $interval=max(cfHGetVar('transferRefreshRate',2.5)*1000,1000); else $interval=cfHGetVar('transferIdleRefreshRate',10)*1000;
		if(cfIsAsync())
			echo cfAsyncXMLJSaction('updateInterval('.$interval.')');
		else
			echo '<script language="javascript" type="text/javascript">updateInterval('.$interval.");</script>\n";
	/*
	 * FOOTER
	 */
		if(!cfIsAsync()) echo '</div></body></html>';
	}



	/*
	 ***************************************************************************************************************************
	 * UI Activity page
	 ***************************************************************************************************************************
	 */
	else{
		if(!cfIsAsync()){
			// Javascript stuff...
?>
<script language="javascript" type="text/javascript">
var transfersRefreshTO=0;
function transfersRefresh(immediate){
	if(transfersRefreshTO) clearTimeout(transfersRefreshTO);
	transfersRefreshTO=setTimeout('wl.asr.send({"transfersRefresh":1});transfersRefreshTO=0',(immediate)?1:2000);
}
function transfersClear(id){wl.asr.send({'transfersClear':id})}
function transfersStop(id){wl.asr.send({'transfersStop':id})}
function transfersJsConv(str){return str.replace(/<br>/gi,String.fromCharCode(10));}
<?php if($autoRefresh) echo 'transfersRefresh();';?>
</script>
		<?php
			// Insert standard com form
			outInsertStandardComForm($_SERVER['PHP_SELF']);
		}
		else cfAsyncHeader();

		// Display transfers
		$displayedTransfers=0; $needRefresh=false;$output='';
		if(is_array($transfersArray)){
			$output.=outTableTransparent((cfIsInApp())?'frame1':'',false,'id="transfersTable"');

			//$output.= '<colgroup><col width="50"><col width="80"><col width="90"><col width="50"><col width="*"><col width="45"><col width="45"></colgroup>';
			$output.= '<colgroup><col width="45"><col width="100"><col width="210"><col width="*"><col width="28"><col width="22"></colgroup>';
			// Display transfer if it belongs to user or if user is	 administrator	(note : only administrator can access this script)
			foreach ($transfersArray as $id=>$transfer) if($transfer->status!=9){
				// If one should display only running tranfers and this one is not, don't show
				if($displayOnlyActiveTransfersTime && $transfer->status>3 && (time()-$transfer->lastUpdated>$displayOnlyActiveTransfersTime)) continue;

				// If user is not admin and transfer belongs to another user, don't show
				if(!cfUGetVar('administrator') && !cfIsInApp() && $transfer->sessionId!==wSession_id()) continue;

				if(0){
					$output.='<tr id="tr'.$displayedTransfers.'" class="eTC" eH=1 style="padding-left:0.5em;cursor:default" align="left">'."\n";
					// Transfer Icon
					$output.='<td class="eTCLeft">';
					if($transfer->type=='upload') $output.=outImage(outIcon('upload'),false,'title="'.cfCaption('genFileUpload').'"').'</td>';
					elseif($transfer->type=='remoteDownload') $output.=outImage(outIcon('ddl'),false,'title="'.cfCaption('genDirectDownload').'"').'</td>';
					elseif($transfer->type=='download') $output.=outImage(outIcon('tdl'),false,'title="'.cfCaption('transfersDownload').'"').'</td>';
					elseif($transfer->type=='multipleDownload') $output.=outImage(outIcon('tmdl'),false,'title="'.cfCaption('explorerMultipleDownload').'"').'</td>';
					elseif($transfer->type=='bittorrentDownload') $output.=outImage(outIcon('tbtdl'),false,'title="'.cfCaption('bittorrent').'"').'</td>';
					// Transfer date
					$output.='<td>'.$transfer->formatedDate($transfer->startTime).'&nbsp;</td>';
					// Transfer source
					$output.='<td>'.cfCaption('transferCfgSource').' : <a class="link" target="_blank" href="'.cfGGetVar('ipLocatorProvider').$transfer->sessionIP.'">'.$transfer->sessionIP.'</td>';
					// Transfer Status
					$output.='<td>'.$transfer->captionStatus().'</td>';
					// Progress bar
					if(1||cfIsInApp()) $output.='<td><div style="padding-top:2px">'.$transfer->tProgressBar('100%').'</div>';
					else $output.= '<td>'.$transfer->tProgressBar('280px').'</td><td>';

					// Source URL, complete destination file name
					if(cfIsInApp()) $onclick=' onclick="$(this).e.changeToInput()"'; else $onclick='';

					if($transfer->type=='upload') $output.= '<span style="margin-left:0.5em"'.$onclick.'>'.cfUTF8Encode($transfer->sourceFile).' -> '.cfUTF8Encode(str_replace('\\','/',$transfer->destinationFile)).'</span></td>';
					elseif($transfer->type=='remoteDownload') $output.= '<span style="margin-left:0.5em"'.$onclick.'>'.cfUTF8Encode(basename($transfer->host)).' -> '.cfUTF8Encode(str_replace('\\','/',$transfer->destinationFile)).'</span></td>';
					elseif($transfer->type=='download') $output.= '<span style="margin-left:0.5em"'.$onclick.'>'.cfUTF8Encode($transfer->sourceFile).'</span></td>';


					// Clear finished transfers
					if($transfer->status==4 || $transfer->status==6 || $transfer->status==8) $output.= '<td nowrap="nowrap">'.outButton('','javascript:transfersClear(\''.$transfer->id.'\')',outIcon('eraser'),cfCaption('transferRemove')).'</td>';
					// Stop transfer
					elseif($transfer->progress!=1) $output.= '<td nowrap="nowrap">'.outButton('','javascript:transfersStop(\''.$transfer->id.'\')',outIcon('cancel'),cfCaption('genCancel')).'</td>';
					else $output.= '<td>&nbsp;</td>';
					// Detail button
					$output.= '<td class="eTCRight" nowrap="nowrap"><span style="display:none" id="hlpLb'.$id.'">'.$transfer->detail(false).'</span>'.outButton('', 'javascript:alert(transfersJsConv(dgi(\'hlpLb'.$id.'\').innerHTML))', outIcon('info'), cfCaption('genDetails')).'</td>';
					$output.= "</tr>\n";
					if(cfIsInApp()) $output.='<tr height=3><td></td></tr>';
				}
				else{
					$output.='<tr id="tr'.$displayedTransfers.'" style="padding-left:0.5em;cursor:default;border:1px solid red" align="left">'."\n";
					// Transfer Icon
					$output.='<td>';
					if($transfer->type=='upload') $output.=outImage(outIcon('upload'),false,'title="'.cfCaption('genFileUpload').'"');
					elseif($transfer->type=='remoteDownload') $output.=outImage(outIcon('ddl'),false,'title="'.cfCaption('genDirectDownload').'"');
					elseif($transfer->type=='download') $output.=outImage(outIcon('tdl'),false,'title="'.cfCaption('transfersDownload').'"');
					elseif($transfer->type=='multipleDownload') $output.=outImage(outIcon('tmdl'),false,'title="'.cfCaption('explorerMultipleDownload').'"');
					elseif($transfer->type=='bittorrentDownload') $output.=outImage(outIcon('tbtdl'),false,'title="'.cfCaption('bittorrent').'"');
					$output.='</td>';
					// Transfer date
					$output.='<td>'.$transfer->formatedDate($transfer->startTime).'&nbsp;</td>';

					// Transfer source
					if($transfer->type=='upload') {
						$full=cfUTF8Encode($transfer->sourceFile).' -> '.cfUTF8Encode(str_replace('\\','/',$transfer->destinationFile));
						$short=basename(cfUTF8Encode($transfer->sourceFile));
					}
					elseif($transfer->type=='remoteDownload') {
						$full=cfUTF8Encode(basename($transfer->host)).' -> '.cfUTF8Encode(str_replace('\\','/',$transfer->destinationFile));
						$short=basename(cfUTF8Encode($transfer->destinationFile));
					}
					elseif($transfer->type=='download') {
						$full=cfUTF8Encode($transfer->sourceFile);
						$short=cfUTF8Encode(basename($transfer->sourceFile));
					}
					elseif($transfer->type=='bittorrentDownload') {
						$full=cfUTF8Encode($transfer->sourceFile);
					}
					else{
						$short=$full=$transfer->type.' ???';
					}

					$output.='<td><div style="width:210px;text-overflow:ellipsis;overflow:hidden;white-space:nowrap"'.outToolTip($full).'>';
					$output.=@$short;
					$output.='</div></td>';

					// Progress bar
					if(1||cfIsInApp()) $output.='<td>'.$transfer->tProgressBar('100%').'</td>';


					// Clear finished transfers
					if($transfer->status==4 || $transfer->status==6 || $transfer->status==8) $output.= '<td align="right"><img onclick="transfersClear(\''.$transfer->id.'\')" src="'.outIcon('eraser').'" '.outToolTip(cfCaption('transferRemove')).' style="cursor:pointer"></td>';
					// Stop transfer
					elseif($transfer->progress!=1) $output.= '<td align="right"><img onclick="transfersStop(\''.$transfer->id.'\')" src="'.outIcon('cancel').'" '.outToolTip(cfCaption('genCancel')).'></td>';
					else $output.= '<td>&nbsp;</td>';
					// Detail button
					$output.= '<td align="right"><img onclick="alert(transfersJsConv(dgi(\'hlpLb'.$id.'\').innerHTML))" src="'.outIcon('info').'" '.outToolTip(cfCaption('genDetails'),false,false,true).' style="cursor:pointer"><span style="display:none" id="hlpLb'.$id.'">'.$transfer->detail(false).'</span></td>';
					$output.= "</tr>\n";

				}
				if($transfer->needRefresh()) $needRefresh=true;
				$displayedTransfers++;
			}
			$output.="</table><br>\n";
		}
		// If no transfer, display "no transfer" message
		if(!$displayedTransfers){
				$output=''; // reset output
				$output.= '<div style="position:relative;top:40%;text-align:center">'.cfCaption('activityNoTransfer')."</div>\n";
		}
		// Flush page to output (XML response or HTML page)
		if(cfIsAsync()) {
			echo cfAsyncXMLInnerHTMLbyId($output,'transfersDiv');
			if($autoRefresh) echo cfAsyncXMLJSaction('transfersRefresh()');
		}
		else echo $output;
	}

	// Memorize display time for this resource or at general-level (for UI), so it's not refreshed without need
	if(cfIsResource()) cfRSetVar('transfersLastDisplayTime',time().'.'.microtime()); else cfGSetVar('transfersLastDisplayTime',time().'.'.microtime());

	if(cfIsAsync() && $autoRefresh) die(cfAsyncFooter());

	return $displayedTransfers;
}

/**
 * @desc remove a transfer from transfers list
 *
 * @param string $transferId: id of transfer to remove
 */
function tRemoveTransfer($transferId){
	$transfersArray=tReadTransfersFiles();
	if(isset($transfersArray[$transferId])){
		$transfersArray[$transferId]->clear(true);
		tWritePHPTransfersFile($transfersArray,true);
	}
}
?>