/**
 * Standalone File Browser, enabling single file or folder selection, or multiple files/folders selection
 * This file browser is intended for use on the host, without file access right management
 */
var LocalFileBrowser={
	selectType:'dir',		// "dir", "file", "multiple"
	mask:false, 			// File extension mask array, false to allow all files

	dir:'computerRoot',		// Current directory
	selected:[],			// Array of selected files/dir, with 'f:' or 'd:' prepended
	files:{},				// Array of id:file in current directory

	// On object load
	init:(function(){
		setTimeout(function(){
			$(D).e.setKeycodeListener(LocalFileBrowser.keyup);
		},100);
	})(),

	/**
	 * @desc Draw files & folders for passed directory (and update current dir)
	 * @param string dir
	 */
	update:function(files,dir){
		$('fileBrowserLocation').value=((dir=='computerRoot')?wl.capt.explorerComputerRoot:dir);
		LocalFileBrowser.files=[];

		// Store files and current directory
		var shift=0;
		if(dir!='computerRoot') {
			LocalFileBrowser.files[0]={'1':'d','2':'..','5':'/gfx/fi/folderUp.png','9':'','10':'','11':''};
			shift=1;
		}
		for(var i=0;i<files.length;i++) LocalFileBrowser.files[i+shift]=wl.objectClone(files[i]);
		LocalFileBrowser.dir=dir;
		LocalFileBrowser.setValidDir(1);

		// Update files
		W.fL=new FilesList(LocalFileBrowser.files,false,"fileBrowserFrame",32,dir);
		fL.layoutFunction=LocalFileBrowser.layoutFunction;
		fL.draw();

		// Enable/disable validate button
		LocalFileBrowser.updateValidateBtState();

		winResized();
	},

	/**
	 * @desc Request folder change: trigger async request to update browser (by calling "show" function)
	 * @param id: id of clicked directory
	 */
	navigateTo:function(id){
		if(LocalFileBrowser.selectType!='multiple') LocalFileBrowser.selected=[];
		D.comForm.data1.value=LocalFileBrowser.dir;
		fAS(fL.file(LocalFileBrowser.files[id]).name,"go","","","async");
	},


	/**
	 * User changed text locatino
	 */
	fileBrowserLocationChanged:function(){
		if($('fileBrowserLocation').value==LocalFileBrowser.dir || $('fileBrowserLocation').value==LocalFileBrowser.dir+'/') {
			LocalFileBrowser.setValidDir(1);
			return;
		}
		D.comForm.data1.value=$('fileBrowserLocation').value;
		fAS('',"checkDirectory","","","async");
	},


	/**
	 * Document keyup
	 * int kc: key code
	 */
	keyup:function(kc){
		if(kc==27) LocalFileBrowser.cancel();
		if(kc==13) LocalFileBrowser.validate();

		// Text entered into input
		if(D.activeElement && D.activeElement.id==='fileBrowserLocation') LocalFileBrowser.fileBrowserLocationChanged();
	},

	/**
	 * User selected (single click) a directory or a file
	 * @param string id: id of item
	 */
	click:function(id){
		var f=fL.file(LocalFileBrowser.files[id]);
		if(LocalFileBrowser.selectType=='dir') {
			if(f.name!='..') LocalFileBrowser.toggleSelect(id);
		}
		if(LocalFileBrowser.selectType=='file') {
			if(f.isDir) LocalFileBrowser.navigateTo(id);
			else LocalFileBrowser.toggleSelect(id);
		}
	},

	/**
	 * User double clicked a directory or a file
	 * @param string id: id of item
	 */
	dblclick:function(id){
		var f=fL.file(LocalFileBrowser.files[id]);

		if(LocalFileBrowser.selectType=='dir') LocalFileBrowser.navigateTo(id);
		if(LocalFileBrowser.selectType=='file') {
			if(f.isDir) LocalFileBrowser.navigateTo(id);
			else {
				LocalFileBrowser.select(id);
				LocalFileBrowser.validate();
			}
		}
	},

	/**
	 * @desc Toggle file/dir selection
	 * @param string id: id of item
	 * @return bool: true if selected, false if not selected (after toggle)
	 */
	toggleSelect:function(id){
		if (LocalFileBrowser.isSelected(id)) {
			LocalFileBrowser.unselect(id);
			return 0;
		}
		else {
			LocalFileBrowser.select(id);
			return 1;
		}
	},
	/**
	 * @desc Toggle file/dir selection
	 * @param string id: id of item
	 * @return bool: true if selected, false if not selected (after toggle)
	 */
	isSelected:function(id){
		if (LocalFileBrowser.selectType=='multiple') {
			return $('LFBCheck'+id).checked;
		}
		else {
			return $('tid'+id).e.className().has('selected');
		}
	},


	/**
	 * @desc Unselect file/dir
	 * @param string id: id of item
	 */
	select:function(id){
		if (LocalFileBrowser.selectType=='multiple'){
			$('LFBCheck'+id).checked=false;
			LocalFileBrowser.selected.push({file:LocalFileBrowser.files[id],dir:LocalFileBrowser.dir})
		}
		else {
			LocalFileBrowser.selected[0]={file:LocalFileBrowser.files[id],dir:LocalFileBrowser.dir};
			LocalFileBrowser.resetClassSingleSelect(id);
		}
		LocalFileBrowser.updateValidateBtState();
	},

	/**
	 * @desc Select file/dir
	 * @param string id: id of item
	 */
	unselect:function(id){
		var s,f=LocalFileBrowser.files;
		if (LocalFileBrowser.selectType=='multiple') {
			$('LFBCheck'+id).checked=false;
			// Remove unselected file from selected files list
			for(var i=0;i<LocalFileBrowser.selected.length;i++){
				s=LocalFileBrowser.selected[i];
				if(f[id][2]==s.file[2] && LocalFileBrowser.dir==s.dir){
					LocalFileBrowser.selected.splice(id,1);
					break;
				}
			}
		}
		else{
			LocalFileBrowser.resetClassSingleSelect(-1);
			LocalFileBrowser.selected=[];
		}
		LocalFileBrowser.updateValidateBtState();
	},

	/**
	 * @desc Apply selected style on select item only
	 * @param string id: id of item, -1 to unselect all
	 */
	resetClassSingleSelect:function(id){
		for(var i=0;i<LocalFileBrowser.files.length;i++){
			if(i===id*1) $('tid'+i).e.className().add('selected');
			else $('tid'+i).e.className().remove('selected');
		}
	},

	/**
	 * Change input color depending on directory existance
	 * bool isValid
	 */
	setValidDir:function(isValid){
		$('fileBrowserLocation').style.color=((isValid)?'black':'red');
	},

	/**
	 * Update validate button state (enabled / disabled)
	 */
	updateValidateBtState:function(){
		var en=(LocalFileBrowser.selected.length>0);

		if(LocalFileBrowser.selectType=='dir'){
			en=(en || LocalFileBrowser.dir!=='computerRoot')
		}
		if(en) btEnable('fileBrowserValidateBt'); else btDisable('fileBrowserValidateBt');
		LocalFileBrowser.validateOk=en;
	},

	/**
	 * User click on validate button
	 */
	validate:function(){
		if(!this.validateOk) return;
		var i,f,ret='',dir=LocalFileBrowser.dir;
		if(dir=='computerRoot') dir='';

		// If we are selecting a directory and no one is selected, use current directory
		if(LocalFileBrowser.selectType=='dir' && (!LocalFileBrowser.selected || !LocalFileBrowser.selected.length)){
			ret='d:'+LocalFileBrowser.dir;
		}
		// Regular selection: concatenate selected files/dirs
		else{
			for(i=0;i<LocalFileBrowser.selected.length;i++) {
				f=fL.file(LocalFileBrowser.selected[i].file);
				ret+=((i>0)?'|':'')+((f.isDir)?'d:':'f:')+dir;
				if(dir && dir.substr(-1,1)!='/') ret+='/';
				ret+=f.name;
			}
		}
		wl.UIWebPopupReturn(ret);
	},

	/**
	 * User click on cancel button
	 */
	cancel:function(){
		wl.UIWebPopupReturn('');
	},

	/**
	 * Return complete file name of a file in curent directory from it's basename
	 * @Param string basename: name of file, without path
	 * @Return string;
	 */
	cfn:function(basename){
		if(LocalFileBrowser.dir==='computerRoot'){
			if(LocalFileBrowser.hostOS==='windows') return basename;
			return '/'+basename;
		}
		else return LocalFileBrowser.dir+'/'+basename;
	},

	/**
	 * files/folders layout function used by FilesList Object
	 * @param array files
	 * @param int tnw
	 * @param int tnh
	 * @return string: innerHTML code
	 */
	layoutFunction:function(files,tnw,tnh){
		var i,o,selected;
		$('fileBrowserFrame').e.disableSelection();

		o='<table id="filesTable" onmousemove="eHover(event)" onmouseout="eHover()" style="text-align:right" cellspacing=0 cellpadding=0 width="100%">';
		o+='<colgroup><col width="'+((LocalFileBrowser.selectType=='multiple')?49:16)+'"><col width="*"><col width="40">';
		if(LocalFileBrowser.selectType!='dir') o+='<col width="30"><col width="40"></colgroup>';

		for(i=0;i<files.length;i++){
			files[i][7]=0; // disable download property so FilesList processing doesn't add drag & drop

			f=fL.file(files[i]);

			if(LocalFileBrowser.selectType=='dir' && !f.isDir) continue;

			o+='<tr class="eTC '+((i%2)?'even':'odd')+'" eH=1 id="tid'+i+'" ';

			o+='onclick="LocalFileBrowser.click(\''+i+'\')"';
			o+='ondblclick="LocalFileBrowser.dblclick(\''+i+'\')"';
			o+='>';

			ico='<img src="'+((f.icon)?f.icon:efFolderIcon)+'" alt="" style="height:16px;width:16px">&nbsp;';

			o+='<td class="eTCLeft" style="text-align:left">';
			if(LocalFileBrowser.selectType=='multiple'){
				selected=0;
				//for(i=0;i<LocalFileBrowser.selected.length;i++) if(LocalFileBrowser.selected[i]==LocalFileBrowser.cfn(f[2])) selected=1;
				o+='<input id="LFBCheck'+i+'" type="checkbox" style="vertical-align:top;margin:2px 5px 0 0" '+((selected)?'selected="selected"':'')+'/>';
			}
			o+=ico+'</td>'; // Icon
			o+='<td style="text-align:left;overflow:hidden">';
			o+='<div id="id'+i+'" class="mainCapt">'+ico+f.name+'</div>'; // Name  //  onmouseover="LocalFileBrowser.mouseOver(\'LFBCheck'+i+'\')"
			o+='</td>';
			if(LocalFileBrowser.selectType!='dir') o+='<td>&nbsp;'+((f.size)?f.size:'')+'</td>'; // Size
			o+='<td>&nbsp;&nbsp;&nbsp;'+f.mdate+'</td>'; // Date
			if(LocalFileBrowser.selectType!='dir') o+='<td>&nbsp;&nbsp;&nbsp;'+((f.extra)?f.extra:'')+'</td>'; // Extra info
			o+='</tr>';

		}
		o+='</table>'

		return o;
	}
}
