<?php
/**
 * User Interface community tab
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 2.2.0
 */

require_once(INCLUDE_DIR.'outputFunctions.php');
require_once(INCLUDE_DIR.'facebookFunctions.php');
require_once(INCLUDE_DIR.'transferFunctions.php');

if(!cfIsInApp() && !WEnv::user()->isAdministrator()) die('unauthorized access');

/**
 * @desc Process async POST commands
 *
 */
function uiActivityProcessPOST(){
	if(!cfIsAsync()) return;
	cfAsyncHeader();

	// Other commands
	if($command=@$_POST['command']){
		switch ($command){
			case 'ban':
				if(isset($_POST['sessionId']) && WEnv::usersLogged()->getUser($_POST['sessionId'])) cfIPBan(WEnv::usersLogged()->getUser($_POST['sessionId'])->ip);
				// don't break as we'll proceed with user logout
			case 'logout':
				// Logout connected user
				if(isset($_POST['sessionId']) && WEnv::usersLogged()->getUser($_POST['sessionId'])) WEnv::usersLogged()->disconnectUser($_POST['sessionId']);
				break;
			case 'ban':
				// Break now!
				break;
			case 'updateGVar':
				if(isset($_POST['displayOnlyActiveTransfers'])){
					cfGUpdateVar('displayOnlyActiveTransfersTime',($_POST['displayOnlyActiveTransfers']==='true')?cfConstant('COMPLETED_TRANSFERS_DISPLAY_TIME'):0);
				}
				// Change log display level
				elseif (@$_POST['applicationLogDetail']=='inf' || @$_POST['applicationLogDetail']=='det' || @$_POST['applicationLogDetail']=='dbg') {
					cfGUpdateVar('applicationLogDetail',$_POST['applicationLogDetail']);
					cfMSetVar('uiActivityLogsDisplayTime',0); // Force display update
				}
				break;
		}
	}

	// Process frame specific async commands
	uiActivityConnectedUsersBox();
	uiActivityTransfersBox();
	uiActivityLogsBox();

	// Frames heights
	if(isset($_POST['uiActivityConnectedUsersBoxHeight'])&&is_numeric($_POST['uiActivityConnectedUsersBoxHeight'])) cfGUpdateVar('uiActivityConnectedUsersBoxHeight',(int)$_POST['uiActivityConnectedUsersBoxHeight']);
	if(isset($_POST['uiActivityTransfersBoxHeight'])&&is_numeric($_POST['uiActivityTransfersBoxHeight'])) cfGUpdateVar('uiActivityTransfersBoxHeight',(int)$_POST['uiActivityTransfersBoxHeight']);

	// Schedule next async refresh
	if(isset($_POST['uiActivityRefresh'])) echo cfAsyncJSaction('setTimeout("uiActivity.refresh()",'.cfConstant('UIACTIVITY_REFRESH_INTERVAL').')');

	die(cfAsyncFooter());
}

/**
 * @desc Connected users frame
 *
 */
function uiActivityConnectedUsersBox(){
	// Async command not meant to refresh content
	if(cfIsAsync() && !isset($_POST['uiActivityRefresh'])) return;
//cfDbg('last update in display: '.cfTimtestampToFullDate(cfMGetVar('uiActivityLastConnectedUsersUpdate')).' last user update event : '.cfTimtestampToFullDate(WEnv::usersLogged()->lastUpdate),'');
	// Displayed transfers already up to date
	if(cfIsAsync() && cfMGetVar('uiActivityLastConnectedUsersUpdate')>WEnv::usersLogged()->lastUpdate) return;


	// Build connected users list
	$out=''; $nbConnected=0;
	foreach (WEnv::usersLogged()->getUsers() as $sessId=>$loggedUser){
		if(0) $loggedUser=new WUserLogged(); // Zend IDE WA

		$browserIcon='/gfx/browsers/'.$loggedUser->browserName.'.png';
		if(!file_exists(cfAppDocRoot().$browserIcon)) $browserIcon='/gfx/browsers/Default.png';

		$content='';
		// Icon
		$content.='<img src="'.cfExtGFX('/gfx/icons/'.$loggedUser->icon,24,24).'" style="width:24px;height:24px;margin-right:10px;vertical-align:middle">';
		$content.='<img src="'.$browserIcon.'" style="position:absolute;top:16px;left:16px">';

		// Group name
		$content.='<fieldset style="margin-right:5em;vertical-align:middle"><b>'.cfUTF8Encode($loggedUser->groupName).'</b>';

		// User pseudo or subUser name
		if($loggedUser->pseudo) $content.=' ('.cfUTF8Encode($loggedUser->pseudo).')';
		if($loggedUser->subUserId) {
			$su=new WSubUser($loggedUser->subUserId);
			$content.=' ('.cfUTF8Encode($su->name()).')';
		}
		$content.='<br>';

		// IP address
		if(cfIsOnLAN($loggedUser->ip))
			$content.=cfCaption('activityIP').cfCaption('genSeparator').$loggedUser->ip;
		else
			$content.=cfCaption('activityIP').cfCaption('genSeparator').'<a href="'.cfIPLocatorLink($loggedUser->ip).'" target="_blank" class="link">'.$loggedUser->ip.'</a>';

		$content.='</fieldset>';

		// Connected since
		$content.=cfCaption('activityConnectedSince').cfCaption('genSeparator').date(cfCaption('_HMS_DATE_FORMAT'),$loggedUser->connectionTime);

		// Linked resources
		$content.='<span style="position:absolute;right:1px;top:1px;text-align:right">';
		$resLinked=$loggedUser->config()->getResources(true);
		$nb=0;
		if(count($resLinked)==1){ // Single resource
			$content.='<span style="overflow:hidden;width:28px;height:28px">'.outImage(current($resLinked)->icon32(),false,false,'margin:-2px 0 0 -2px').'</span>';
		}
		else foreach ($resLinked as $rid=>$res){ // Many resources
			if(0) $res=new WResConfig();
			$content.='<span style="overflow:hidden;width:14px;height:14px"><img alt="" src="'.$res->icon16().'" style="margin:-1px 0 0 -1px" '.outToolTip(' ', cfUTF8Encode($res->name()).'<br>&nbsp;',$res->icon32(),true).'></span>';
			if(++$nb==floor(count($resLinked)/2)) $content.='<br>';
		}
		$content.='</span>';

		// Block item
		$out.=outBlockItem('connectedUsers',$content,'id="cUB'.$sessId.'"','width:100%;margin-top:1px;margin-bottom:1px;padding:3px',array('onclick'=>"uiActivity.connectedUsersSelect('".$loggedUser->sessionId."','".$loggedUser->ip."','".($loggedUser->hasChat)."')"));
		$nbConnected++;

	}
//cfDbg('update:'.$nbConnected,'');
	cfMSetVar('uiActivityLastConnectedUsersUpdate',time());

	// Async refresh
	if(cfIsAsync()) {
		echo cfAsyncXMLInnerHTMLbyId($out,'connectedUsersDiv');
		return ;
	}

	// Sync display
	?>
	<div class="frame2Header"><?php echo outImageIcon('users').cfCaption('adminConnectedUsers');?></div>
	<div class="leftControls">
	<?php
	echo outBt(cfCaption('chatDoChat'),outIcon('chat'),'javascript:uiActivity.connectedUsersChat()','chatBt',false,'','off',90,'center').'<br>';
	echo outBt(cfCaption('mainActivityButton').'...',outIcon('view'),'javascript:uiActivity.connectedUsersActivity()','activityBt',false,'','off',90,'center').'<br>';
	echo outBt(cfCaption('genLogout'),outIcon('off'),'javascript:uiActivity.connectedUsersLogout()','logoutBt',false,'','off',90,'center').'<br>';
	echo outBt(cfCaption('genBan'),outIcon('noX'),'javascript:uiActivity.connectedUsersBan()','banBt',false,'','off',90,'center');
	?>
	</div>
	<div id="connectedUsersDiv" class="frame2 alt cont">
	<?php

	// Connected users list
	echo $out;
	?>
	</div>
	<?php
}

/**
 * @desc Transfers frame
 *
 */
function uiActivityTransfersBox(){
	if(cfIsAsync()){
		// Async update
		if(isset($_POST['uiActivityRefresh'])) $_POST['transfersRefresh']=true; // Set this post to true to "fool" tDisplayTransfers function
		if(isset($_POST['displayOnlyActiveTransfers'])) $forceRefresh=true; else $forceRefresh=false;
		tDisplayTransfers(false,false,false,$forceRefresh);
		return;
	}
	?>
	<div class="frame2Header"><?php echo outImageIcon('transfers').cfCaption('explorerFileTransfer');?></div>
	<div class="leftControls">
	<?php echo outBt(cfCaption('explorerClearFinishedTransferts'),outIcon('eraser'),"javascript:transfersClear('all')",'chatBt',false,'','up',90,'center');?>
	<br>
	<div style="width:120px;overflow:hidden;text-overflow:ellipsis;white-space:nowrap" <?php echo outToolTip(cfCaption('displayOnlyActiveTransfers'));?>>
	<input type="checkbox" id="displayOnlyActiveTransfers" onclick="wl.asr.send({'command':'updateGVar','displayOnlyActiveTransfers':this.checked})" <?php if(cfGGetVar('displayOnlyActiveTransfersTime')>0) echo ' checked'; ?>>&nbsp;
	<span onclick="$('displayOnlyActiveTransfers').checked=!$('displayOnlyActiveTransfers').checked;wl.asr.send({'command':'updateGVar','displayOnlyActiveTransfers':$('displayOnlyActiveTransfers').checked})"><?php echo cfCaption('displayOnlyActiveTransfers'); ?></span>
	</div>
	</div>
	<div id="transfersDiv" class="frame2 alt cont">
	<?php
	$nb=tDisplayTransfers(false,false,false);
	?></div>
	<?php
}

/**
 * @desc Application log
 *
 */
function uiActivityLogsBox(){
	global $logFile;
	$logFile=cfAppLogDir().'/'.MAIN_LOG_FILE;

	/**
	 * @desc Read tail of log file and parse lines
	 *
	 * @return array of utf8 encoded lines
	 */
	function getRawLines(){
		global $logFile;

		// Read log
		if($fp=@fopen($logFile,'r')){
			$start=max(0,filesize($logFile)-120*cfGGetVar('applicationLogMaxLines'));
			fseek($fp,$start); // Read only end of file
			$lines=fread($fp,filesize($logFile)-$start);
			fclose($fp);
			$lines=explode("\n",cfUTF8Encode($lines));
		}
		else $lines=array();
		return $lines;
	}

	/**
	 * @desc Format log line
	 */
	function formatedLine($l){
		global $logFile;
		static $logLevel,$sep,$morning=0,$icoLocal,$icoLan,$icoWeb,$icoApp,$icoErr,$resConfigs,$userConfigs;

		// Init function
		if(!$morning) {
			$morning=mktime(0,0,0,date("m"),date("d"),date("Y"));
			$logLevel=cfGGetVar('applicationLogDetail');
			$sep=cfCaption('genSeparator');
			$icoLocal=outImageIcon('localhost',''); // prod
			$icoLan=outImageIcon('LAN2','');
			$icoWeb=outImageIcon('web','');
			$icoApp='<img src="'.outIconApp(16).'" >';
			$icoErr=outImageIcon('warning','');
			$resConfigs=WEnv::resConfigs();
			$userConfigs=WEnv::userConfigs();
		}
		if(!($l=str_replace("\r","",$l))) return;

		// Parse list
		@list($ts,$src,$eventType,$level,$ip,$str)=explode('/',$l,6);
		if(!is_numeric($eventType)) @list($ts,$src,$level,$ip,$str)=explode('/',$l,5); // Compatibility with pre-4.0.1 versions
		$eventType=(int)$eventType;

		if(!$ip) return '<div class="dbg">'.$l.'/</div>';

		// Only display if level >= logLevel
		if($logLevel=='dbg' || ($logLevel=='det'&& ($level=='det'||$level=='err')) || $level=='inf'){
			// Icon
			if($src=='app') $ico=$icoApp;
			elseif($ip=='127.0.0.1') $ico=$icoLocal;
			elseif(cfIsOnLAN($ip)) $ico=$icoLan;
			else $ico=$icoWeb;

			// Extra icon
			if(substr($src,0,3)=='res' && isset($resConfigs[substr($src,3)])) $ico.='<img src="'.cfExtGFX($resConfigs[substr($src,3)]->icon32(),12,12).'">';
			elseif(substr($src,0,3)=='usr' && isset($userConfigs[substr($src,3)])) $ico.='<img src="'.cfExtGFX($userConfigs[substr($src,3)]->iconPath(),12,12).'">';
			elseif($src=='account') $ico.='<img src="/gfx/dotNet12.png">';
			else $ico.='<img src="/gfx/v.gif">';

			// Clean text
			$str=str_replace('&amp;','&',$str);

			// Extra icon
			if($eventType==LOG_EVENT_LOGIN && !cfIsOnLAN($ip)) $str.='&nbsp;<a href="'.cfIPLocatorLink($ip).'" target="_blank"><img src="'.outIcon('maps').'"></a>';

			// Return formated line
			return '<div class="'.$level.' et'.$eventType.'">'.$ico.(($level=='err')?$icoErr:'').cfFixedDate($ts,(($ts>$morning)?cfCaption('_HMS_DATE_FORMAT'):cfCaption('_FULL_DATE_FORMAT'))).$sep.$str.'</div>';
		}
		return '';
	}



	// Refresh logs with transfers refresh command
	if(cfIsAsync()){
		if(isset($_POST['uiActivityRefresh']) || isset($_POST['uiActivityForceRefreshLogs']) || isset($_POST['applicationLogDetail'])){
			// If file has not been updated since last display, don't do anything
			if(!isset($_POST['uiActivityForceRefreshLogs']) && !isset($_POST['applicationLogDetail']) && @filemtime($logFile)<cfMGetVar('uiActivityLogsDisplayTime')) return;

			// Set last display time
			cfMSetVar('uiActivityLogsDisplayTime',time());

			// Read log file
			$lines=getRawLines();
			$nb=count($lines);

			// Output last lines in inverted order
			$out='';
			$skipped=0;
			for ($i=0;$i<$skipped+cfGGetVar('applicationLogMaxLines') && $nb-$i-2-$skipped>=0;$i++) {
				$line=formatedLine($lines[$nb-$i-2-$skipped]);
				if($line) $out.=$line; else $skipped++;
			}
			echo cfAsyncXMLInnerHTMLbyId($out,'uiLogDiv');
		}
		return;
	}

	// Set last display time
	cfMSetVar('uiActivityLogsDisplayTime',time());

	// Read log
	$lines=getRawLines();
	$nb=count($lines);

	?>
	<div class="frame2Header"><?php echo outImageIcon('log').cfCaption('activityLog');?></div>
	<div class="leftControls">
	<select class="textInput" size="1" onchange="uiActivity.setLogLevel(this.value)" style="width:120px" id="applicationLogDetail">
	<option value="inf"<?php if(cfGGetVar('applicationLogDetail')=='inf') echo 'selected';?>><?php echo cfCaption('activityLogDetailInf'); ?></option>
	<option value="det"<?php if(cfGGetVar('applicationLogDetail')=='det') echo 'selected';?>><?php echo cfCaption('activityLogDetailDet'); ?></option>
	<?php if(cfGGetVar('PHPDebug')) echo '<option value="dbg" '.((cfGGetVar('applicationLogDetail')=='dbg')?'selected':'').'>'.cfCaption('activityLogDetailDbg').'</option>';?>
	</select><br><br>
	<?php
	echo outBt(cfCaption('genStats'),outIcon('stats2'),"javascript:wl.UIWebPopup('/local/showStats.php')",'statsBt',false,'','up',90,'center').'<br>';
	echo outBt(cfCaption('genHistory'),outIcon('log'),"javascript:wl.UIWebPopup('/local/showHistory.php')",'historyBt',false,'','up',90,'center');
	?>
	</div>
	<div id="uiLogDiv" uiSelectable="1" class="frame2 alt cont">
<?php
	$skipped=0;
	for ($i=0;$i<$skipped+cfGGetVar('applicationLogMaxLines') && $nb-$i-2-$skipped>=0;$i++) {
		$line=formatedLine($lines[$nb-$i-2-$skipped]);
		if($line) echo $line; else $skipped++;
	}
?>
	</div>
	<?php
}




/**
 ***************************************************************************************************************************
 * Process async POST commands
 ***************************************************************************************************************************
 */
uiActivityProcessPOST();



/**
 ***************************************************************************************************************************
 * Insert page HEADER
 ***************************************************************************************************************************
 */
cfInsertHEAD(false);
?>
<style type="text/css">
.cont{overflow:auto;margin:0 5px 0px 135px;overflow-x:hidden}
.leftControls{position:absolute;left:5px}
</style>
</head>
<body style="height:100%;overflow:hidden">
<input type="hidden" id="UIScript" name="UIScript" onkeyup="eval(this.value)">
<?php
echo outLoadMask(false); flush();
echo cfScriptLink('wz_dragdrop.js');
echo cfScriptLink('winClient.js');
?>
<div id="uiActivityConnectedUsersBox" class="frame2 alt" style="height:<?php echo (cfGGetVar('uiActivityConnectedUsersBoxHeight'))?cfGGetVar('uiActivityConnectedUsersBoxHeight'):'200';?>px;overflow:auto">
	<?php uiActivityConnectedUsersBox();?>
</div>

<div id="sep1"></div>

<div id="uiActivityTransfersBox" class="frame2 alt" style="height:<?php echo (cfGGetVar('uiActivityTransfersBoxHeight'))?cfGGetVar('uiActivityTransfersBoxHeight'):'200';?>px;overflow:auto">
<?php uiActivityTransfersBox();?>
</div>

<div id="sep2"></div>

<div id="uiActivityLogsBox" class="frame2 alt" style="height:<?php echo (cfGGetVar('uiActivityLogsBoxHeight'))?cfGGetVar('uiActivityLogsBoxHeight'):'200';?>px;overflow:auto"><?php uiActivityLogsBox();?></div>

<?php echo cfDragRegisterItems('');?>

<script type="text/javascript">
var uiActivity={
	frameHeight:0,
	connectedUsersSelectedChat:'',
	connectedUsersSelectedSessId:'',

	// Sync init stuff
	init:function(){
		$('sep1').e.setHSeparator({'minHeight':170,'asyncVar':'uiActivityConnectedUsersBoxHeight','callback':function(){uiActivity.connectedUsersResized();uiActivity.connectedTransfersResized();}});
		$('sep2').e.setHSeparator({'minHeight':80,'asyncVar':'uiActivityTransfersBoxHeight','callback':function(){uiActivity.connectedTransfersResized();uiActivity.connectedLogsResized()}});
		winResized();
		setTimeout(uiActivity.refresh,<?php echo cfConstant('UIACTIVITY_REFRESH_INTERVAL');?>)
		setInterval(uiActivity.refreshSecurity,<?php echo cfConstant('UIACTIVITY_REFRESH_INTERVAL');?>)
	},

	// Window resized
	winResized:function(){
		var pH=uiActivity.frameHeight,h=dd.getWndH(),bH,s;

		alignBottom($('uiActivityLogsBox'),winMe,5);
		if(!pH){
			if($('uiActivityLogsBox').e.actualOffsetBottom()>=winMe.getInnerHeight()-5 || $('uiActivityLogsBox').offsetHeight<75) {
				alignBottom($('uiActivityTransfersBox'),winMe,80);
				alignBottom($('uiActivityLogsBox'),winMe,5);
			}
		}
		else{
			s=$('uiActivityConnectedUsersBox').style;
			bH=1*s.height.replace(/px/,'')
			s.height=Math.max(170,Math.floor(bH+0.3*(h-pH)))+'px';
			s=$('uiActivityTransfersBox').style;

			bH=1*s.height.replace(/px/,'')
			s.height=Math.max(80,Math.floor(bH+0.3*(h-pH)))+'px';

			alignBottom($('uiActivityLogsBox'),winMe,5);
		}
		this.frameHeight=h;
		this.connectedUsersResized();
		this.connectedTransfersResized();
		this.connectedLogsResized();

	},
	// Connected users frame resized
	connectedUsersResized:function(){alignBottom($('connectedUsersDiv'),$('uiActivityConnectedUsersBox'),5)},
	// Transfers frame resized
	connectedTransfersResized:function(){alignBottom($('transfersDiv'),$('uiActivityTransfersBox'),5)},
	// Logs frame resized
	connectedLogsResized:function(){alignBottom($('uiLogDiv'),$('uiActivityLogsBox'),5)},

	// Async update content
	refresh:function(){
		wl.asr.send({'uiActivityRefresh':1,'noDebug':1})
		uiActivity.refreshTime=wl.chrono.time();
	},

	// Security on async content refresh: as any php-side bug would block refresh, an interval calls this function which will relaunch refresh if not called for too long
	refreshSecurity:function(){
		if(wl.chrono.time() - uiActivity.refreshTime > <?php echo 3*cfConstant('UIACTIVITY_REFRESH_INTERVAL')/1000;?>) uiActivity.refresh();
	},

	// A connected user has been selected
	connectedUsersSelect:function(sessionId,ip,hasChat){
		if(hasChat) btEnable('chatBt'); else btDisable('chatBt');
		uiActivity.connectedUsersSelectedChat=hasChat;
		uiActivity.connectedUsersSelectedSessId=sessionId;

		if(ip!='127.0.0.1') btEnable('banBt'); else btDisable('banBt');
		btEnable('activityBt');
		btEnable('logoutBt');
	},

	// Chat with a connected user
	connectedUsersChat:function(){
		var chatFN=uiActivity.connectedUsersSelectedChat;
		chatFN=chatFN.substr(chatFN.indexOf('/')+1);
		if(chatFN.indexOf(',')!=-1) chatFN=chatFN.substr(0,chatFN.indexOf(','));
		wl.UIWebPopup('/local/displayResource.php?resourceConfigFile='+chatFN+'&forceOpen='+uiActivity.connectedUsersSelectedSessId+'&userPseudo=<?php echo (($regName=cfRegistrationData('name'))?$regName:cfCaption('userAdministrator'));?>',false,{'width':800,'height':600,'resizable':1})
	},

	// Show activity of a connected user
	connectedUsersActivity:function(){
		wl.UIWebPopup('/local/showHistory.php?sessionId='+uiActivity.connectedUsersSelectedSessId,false,{'width':800,'height':600,'resizable':1})
	},

	// Logout a connected user
	connectedUsersLogout:function(){
		$('cUB'+uiActivity.connectedUsersSelectedSessId).e.remove();
		wl.asr.send({'command':'logout','sessionId':uiActivity.connectedUsersSelectedSessId})
	},

	// Chat with a connected user
	connectedUsersBan:function(){
		$('cUB'+uiActivity.connectedUsersSelectedSessId).e.remove();
		wl.asr.send({'command':'ban','sessionId':uiActivity.connectedUsersSelectedSessId})
	},

	// Set log level (inf, det, dbg)
	setLogLevel:function(l){wl.asr.send({'command':'updateGVar','applicationLogDetail':l,'uiActivityForceRefreshLogs':1})}

}
function winResized(){uiActivity.winResized()}
uiActivity.init();
</script>
<?php
