<?php
/**
 * Site access statistics and history (V1.2.0+)
 *
 * this script is included by /administration/std/index.php or /local/showStats.php or /local/showHistory.php and shouldn't be directly called
 *
 * $displayType: history | stats
 * $infoType: type of info displayed.
 * 				stats: synth (graphs) | details (list of connections grouped by IP)
 * 				history: single (single selected connection) | list (all connections in given time interval)
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

define('MONITOR_LAST_VIEWED_FILE_INTERVAL',2000); // Interval (ms) for update of last viewed file thumnail (single history)
define('STATS_MAX_THUMBNAIL_WIDTH',160); // Left panel thumbnail max width, file info thumbnail max width
define('STATS_MAX_THUMBNAIL_HEIGHT',120); // Left panel thumbnail max height, file info thumbnail max height
$_ENV['colorPath']=array(array(150,100,160),array(250,100,250),array(250,125,0),array(50,50,250),array(50,250,50),array(250,50,50),array(0,250,250),array(255,0,0),array(255,255,0)); // Color path used for graphs coloration (linear interpolation between dots)


/**
 * @desc set interval and localhost condition on a request on connection and resAccess tables
 *
 * @param string $query : SQL query
 * @param integer $ts : "from" timestamp
 * @param string $interval  : interval length : 'month', 'week' or 'day'
 * @param string $groupBy : GROUP BY condition : 'day', 'hour', 'no'
 */
function createView1($ts, $ets, $showLocalhost=false){
	global $db;

	$query ="CREATE TEMP VIEW baseView AS SELECT connection.id AS id, connection.ip AS ip, connection.hostname AS hostname, connection.pseudo AS pseudo, connection.ts AS ts, connection.accountId AS accountId, Sum(connection.duration) AS totalDuration, resAccess.resourceId as resourceId, Sum(resAccess.pages) AS nbPages ";
	$query.="FROM connection LEFT JOIN resAccess ON connection.id = resAccess.connectionId  WHERE ";

	if(!$showLocalhost) $query.="connection.ip!='127.0.0.1' AND ";
	$query.="connection.ts>='".$ts."' AND connection.ts<='".$ets."' ";
	$query.=" GROUP BY connection.ip, connection.ts, connection.accountId, resAccess.resourceId";

	dbExec($query, $db);
}

/**
 * @desc set interval and localhost condition on a request on connection and resAccess tables
 *
 * @param string $query : SQL query
 * @param integer $ts : "from" timestamp
 * @param string $interval  : interval length : 'month', 'week' or 'day'
 * @param string $groupBy : GROUP BY condition : 'day', 'hour', 'no'
 */
function createViewHistory($ts, $ets, $showLocalhost=false){
	global $db;
	$query ="CREATE TEMP VIEW baseView AS SELECT connection.id AS id, connection.ip AS ip, connection.hostname AS hostname, connection.pseudo AS pseudo, connection.ts AS ts, connection.accountId AS accountId, Sum(connection.duration) AS totalDuration, resAccess.resourceId AS resourceId, Sum(resAccess.pages) AS nbPages ";
	$query.="FROM connection LEFT OUTER JOIN resAccess ON connection.id = resAccess.connectionId  WHERE ";

	if(!$showLocalhost) $query.="connection.ip!='127.0.0.1' AND ";
	$query.="connection.ts>='".$ts."' AND connection.ts<='".$ets."' ";
	$query.=" GROUP BY connection.ip, connection.ts, connection.accountId, resAccess.resourceId";
	dbExec($query, $db);
}

/**
 * @desc set interval and localhost condition on a request on connection and resAccess tables
 *
 * @param string $query : SQL query
 * @param integer $ts : "from" timestamp
 * @param string $interval  : interval length : 'month', 'week' or 'day'
 * @param string $groupBy : GROUP BY condition : 'day', 'hour', 'no'
 */
function createViewDetail($connectionId){
	global $db;
	$query ="CREATE TEMP VIEW baseView AS SELECT connection.id AS id, connection.ip AS ip, connection.hostname AS hostname, connection.pseudo AS pseudo, connection.ts AS ts, connection.accountId AS accountId, Sum(connection.duration) AS totalDuration, resAccess.resourceId AS resourceId, Sum(resAccess.pages) AS nbPages ";
	$query.="FROM connection LEFT OUTER JOIN resAccess ON connection.id = resAccess.connectionId  WHERE connection.id = ".$connectionId;
	$query.=" GROUP BY connection.ip, connection.ts, connection.accountId, resAccess.resourceId";
	dbExec($query, $db);
}

/**
 * @desc set interval and localhost condition on a request on connection table
 *
 * @param string $query : SQL query
 * @param integer $ts : "from" timestamp
 * @param string $interval  : interval length : 'month', 'week' or 'day'
 * @param string $groupBy : GROUP BY condition : 'day', 'hour', 'no'
 */
function createView2($ts, $ets, $showLocalhost=false){
	global $db;
	$query ="CREATE TEMP VIEW baseView AS SELECT * FROM connection WHERE ";
	if(!$showLocalhost) $query.="connection.ip!='127.0.0.1' AND ";
	$query.="connection.ts>='".$ts."' AND connection.ts<='".$ets."' ";
	dbExec($query, $db);
}

function array_plus($a,$b){return $a+$b;}

function yMax($nb){
	if($nb<5) return 5;
	$round=max(5,5*(pow(10,floor(log10($nb))-1)));
	return $res=ceil($nb/$round)*$round;
}

function truncateCaption($capt,$nb){
	if(strlen($capt)>$nb) return substr($capt,0,$nb-3).'...'; else return $capt;
}

/**
 * @desc draw connections by account bar graph
 *
 * @param integer $intStart : start timestamp
 * @param integer $intEnd : end timestamp
 * @param string $intLength : length of interval (day, week, month)
 * @param boolean $showLocalhost : show localhost connections
 */
function drawConnections($intStart, $intEnd, $intLength, $showLocalhost){
	global $db;

	WHeaders::noCache();

	if($intLength=='all'){
		if(($intEnd-$intStart)>24*31*24*3600) {
			$intStep='year';
			$intStart=mktime(0,0,1,1,1,date('Y',$intStart));
		}
		elseif(($intEnd-$intStart)>4*31*24*3600) {
			$intStep='month';
			$intStart=mktime(0,0,1,date('m',$intStart),1,date('Y',$intStart));
		}
		elseif(($intEnd-$intStart)>31*24*3600){
			$intStep='week';
		}
		else $intStep='day';
	}
	elseif($intLength=='week' || $intLength=='month') $intStep='day';

	// Create view according to defined period and other options
	createView2($intStart,$intEnd,$showLocalhost);

	if($showLocalhost=='1') $showLocalhost=true; else $showLocalhost=false;

	// Graph array
	$xValues=array();
	$t=$intStart;
	while ($t<=$intEnd) {
		if($intStep=='day'){
			$xValues[date('Y-m-d',$t)]=cfUTF8Encode(cfFixedDate($t,cfCaption('_DATE_FORMAT')));
			$t+=24*3600;
			$sqlStep='%Y-%m-%d';
		}
		elseif($intStep=='week'){
			$xValues[date('Y-W',$t)]=cfUTF8Encode(strftime(cfCaption('calWeekShort'),$t));
			$t+=7*24*3600;
			$sqlStep='%Y-%W';
		}
		elseif($intStep=='month'){
			$xValues[date('Y-m',$t)]=cfUTF8Encode(strftime('%b %Y',$t));
			$t+=date('t',$t)*24*3600;
			$sqlStep='%Y-%m';
		}
		else{
			$xValues[date('Y',$t)]=cfUTF8Encode(strftime('%Y',$t));
			$t=mktime(0,0,1,1,1,date('Y',$t)+1);
			$sqlStep='%Y';
		}
	}

	// Db request
	$result=dbArrayQuery("SELECT strftime('".$sqlStep."',ts,'unixepoch') AS dte,Count(ip) AS cnt, accountId FROM baseView GROUP BY strftime('".$sqlStep."',baseView.ts,'unixepoch'),accountId",$db);
	$accounts=dbArrayQuery("SELECT id,name FROM account",$db);

	foreach ($accounts as $key=>$value){
		$found=false;
		foreach ($result as $value2){
			if($value2['accountId']==$value['id']){
				$found=true;
				break;
			}
		}
		if($found){
			$accountsNames[$value['id']]=$value['name'];
			$accountsValues[$value['id']]=array_pad(array(),count($xValues),0);
		}
	}
	unset($accounts);
	if(!isset($accountsNames)) {
		$accountsNames=array('void'=>'void');
		$accountsValues['void']=array_pad(array(),count($xValues),0);
	}
	$datesList=array_flip(array_keys($xValues));

	// Fill accountsValues array with $result content
	foreach ($result as $value)	if($value['accountId']) @$accountsValues[$value['accountId']][$datesList[$value['dte']]]+=$value['cnt'];
	// Sum values for bar graph
	foreach ($accountsNames as $key=>$value) {
		if(isset($prevKey)) $accountsValues[$key]=array_map('array_plus',$accountsValues[$key],$accountsValues[$prevKey]);
		$prevKey=$key;
	}

	dbClose($db);
	require_once INCLUDE_DIR."/artichow/BarPlot.class.php";

	$graph = new Graph(400, 350);
	$graph->setAntiAliasing(TRUE);
	$graph->title->setFont(new Tuffy(12));

	$nb=0;
	foreach (array_reverse($accountsNames,true) as $key=>$value) {
		$plot = new BarPlot($accountsValues[$key]);
		$plot->grid->setBackgroundColor(new Color(0, 240, 240,100));

		// compute bar color
		$cpi=2;
		$bottom=min(floor($nb/$cpi),count($_ENV['colorPath'])-2);
		$r=floor(($_ENV['colorPath'][$bottom][0]+($nb-$cpi*$bottom)*($_ENV['colorPath'][$bottom+1][0]-$_ENV['colorPath'][$bottom][0])/($cpi+1)));
		$g=floor(($_ENV['colorPath'][$bottom][1]+($nb-$cpi*$bottom)*($_ENV['colorPath'][$bottom+1][1]-$_ENV['colorPath'][$bottom][1])/($cpi+1)));
		$b=floor(($_ENV['colorPath'][$bottom][2]+($nb-$cpi*$bottom)*($_ENV['colorPath'][$bottom+1][2]-$_ENV['colorPath'][$bottom][2])/($cpi+1)));

		$color=new Color($r,$g,$b,0);

		$colors[]=$color;
		$plot->setBarColor($color);
		$graph->add($plot);

		if(!isset($axisDrawn)){
			$plot->setBackgroundGradient(new LinearGradient(new Color(230, 230, 250),new Color(255, 255, 255),0));
			$xPlotLabel=array_values($accountsValues[$key]);foreach ($xPlotLabel as $k=>$v) if($v==0) $xPlotLabel[$k]='';
			$plot->xAxis->setLabelText(array_values($xValues));
			$plot->xAxis->setLabelAngle(90);
			$plot->label->set($xPlotLabel);
			$plot->label->move(0, -10);
			$plot->setYMin(0);
			$yMax=yMax($plot->getYMax());

			$graph->title->set(cfCaption('calBarPlotTitle').' '.cfCaption('calTotal',array_sum($accountsValues[$key])));

			$axisDrawn=true;
		}
		else{
			$plot->yAxis->hide(true);
			$plot->xAxis->hide(true);
			$plot->grid->hide(true);
			$plot->setYMin(0);
		}
		$plot->setPadding(20, 80, 50, 60);
		$plot->setYMax($yMax);
		$nb++;
	}

	require_once INCLUDE_DIR."/artichow/Pie.class.php";

	$values = array_pad(array(),count($accountsNames),1);

	$plot = new Pie($values, $colors);

	$plot->setCenter(0.4, 0.55);
	$plot->setSize(0.7, 0.6);
	$plot->setPadding(-400, 400, NULL, 60);

	$plot->setLegend(array_reverse(array_values($accountsNames)));
	//$plot->legend->setBackgroundColor(new Color(255, 255, 255,125));

	$plot->legend->setPosition(1.3);
	$plot->legend->shadow->setSize(0);

	$graph->add($plot);

	$graph->draw();
	exit;
}

/**
 * @desc draw unique visitors by account pie
 *
 * @param integer $intStart : start timestamp
 * @param integer $intEnd : end timestamp
 * @param string $intLength : length of interval (day, week, month)
 * @param boolean $showLocalhost : show localhost connections
 */
function drawVisitors($intStart, $intEnd, $intLength, $showLocalhost){
	global $db;

	if($showLocalhost=='1') $showLocalhost=true; else $showLocalhost=false;

	// Create view according to defined period and other options
	createView2($intStart,$intEnd,$showLocalhost);

	// Db requests
	$result=dbArrayQuery("SELECT ts,ip FROM baseView",$db);

	$tmpFirstVisit=dbArrayQuery("SELECT ip,min(ts) AS first FROM connection GROUP BY ip",$db);
	$firstVisit=array();foreach ($tmpFirstVisit as $value) $firstVisit[$value['ip']]=$value['first'];
	unset($tmpFirstVisit);

	$first=0;$returning=0;
	foreach ($result as $value){
		if($firstVisit[$value['ip']]==$value['ts']) $first++; else $returning++;
	}
	dbClose($db);
	require_once INCLUDE_DIR."/artichow/Pie.class.php";
	$graph = new Graph(350,350);

	//$graph->setAntiAliasing(TRUE);
	$graph->title->set(cfCaption('calBarPieTitle'));
	$graph->title->setFont(new Tuffy(12));

	$graph->setBackgroundGradient(new LinearGradient(new Color(230, 230, 250),new Color(255, 255, 255),0));

	if($first+$returning){
		$plot = new Pie(array($first, $returning), Pie::AQUA);
		$plot->setCenter(0.45, 0.49);
		$plot->setSize(0.8, 0.8);
		$plot->set3D(20);
		$plot->explode(array(0 => 30));
		$plot->setStartAngle(320);

		$plot->setLegend(array(cfCaption('calBarPieFirst').' ('.$first.')',cfCaption('calBarPieReturning').' ('.$returning.')'));
		//$plot->legend->setBackgroundColor(new Color(255, 255, 255,125));
		$plot->legend->setPosition(1.15,1);
		$plot->legend->shadow->setSize(0);

		$graph->add($plot);
	}
	$graph->draw();
	exit;
}

/**
 * @desc draw unique visitors by account pie
 *
 * @param integer $intStart : start timestamp
 * @param integer $intEnd : end timestamp
 * @param string $intLength : length of interval (day, week, month)
 * @param boolean $showLocalhost : show localhost connections
 */
function drawResources($intStart, $intEnd, $intLength, $showLocalhost){
	global $db;
	if($showLocalhost=='1') $showLocalhost=true; else $showLocalhost=false;

	// Create view according to defined period and other options
	createView1($intStart,$intEnd,$showLocalhost);

	// Db requests
	$accessedResources=array();

	foreach (dbArrayQuery("SELECT resourceId, sum(nbPages) AS nbPages FROM baseView GROUP BY resourceId",$db) as $value)
		if($value['resourceId']) $accessedResources[$value['resourceId']]=$value['nbPages'];

	$resourceNames=array();
	foreach(dbArrayQuery("SELECT * FROM resource",$db) as $value)
		$resourceNames[$value['id']]=$value['name'];

	$resources=array();
	$nb=0;

	// Required for Color class
	require_once INCLUDE_DIR."/artichow/BarPlot.class.php";

	$colors=array();
	foreach ($accessedResources as $key=>$value)
		if(isset($resourceNames[$key])) {
			$resources[$key]=(truncateCaption($resourceNames[$key],15)).'('.$value.')';
/*
			$bottom=min(floor($nb/3),count($_ENV['colorPath'])-2);
			$r=floor(($_ENV['colorPath'][$bottom][0]+($nb-3*$bottom)*($_ENV['colorPath'][$bottom+1][0]-$_ENV['colorPath'][$bottom][0])/4));
			$g=floor(($_ENV['colorPath'][$bottom][1]+($nb-3*$bottom)*($_ENV['colorPath'][$bottom+1][1]-$_ENV['colorPath'][$bottom][1])/4));
			$b=floor(($_ENV['colorPath'][$bottom][2]+($nb-3*$bottom)*($_ENV['colorPath'][$bottom+1][2]-$_ENV['colorPath'][$bottom][2])/4));
*/
			$cpi=2;
			$bottom=min(floor($nb/$cpi),count($_ENV['colorPath'])-2);
			$r=floor(($_ENV['colorPath'][$bottom][0]+($nb-$cpi*$bottom)*($_ENV['colorPath'][$bottom+1][0]-$_ENV['colorPath'][$bottom][0])/($cpi+1)));
			$g=floor(($_ENV['colorPath'][$bottom][1]+($nb-$cpi*$bottom)*($_ENV['colorPath'][$bottom+1][1]-$_ENV['colorPath'][$bottom][1])/($cpi+1)));
			$b=floor(($_ENV['colorPath'][$bottom][2]+($nb-$cpi*$bottom)*($_ENV['colorPath'][$bottom+1][2]-$_ENV['colorPath'][$bottom][2])/($cpi+1)));
			$colors[]=new Color($r,$g,$b,0);
			$nb++;
		}

	dbClose($db);
	require_once INCLUDE_DIR."/artichow/Pie.class.php";
	$graph = new Graph(400,350);

	//$graph->setAntiAliasing(TRUE);
	$graph->title->set(cfCaption('calResourcePieTitle').' '.cfCaption('calTotal',array_sum($accessedResources)));
	$graph->title->setFont(new Tuffy(12));

	$graph->setBackgroundGradient(new LinearGradient(new Color(230, 230, 250),new Color(255, 255, 255),0));

	if(count($accessedResources)){
		$plot = new Pie(array_values($accessedResources), $colors);
		$plot->setCenter(0.4, 0.60);
		$plot->setSize(0.7, 0.7);
		$plot->set3D(20);
		$plot->setLabelMinimum(1);
		$plot->setStartAngle(200);

		$plot->setLegend(array_values($resources));
		//$plot->legend->setBackgroundColor(new Color(255, 255, 255,125));
		$plot->legend->setPosition(1.3,0.15);
		$plot->legend->shadow->setSize(0);
		$graph->add($plot);
	}
	$graph->draw();
	exit;
}

/**
 * @desc Single user real-time history
 *
 */
function statsSingleUserHistory(){
	global $db,$dbId;

	require_once(INCLUDE_DIR.'explorerFunctions.php');

	// Get resources and accounts id/name correspondance
	$resources=array(); foreach (dbArrayQuery('SELECT * FROM resource',$db) as $value) $resources[$value['id']]=$value['name'];
	$accounts=array();  foreach (dbArrayQuery('SELECT * FROM account',$db)  as $value) $accounts[$value['id']] =$value['name'];
	$accounts['standalone']=cfUTF8Decode(cfCaption('directResourceAccess'));

	$resourcesConfigs=cfResourcesGetAll();
	$usersConfigs=cfMGetVar('weezoUsers');


	// Filter on connection id
	createViewDetail($dbId);

	// If content is logged and history is viewed by administrator or from application, include content
	if(cfGGetVar('historyLogViewedContent') && (cfIsInApp() || cfUGetVar('administrator'))){
		$result=dbArrayQuery('SELECT baseView.id as id, baseView.ip as ip, baseView.pseudo as pseudo, baseView.ts as ts, baseView.accountId as accountId, baseView.resourceId as resourceId, contentAccess.id AS contentId, contentAccess.content AS content, contentAccess.ts as contentTs FROM baseView LEFT OUTER JOIN contentAccess ON baseView.id=contentAccess.connectionId AND baseView.resourceId=contentAccess.resourceId ORDER BY contentTs DESC',$db);
		$fullHistory=true;
	}
	// Else, show only basic info
	else{
		$result=dbArrayQuery('SELECT * FROM baseView GROUP BY ip ORDER BY id DESC',$db);
		$fullHistory=false;
	}

	$currentId=false;
	$currentResourceId=false;
	$hasContent=false;
	$firstConnection=true;

	foreach ($result as $k=>$record){
		// New connection
		if($record['id']!=$currentId){
			$currentId=$record['id'];
			$currentResourceId=false;
			$hasContent=false;

			// New connection header and collapse button
			if(isset($usersConfigs[$record['accountId']])) $src='/gfx/icons/'.$usersConfigs[$record['accountId']]['icon'];
			else $src='/gfx/icons/blankIcon.gif';
			$l='&nbsp;<img src="'.$src.'" class="themeIcon" alt="">';
			$l.=cfFixedDate($record['ts'],cfCaption('_FULL_DATE_FORMAT')).' &nbsp; &nbsp; ';
			$l.='<a href="'.cfIPLocatorLink($record['ip']).'" target="_blank" class="link">'.$record['ip'].'</a> &nbsp; &nbsp; ';
			$l.='('.cfUTF8Encode($accounts[$record['accountId']]);
			if($record['pseudo']) $l.=' - '.cfUTF8Encode($record['pseudo']);
			$l.=')';

			$r='';

			// Chat button
			if(isset($chatResourceFilename)) $r.='&nbsp;'.outButton(cfCaption('chatDoChat'),'javascript:chatNow()',outIcon('chat'));

			echo outFrameHeaderTable('frame3Header',$l,$r);

			echo '<div id="connectionInnerFrame">';

			// Total time
			if(isset($record['totalDuration']) && ($totalTime=$record['totalDuration'])){
				$sec=$totalTime%60; $totalTime=floor($totalTime/60);
				$min=$totalTime%60; $totalTime=floor($totalTime/60);
				echo '<b>'.cfCaption('calTotalTime').cfCaption('genSeparator').'</b>'.cfCaption('calHourMinSec',cfUTF8Encode($totalTime),cfUTF8Encode($min), cfUTF8Encode($sec)).'<br/><br/>';
			}

			//echo '<div id="cnxDiv'.$record['id'].'" style="'.(($infoType=='single')?'':'display:none').'">';
			$firstConnection=false;
		}

		// New resource access
		if($record['resourceId']!=$currentResourceId){
			if($hasContent!==false) echo ' </div>'; // Close content list
			if($currentResourceId!==false) echo "</div>\n";
			$currentResourceId=$record['resourceId'];
			$hasContent=false;
			echo outDivFrame('frame4','id=resFrame_'.$currentResourceId).'<div class="frame4Header">';
			// Resource icon
			if(isset($resourcesConfigs[$currentResourceId])) $iconSrc=$resourcesConfigs[$currentResourceId]['definition']['resourceIconSmall'];
			else $iconSrc=outIcon('broken');
			echo outImage($iconSrc,false,'class="themeIcon"');

			// Respource name
			if(isset($resourcesConfigs[$currentResourceId])) echo cfUTF8Encode($resourcesConfigs[$currentResourceId]['name']).'</div>';
			else if(isset($resources[$currentResourceId])) echo cfUTF8Encode($resources[$currentResourceId]).'</div>';
			else echo '?</div>';

			if($fullHistory && $record['content']){
				echo '<div class="historyContent">';
				$hasContent=true;
			}

		}

		// New content
		if($fullHistory && $record['content']){
			if(!$hasContent) echo '<div class="historyContent">';
			$hasContent=true;
			echo '<span class="historyContentDate">'.cfFixedDate($record['contentTs'],cfCaption('_HMS_DATE_FORMAT')).'</span>';
			echo '<li class="historyContentList"';
			if((cfIsInApp() && strpos($record['content'],'/')!==false)) {
				echo ' onclick="shell(this)" alt="'.$record['contentId'].'" onmouseover="tooltip(this)">';

				if(($ft=efFileType($record['content']))=='image' || $ft=='video')
					echo '<img name="ctn" class="ctn" alt="">';
				else
					echo outImage(efIcon($record['content']),false,' name="ctn" class="ctn"');
			}
			else echo '>';
			echo '<span class="histoSpacer">'.cfUTF8Encode($record['content']).'</span>';
			echo "</li>\n";
		}
	}
	if($hasContent!==false) echo "</div>\n"; // Close content list
	if($currentResourceId) echo "</div>\n"; // Close resource
	if($currentId) {
		echo outDivFrame('frame4',false,'padding:6px;margin-top:10px').outImageIcon('new').'<b>'.cfCaption('loginConnectButton').cfCaption('genSeparator').'</b>'.cfFixedDate($record['ts'],cfCaption('_FULL_DATE_FORMAT')).'</div>';
		echo "</div>\n"; // connectionInnerFrame
	}
}


require_once(INCLUDE_DIR.'databaseFunctions.php');
require_once(INCLUDE_DIR.'outputFunctions.php');


/*
 * Verify access rights
*/
if(!isset($_ENV['configurationEnvironment'])) $_ENV['configurationEnvironment']='browser';
if(($_SERVER['REMOTE_ADDR']=='127.0.0.1' && cfIsInApp()) ||
	(cfUGetVar('administrator')==true && cfGGetVar('remoteAdministrationAuthorized'))){}
else
	outDisplayErrorPage(cfcaption('errorRemoteAdministrationNotAuthorized'));


// Open stats database
$db=dbOpenDB();

// Select history or statistics
if(!isset($displayType)){
	if(basename($_SERVER['SCRIPT_NAME'])=='showStats.php') $displayType='stats'; else $displayType='history';
}

/*
 ***************************************************************************************************************************
 * Init default data
 ***************************************************************************************************************************
 */
$selectedMonth = mktime(0,0,1,date('n'),1,date('Y'));
$intStart = mktime(0,0,1,date('n',time()-6*24*3600),date('j',time()-6*24*3600),date('Y',time()-6*24*3600));
$intLength ='week';


if(cfIsResource()){
	cfRInitVar('showLocalhost',true);
	cfRInitVar('infoType','synth');
	$showLocalhost = cfRGetVar('showLocalhost');
}
else{
	$showLocalhost = true;
}

// Initialize $infoType for stats display
if(!@$infoType) {
	$infoType=cfRGetVar('infoType');
	if(!$infoType) $infoType = 'synth';
}

$displayedGraph='connections';
@setlocale(LC_TIME , cfCaption('_ISO_639'));


/*
 ***************************************************************************************************************************
 * Content thumbnails (from tooltip or viewed files list)
 ***************************************************************************************************************************
 */
if(isset($_GET['thumbnail']) && isset($_GET['contentId'])){
	$completeFilename=dbSingleQuery("SELECT content FROM contentAccess WHERE id='".dbEscapeString($_GET['contentId'],$db)."'",$db);
	require_once(INCLUDE_DIR.'explorerFunctions.php');

	// Limit size
	if(!isset($_GET['w']))$_GET['w']=cfGGetVar('historyThumbnailsWidth'); $_GET['w']=min($_GET['w'],STATS_MAX_THUMBNAIL_WIDTH);
	if(!isset($_GET['h']))$_GET['h']=cfGGetVar('historyThumbnailsHeight');; $_GET['h']=min($_GET['h'],STATS_MAX_THUMBNAIL_HEIGHT);

	// Tooltip's thumbnail: don't cache (bug WA - don't know why some images got cached with another one's id)
	if($_GET['w']>99) WHeaders::noCache(); else WHeaders::cache($completeFilename);

	// Images thumbnails
	if(efFileType($completeFilename)=='image'){
		header('Content-Type: image/jpeg');
		cfCreateResizedJPG($completeFilename,0,0,'',$_GET['w'],$_GET['h'],false,90);
	}

	// Videos thumbnails
	if(efFileType($completeFilename)=='video'){
		header('Content-Type: image/jpeg');
		efMakeVideoThumbnail($completeFilename,'',$_GET['w'],$_GET['h'],90,10);
	}
	exit;
}


/*
 ***************************************************************************************************************************
 * Process POST AND GET ASYNC commands
 *
 * Format:
 * - data2: extra data (depends on command)
 * - data3: command (setLogViewedContent, setlogViewedContentDisplayThumbnails, monitorLastViewedFile, getFileInfo, shell, chatNow
 * - data4: extra data (depends on command)
 * - data5: tracked user's session id (user tracking) or graph type (stats: connections, visitors, resources)
 ***************************************************************************************************************************
 */
if(cfIsAsync() && isset($_POST['data2']) && isset($_POST['data3'])){
	// XML response header
	cfAsyncHeader();

	// toggle historyLogViewedContent general value
	if($_POST['data3']=='setLogViewedContent'){
		if(cfIsInApp() || cfGGetVar('advancedAdministration')){
			$historyLogViewedContent=($_POST['data2']=='true')?true:false;
			cfGUpdateVar('historyLogViewedContent',$historyLogViewedContent);
			sleep(1);
		}
		// Reload page for single user tracking
		if($displayType=='history') echo cfAsyncXMLJSaction('wl.goURL()');
	}

	// toggle logViewedContentDisplayThumbnails general value
	elseif($_POST['data3']=='setlogViewedContentDisplayThumbnails'){
		if(cfIsInApp() || cfGGetVar('advancedAdministration')){
			$logViewedContentDisplayThumbnails=($_POST['data2']=='true')?true:false;
			cfGUpdateVar('logViewedContentDisplayThumbnails',$logViewedContentDisplayThumbnails);
		}
	}

	// Last viewed file information (single history, left panel)
	elseif($_POST['data3']=='monitorLastViewedFile'){
		// Last async updated content id (-1 on 1st async update)
		$lastViewedContentId=$_POST['data4'];

		// Get all viewed resource
		$query='SELECT resourceId FROM resAccess WHERE connectionId = '.dbEscapeString($_POST['data2'],$db);

		foreach (dbArrayQuery($query,$db) as $resId){
			$rc=new WResConfig($resId['resourceId']);
			if(!$rc->isValid()) continue;
			// Insert new resources or reorder resources
			echo cfAsyncXMLJSaction('historyAsyncInsertResource("'.$rc->id().'","'.$rc->icon16().'","'.addslashes(cfUTF8Encode($rc->name())).'")');
		}

		// If viewed files are not logged (and not displayed), just update resources access, not last viewed file
		if(!cfGGetVar('historyLogViewedContent')) die(cfAsyncFooter());

		// Get last viewed file
		$query='SELECT id,resourceId,content,ts FROM contentAccess WHERE connectionId = '.dbEscapeString($_POST['data2'],$db).' AND content!="" ORDER BY ts DESC';
		$res=dbArrayQuery($query,$db);

		// If not any content viewed, set lastViewedContentId to 0 (!=-1) so list will be updated
		if(!count($res) && $lastViewedContentId==-1) echo cfAsyncXMLJSaction('lastViewedContentId=0');

		// If any found
		if(count($res)){
			$contentId=$res[0]['id'];
			$resourceId=$res[0]['resourceId'];
			$completeFilename=$res[0]['content'];
			$content=''; $title='';


			/**
			 * If new content ($_POST['data4'] contains last viewed content's id, or -1 if first async request after sync refresh)
			 */
			if($contentId!=$lastViewedContentId){
				/**
				 * Last viewed file thumbnail
				 */

				// Title
				$title='<div class="frame2Header">'.cfCaption('historyLastViewedFile').'</div><br><div class="historyContentList" style="cursor:pointer;white-space:nowrap;overflow:hidden" onclick="shell(this)" alt="'.$contentId.'" title="'.cfUTF8Encode($completeFilename).'">'.cfUTF8Encode(basename($completeFilename)).'</div>';

				// Thumbnail/content
				if(is_file($completeFilename)){
					require_once(INCLUDE_DIR.'explorerFunctions.php');
					require(INCLUDE_DIR.'fileInfoFunctions.php');

					// Set path to computerRoot to access all files
					$prevPath=cfRGetVar('path'); cfRSetVar('path','computerRoot');

					$fi=fiGetInfo($completeFilename,true,true,true);
					if(!isset($fi['thumbnailURLW'])) $fi['thumbnailURLW']=STATS_MAX_THUMBNAIL_WIDTH;
					if(!isset($fi['thumbnailURLH'])) $fi['thumbnailURLH']=STATS_MAX_THUMBNAIL_HEIGHT;

					// Image or video: display thumbnail
					if(efFileType($completeFilename)=='image' || efFileType($completeFilename)=='video'){
						$src='/local/showHistory.php?thumbnail=1&contentId='.$contentId.'&w='.$fi['thumbnailURLW'].'&h='.$fi['thumbnailURLH'];
						$content='<center style="margin-top:0.3em"><div class="photoFrame"><img alt="" src="'.$src.'" style="width:'.$fi['thumbnailURLW'].'px;height:'.$fi['thumbnailURLH'].'px"></div></center>';
					}

					// Other file types:
					else{
						if(isset($fi['thumbnailURL']) && substr($fi['thumbnailURL'],0,5)=='code:'){
							$content=substr($fi['thumbnailURL'],5);
						}
						elseif(isset($fi['extra'])){
							foreach ($fi['extra'] as $v) if(isset($v['c']) && isset($v['v']) && $v['c'] && $v['v']) $content.='<b>'.($v['c']).cfCaption('genSeparator').'</b> '.cfUTF8Encode($v['v']).'<br>';
						}
						if($content) $content='<div class="frame3">'.$content.'</div>';
					}

					// Restore path
					cfRSetVar('path',$prevPath);
				}

				echo cfAsyncXMLInnerHTMLbyId($title.$content,'lvfTN');
				echo cfAsyncXMLJSaction('lastViewedContentId="'.$contentId.'"');

				/**
				 * Update history list
				 */
				// Skip if first display
				if($lastViewedContentId!=-1){

					// Get resources and accounts id/name correspondance
					$resources=array(); foreach (dbArrayQuery('SELECT * FROM resource',$db) as $value) $resources[$value['id']]=$value['name'];
					$accounts=array();  foreach (dbArrayQuery('SELECT * FROM account',$db)  as $value) $accounts[$value['id']] =$value['name'];
					$resourcesConfigs=cfResourcesGetAll();

					// Go to last non-displayed resources/contents
					for($i=1;$i<count($res);$i++) if($res[$i]['id']==$lastViewedContentId) break;
					// Time-reverse browse updated non-displayed resources/contents
					for($i--;$i>=0;$i--){
						// New resource
						if(!isset($res[$i+1]) || $res[$i]['resourceId']!=$res[$i+1]['resourceId']){
							$resourceId=$res[$i]['resourceId'];
							// Resource icon
							if(isset($resourcesConfigs[$resourceId]['resourceIcon'])) $iconSrc=$resourcesConfigs[$resourceId]['resourceIcon'];
							else $iconSrc=outIcon('broken');
							// Resource name
							if(isset($resourcesConfigs[$resourceId])) $resourceName=cfUTF8Encode($resourcesConfigs[$resourceId]['name']);
							else $resourceName=cfUTF8Encode(@$resources[$resourceId]);
							// Update script
							echo cfAsyncXMLJSaction('historyAsyncInsertResource("'.$resourceId.'","'.$iconSrc.'","'.$resourceName.'",1)');
						}
						// New content
						// icon
						if(is_file($completeFilename)){
							// If has thumbnail, send 'tn' as icon src. Actual src will be built by js, from id
							if(efFileType($completeFilename)=='image' || efFileType($completeFilename)=='video') $iconSrc='tn';
							else $iconSrc=efIcon($completeFilename);
						}
						else $iconSrc='false';
						echo cfAsyncXMLJSaction('historyAsyncInsertContent("'.$res[$i]['id'].'","'.$iconSrc.'","'.cfUTF8Encode($res[$i]['content']).'","'.cfFixedDate($res[$i]['ts'],cfcaption('_HMS_DATE_FORMAT')).'")');
					}
				}
			}
		}
	}

	// Tooltip
	elseif($_POST['data3']=='getFileInfo'){
		require_once(INCLUDE_DIR.'explorerFunctions.php');
		require(INCLUDE_DIR.'fileInfoFunctions.php');

		// Get source filename from content id
		$completeFilename=dbSingleQuery("SELECT content FROM contentAccess WHERE id='".dbEscapeString($_POST['data2'],$db)."'",$db);

		// Create a resource context so fiGetInfo can be used
		if(!isset($_SESSION['activeResourceId'])) cfRCreateFakeResource();

		// Set path to computerRoot to access all files
		$prevPath=cfRGetVar('path'); cfRSetVar('path','computerRoot');

		switch (efFileType($completeFilename)){
			// Image or video: just display thumbnail
			case 'image':
			case 'video':

				// Show thumbnail in tooltip
				$fi=fiGetInfo($completeFilename,true,false,true);

				$fi['thumbnailURL']='/local/showHistory.php?thumbnail=1&contentId='.$_POST['data2'].'&w='.$fi['thumbnailURLW'].'&h='.$fi['thumbnailURLH'];
				unset($fi['footer']);
				unset($fi['cTime']);
				unset($fi['mTime']);

				echo cfAsyncXMLJSaction('tooltipSetProperties('.fiPHPArrayToJSArray($fi).')');
				break;
			// Not an image or video file: use regular tooltip methods
			default:
				// Get file info tooltip content
				if(is_file($completeFilename))
					echo cfAsyncXMLJSaction('tooltipSetProperties('.fiPHPArrayToJSArray(fiGetInfo($completeFilename,true,false,true)).')');
				else
					echo cfAsyncXMLJSaction('tooltipDestroy()');
		}

		// Restore path
		cfRSetVar('path',$prevPath);
	}
	// shell: open viewed file on local computer
	elseif($_POST['data3']=='shell' && $_SERVER['REMOTE_ADDR']=='127.0.0.1'){
		// Get source filename from content id
		$completeFilename=dbSingleQuery("SELECT content FROM contentAccess WHERE id='".dbEscapeString($_POST['data2'],$db)."'",$db);
		// View viewable file
		require_once(INCLUDE_DIR.'explorerFunctions.php');
		$ft=efFileType($completeFilename);
		$fe=cfFileExtension($completeFilename);
		if($ft=='text' || $ft=='audio' || $ft=='video' || $ft=='image' || $fe=='pdf' || $fe=='doc' || $fe=='ppt' || $fe=='xls');
		else $completeFilename=dirname($completeFilename);

		customShellExecute($completeFilename,'open');
		//cfExecFork('explorer','SW_SHOWNORMAL','"'.str_replace('/','\\',$completeFilename).'"');
	}
	// Chat with user
	elseif($_POST['data3']=='chatNow' && $_SERVER['REMOTE_ADDR']=='127.0.0.1' && is_file(cfAppDataDir().'/'.$_POST['data2'])){
		cfServerSendCmd('showChatWindow',array('resourceConfigFilename'=>$_POST['data2'],'destSessionId'=>$_POST['data4']));
	}

	// Clear history/stats database
	elseif($_POST['data3']=='clearHistory'){
		if(cfIsInApp() || cfGGetVar('advancedAdministration')) {
			dbClose($db);
			dbResetDB();
			$db=dbOpenDB();
		}
		echo cfAsyncJSaction('wl.goURL()');
		cfAsyncFooter();
	}



	// Disconnected user ?
	if($displayType=='history' && @$_POST['data5'] && $_POST['data5']!='null' && !file_exists(cfAppDataDir().'/sessionData/sess_'.$_POST['data5'])){
		$usersConfigs=cfMGetVar('weezoUsers');
		$userId=dbSingleQuery('SELECT accountId FROM connection WHERE id="'.dbEscapeString($_POST['data2'],$db).'"',$db);
		echo cfAsyncXMLJSaction('setDisconnected("'.str_replace('"','',cfCaption('chatDisconnected',cfUTF8Encode($usersConfigs[$userId]['name']))).'")');
	}

	cfAsyncFooter();
}

/*
 ***************************************************************************************************************************
 * Process POST AND GET SYNC commands
 ***************************************************************************************************************************
 */
if(isset($_POST['data3']) && isset($_POST['data4'])){
	parse_str($_POST['data4'],$data);

	if(isset($data['infoType'])) {$infoType=$data['infoType'];}
	if(isset($data['showLocalhost']) && $data['showLocalhost']=='true') {$showLocalhost=true;cfRSetVar('showLocalhost',true);}
	if(isset($data['showLocalhost']) && $data['showLocalhost']=='false') {$showLocalhost=false;cfRSetVar('showLocalhost',false);}

	// Time interval
	if(isset($data['selectedMonth']) && is_numeric($data['selectedMonth'])) $selectedMonth=$data['selectedMonth'];
	if(isset($data['intStart']) && is_numeric($data['intStart'])) $intStart=$data['intStart'];
	if(isset($data['intLength']) && is_numeric($data['intLength'])) $selectedMonth=$data['intLength'];
	if($_POST['data3']=='setMonth' && is_numeric($_POST['data2'])){
		$selectedMonth=$_POST['data2'];
		$intStart=$_POST['data2'];
		$intLength='month';
	}
	if($_POST['data3']=='intMonth' && is_numeric($_POST['data2'])){
		$intStart=$_POST['data2'];
		$intLength='month';
	}
	if($_POST['data3']=='intWeek' && is_numeric($_POST['data2'])){
		$intStart=$_POST['data2'];
		$intLength='week';
	}
	if($_POST['data3']=='intAll'){
		$intStart=0;
		$intLength='all';
	}
	// Stats: set info type (details or synth)
	if($_POST['data3']=='setInfo' && $_POST['data2']=='details' || $_POST['data2']=='synth'){
		if(cfIsResource()) cfRSetVar('infoType',$infoType);
		$infoType=$_POST['data2'];
	}
	// Include/remove localhost connections from history/stats
	if($_POST['data3']=='setLocalhost'){
		if($_POST['data2']=='true') $showLocalhost=true; else $showLocalhost=false;
		cfRSetVar('showLocalhost',$showLocalhost);
	}
}
if(isset($_POST['data5']) && $_POST['data5']!='' && $_POST['data5']!='undefined') $displayedGraph=$_POST['data5'];

if(isset($_GET['graph']) && isset($_GET['intStart']) && isset($_GET['intEnd']) && isset($_GET['intLength']) && isset($_GET['showLocalhost']) && is_numeric($_GET['intStart']) && is_numeric($_GET['intEnd'])){
	if($_GET['graph']=='Connections') drawConnections($_GET['intStart'], $_GET['intEnd'], isset($_GET['intLength']), $_GET['showLocalhost']);
	if($_GET['graph']=='Visitors') drawVisitors($_GET['intStart'], $_GET['intEnd'], isset($_GET['intLength']), $_GET['showLocalhost']);
	if($_GET['graph']=='Resources') drawResources($_GET['intStart'], $_GET['intEnd'], isset($_GET['intLength']), $_GET['showLocalhost']);
}

// Set current month
$sm=$selectedMonth;
$firstConnection=dbSingleQuery("SELECT min(ts) FROM connection",$db);

if($intStart==0){
	$intLength='all';
	$intStart=$firstConnection;
	$intEnd=time();
}
else{
// Set interval end
	switch ($intLength){
		case 'day':
			$intEnd=mktime(23,59,59,date('n',$intStart),date('j',$intStart),date('Y',$intStart));
			break;
		case 'week':
			$length=6*24*3600; $length+=(date('h',$intStart)-date('h',$intStart+$length))*3600; // Compute week length including potential clock adjustment
			$intEnd=mktime(23,59,59,date('n',$intStart+$length),date('j',$intStart+$length),date('Y',$intStart+$length));
			break;
		case 'month':
			$intEnd=mktime(23,59,59,date('n',$intStart+(date('t',$intStart)-1)*24*3600),date('j',$intStart+(date('t',$intStart)-1)*24*3600),date('Y',$intStart+(date('t',$intStart)-1)*24*3600));
			break;
		default:
			$intEnd=mktime(23,59,59,date('n',time()+6*24*3600),date('j',time()+6*24*3600),date('Y',time()+6*24*3600));
	}
}

/*
 ***************************************************************************************************************************
 * Javascript functions
 ***************************************************************************************************************************
 */
?>
<style type="text/css">
.historyContentList{position:relative;line-height:26px;}
.histoSpacer{margin-left:30px}
.historyContentList img{position:absolute;left:0;top:-4px}
</style>
<script type="text/javascript">
var lastViewedContentId=-1;
function displayConnections(){
	D.comForm.data5.value="connections";
	dgi("selectedGraph").src=PHP_SELF+"?graph=Connections&intStart=<?php echo $intStart;?>&intEnd=<?php echo $intEnd;?>&intLength=<?php echo $intLength;?>&showLocalhost=<?php echo $showLocalhost;?>&noSt=true";
}
function displayVisitors(){
	document.comForm.data5.value="visitors";
	dgi("selectedGraph").src=PHP_SELF+"?graph=Visitors&intStart=<?php echo $intStart;?>&intEnd=<?php echo $intEnd;?>&intLength=<?php echo $intLength;?>&showLocalhost=<?php echo $showLocalhost;?>&noSt=true";
}
function displayResources(){
	document.comForm.data5.value="resources";
	dgi("selectedGraph").src=PHP_SELF+"?graph=Resources&intStart=<?php echo $intStart;?>&intEnd=<?php echo $intEnd;?>&intLength=<?php echo $intLength;?>&showLocalhost=<?php echo $showLocalhost;?>&noSt=true";
}
function monitorLastViewedFile(){
	fAS("<?php echo (isset($dbId))?$dbId:'-';?>",'monitorLastViewedFile',lastViewedContentId,"<?php echo (isset($_GET['sessionId']))?$_GET['sessionId']:'';?>",'async');
}

var contentThumbnailsShown=<?php if(cfGGetVar('logViewedContentDisplayThumbnails')) echo 1; else echo 0; ?>;
tooltipDelay=1;

function setMonth(ts){
	fAS(ts,'setMonth');
}
function intMonth(ts){
	fAS(ts,'intMonth');
}
function intWeek(ts){
	fAS(ts,'intWeek');
}
function intAll(ts){
	fAS(false,'intAll');
}
function setInfo(inf){
	fAS(inf,'setInfo');
}
function setLocalhost(lh){
	if(lh) fAS('true','setLocalhost'); else fAS('false','setLocalhost')
}
function statsToggleCollapse(anim){
	if(anim.step==0 && !anim.reverse) contentShowThumbnails(anim.item.id)
}
function clearHistory(){
	if(confirm("<?php echo cfCaption('clearHistory');?>")) fAS(true,'clearHistory',false,false,'async');
}
function shell(val){
	fAS(val.alt,'shell',1,null,'async');
}
function chatNow(){
	fAS("<?php echo (isset($chatResourceFilename))?$chatResourceFilename:'-';?>",'chatNow',"<?php echo (isset($_GET['sessionId']))?$_GET['sessionId']:'-';?>",null,'async');
}
function setLogViewedContent(val){
	fAS((val?'true':'false'),'setLogViewedContent',1,null,'async');
}
function setlogViewedContentDisplayThumbnails(val){
	contentThumbnailsShown=val;
	fAS((val?'true':'false'),'setlogViewedContentDisplayThumbnails',1,null,'async');
	contentToggleThumbnails(val);
}
function contentToggleThumbnails(show){
	var ctns=document.getElementsByName("ctn");
	for(var i=0;i<ctns.length;i++){
		if(show){
			if(show && ctns[i].parentNode.parentNode.parentNode.parentNode.style.display!="none"){
				if(!ctns[i].src) ctns[i].src="<?php echo $_SERVER['SCRIPT_NAME']?>?thumbnail=1&contentId="+ctns[i].parentNode.alt
				ctns[i].style.display="inline";
			}
		}
		else ctns[i].style.display="none";
	}
}
function historyAsyncInsertResource(rId,iconSrc,resourceName,moveTop){
	var iht='';
	if(dgi('resFrame_'+rId)){
		if(!moveTop) return;
		iht=dgi('resFrame_'+rId).innerHTML;
		removeNode(dgi('resFrame_'+rId));
	}

	subNode=D.createElement('div');
	subNode.id='resFrame_'+rId;
	subNode.setAttribute('class','frame4');subNode.setAttribute('className','frame4');

	subNode.innerHTML=(iht)?iht:'<div class="frame4Header"><img src="'+iconSrc+'" class="themeIcon">'+resourceName+'</div><div class="historyContent"></div>';

	if(dgi('connectionInnerFrame').firstChild)
		dgi('connectionInnerFrame').insertBefore(subNode,dgi('connectionInnerFrame').firstChild);
	else
		dgi('connectionInnerFrame').appendChild(subNode);
}
function historyAsyncInsertContent(contentId,iconSrc,contentName,contentTs){
	var tsNode=document.createElement('span');
	tsNode.setAttribute('class','historyContentDate');tsNode.setAttribute('className','historyContentDate');
	tsNode.innerHTML=contentTs;

	var subNode=document.createElement('li');
	subNode.setAttribute('class','historyContentList');subNode.setAttribute('className','historyContentList');
	subNode.setAttribute('alt',contentId);
	subNode.setAttribute('onclick',function(){shell(this)});
	subNode.setAttribute('onmouseover',function(){tooltip(this)});
	var ih='';
	if(iconSrc){
		if(iconSrc=='tn'){
			if(contentThumbnailsShown){
				ih+='<img name="ctn" class="ctn" alt="" style="display:inline;height:1px" onload="this.style.height=\'\'" src="<?php echo $_SERVER['SCRIPT_NAME']?>?thumbnail=1&contentId='+contentId+'">';
			}
			else
				ih+='<img name="ctn" class="ctn" alt="" src="<?php echo $_SERVER['SCRIPT_NAME']?>?thumbnail=1&contentId='+contentId+'">';
		}
		else ih+='<img name="ctn" class="ctn" alt="" src="'+iconSrc+'" onmouseover="tooltip(this)"'+((contentThumbnailsShown)?' style="display:inline"':'')+'>';
	}
	ih+='<span class="histoSpacer">'+contentName+'</span>';

	subNode.innerHTML=ih;
	var par=dgi('connectionInnerFrame').firstChild.firstChild.nextSibling;
	if(par.nodeType==1) {
		par.insertBefore(subNode,par.firstChild);
		par.insertBefore(tsNode,par.firstChild);
	}
}

function tooltipGetContent(nodeId){
	fAS(dgi(nodeId).getAttribute("alt"),'getFileInfo',1,null,'async');
	D.comForm.target='';
	return '<img id="tooltipLoading" src="">';
}
function contentShowThumbnails(connectionId){
	if(!contentThumbnailsShown) return;
	if(connectionId) connectionIdNode=dgi(connectionId);
	var ctns=document.getElementsByName("ctn");
	for(var i=0;i<ctns.length;i++){
		if(!connectionId || ctns[i].parentNode.parentNode.parentNode.parentNode==connectionIdNode){
			if(!ctns[i].src) {
				ctns[i].src="<?php echo $_SERVER['SCRIPT_NAME']?>?thumbnail=1&contentId="+ctns[i].parentNode.alt
				ctns[i].style.display="inline";
			}
			else ctns[i].style.display="inline";
		}
	}
}
var disconnected=0;
function setDisconnected(capt){
	if(disconnected) return;
	disconnected=1;
	var subNode=document.createElement('div');
	subNode.setAttribute('class','warning');subNode.setAttribute('className','warning');
	subNode.innerHTML=capt;
	if(dgi('connectionInnerFrame').firstChild)
		dgi('connectionInnerFrame').insertBefore(subNode,dgi('connectionInnerFrame').firstChild);
	else
		dgi('connectionInnerFrame').appendChild(subNode);
}
function init(){
	contentShowThumbnails();
	W.setInterval('monitorLastViewedFile()',<?php echo MONITOR_LAST_VIEWED_FILE_INTERVAL;?>);
	monitorLastViewedFile();
}
function resizeFrames(){
	var f2=dgi('rightFrame2'),ncd=dgi('noConnectionDiv'),msf=dgi('mainStatsFrame');
	if(W.winMe) alignBottom(msf,winMe,5);
	if(W.winMe)	setActualHeight(f2,winMe.getInnerHeight()-5-actualOffsetTop(f2)-getElementStyle(f2,'margin-top',false)-getElementStyle(f2,'margin-bottom',false))
	if(dgi('lvfTN')) setActualHeight(dgi('lvfTN'),f2.offsetHeight);
	//if(ncd && W.winMe) alignBottom(ncd,winMe,5);
}
<?php if($infoType=='single') echo 'document.body.onload=init;'; else echo 'document.body.onload=resizeFrames;'?>
connect(W,'onresize',resizeFrames);

</script>
<style type="text/css">
.ctn{vertical-align:middle;margin-right:0.3em;margin-bottom:2px;display:none}
</style>
<?php
outInsertStandardComForm($_SERVER['PHP_SELF'],false,false,false,'infoType='.$infoType.'&amp;intStart='.$intStart.'&amp;intLength='.$intLength.'&amp;selectedMonth='.$selectedMonth.'&amp;showLocalhost='.(($showLocalhost)?'true':'false'),$displayedGraph);

/*
 ***************************************************************************************************************************
 * Display Tabs
 ***************************************************************************************************************************
 */
echo outDivFrame('frame1','id="mainStatsFrame"');

/**
 * Title
 */
if($displayType=='stats')
	$l=outImage(outicon('stats'),false,false,'margin-right:1em').cfCaption('genStats');
else
	$l=outImage(outicon('log'),false,'class="themeIcon"').cfCaption('genHistory');

// Clear history
if((cfIsInApp() || cfGGetVar('advancedAdministration')) && $infoType=='single'){
	$r='<span>'.cfCaption('historyLogContent').'</span><input type="checkbox" onchange="setLogViewedContent(this.checked)" onclick="setLogViewedContent(this.checked)"'.((cfGGetVar('historyLogViewedContent'))?' checked="checked"':'').' style="margin-right:1em;margin-left:0.5em">';
	$r.=cfCaption('bookmarksDisplayThumbnails').'<input type="checkbox" onchange="setlogViewedContentDisplayThumbnails(this.checked)" onclick="setlogViewedContentDisplayThumbnails(this.checked)"'.((cfGGetVar('logViewedContentDisplayThumbnails'))?' checked="checked"':'').' style="margin-left:0.5em">';
}
elseif((cfIsInApp() || cfGGetVar('advancedAdministration')) && $infoType!='single')
	$r=outButton(cfCaption('clearHistory'),'javascript:clearHistory()',outIcon('cancel'));
else
	$r='';

echo outFrameHeaderTable("frame1Header",$l,$r);

// Frame used for left/right panels
echo outTableTransparent('frame1','table-layout:fixed');
echo "<tr>\n";

// Left panel
if($displayType=='stats' || $infoType=='list' || cfGGetVar('historyLogViewedContent')) echo "<td style=\"vertical-align:top;width:200px\">\n"; else echo "<td style=\"vertical-align:top\">\n";

/**
 * Last viewed file thumbnail
 */
if($infoType=='single' && cfGGetVar('historyLogViewedContent')) {
	echo outDivFrame('frame2',' id="lvfTN"');
	echo '<div class="frame2Header">'.cfCaption('historyLastViewedFile').'</div><br><div class="historyContentList" style="cursor:pointer;white-space:nowrap;overflow:hidden">-</div>';
	echo '<div class="frame3" style="height:120px"></div>';
	echo '</div>';
}

/*
 * Information selection
 */
if($displayType=='stats'){
	echo outDivFrame('frame2');
	echo '<div class="frame2Header">'.cfCaption('genInfos')."</div>\n";
	echo '<input type="radio" name="type" value="synth"'.(($infoType=='synth')?' checked="checked"':'').' onChange="setInfo(this.value)" onClick="setInfo(this.value)">'.cfCaption('calSynth').'<br />';
	echo '<input type="radio" name="type" value="details"'.(($infoType!='synth')?' checked="checked"':'').' onChange="setInfo(this.value)" onClick="setInfo(this.value)">'.cfCaption('genDetails');
	echo "</div>\n";
}


/*
 * Interval selection
 */
if($infoType!='single'){
	echo outDivFrame('frame2');
	echo '<div class="frame2Header">'.cfCaption('genInterval')."</div>\n";
		echo '<center>';
		@setlocale(LC_TIME, cfCaption('_ISO_639'));
		$firstDayOfMonth=mktime(0,0,1,date('n'),1,date('Y'));
		$day=$sm; while (date('w',$day)!=1) $day+=3600*24;
		$offset=(date('w',$sm)==0)?7:date('w',$sm);

		// Months drop box
		$month=mktime(0,0,1,date('n',$firstConnection),1,date('Y',$firstConnection));
		if($month!=$firstDayOfMonth){
			echo '<select name="selectMonth" id="selectMonth" size="1" onChange="setMonth(this.value)">'."\n";
			while ($month<=$firstDayOfMonth){
				echo '<option value="'.$month.'"'.(($month==$sm)?' selected':'').'>'.cfUTF8Encode(ucfirst(strftime('%B %Y',$month))).'</option>';
				if(date('n',$month)==12) $month=mktime(0,0,1,1,1,date('Y',$month)+1); else $month=mktime(0,0,1,date('n',$month)+1,1,date('Y',$month));
			}
			echo "</select><br/>\n";
		}

		// Calendar header
		echo '<table id="calendar" class="frame3" style="text-align:center;table-layout:fixed; cursor:pointer" cellspacing="0" cellpadding="0">';
		echo '<tr class="frame3Header" onmouseover="dgi(\'calendar\').style.background=\'#DDD\'" onmouseout="dgi(\'calendar\').style.background=\'\'" onclick="intMonth(\''.$sm.'\')">';
		for($i=0;$i<7;$i++) {echo '<td style="width:20px">'.cfUTF8Encode(strtoupper(substr(strftime('%A',$day),0,1))).'</td>'; $day+=3600*24;}
		echo '</tr><tr onmouseover="this.style.background=\'#DDD\'" onmouseout="this.style.background=\'\'" onclick="intWeek(\''.mktime(0,0,1,date('n',$sm-($offset-1)*24*3600),date('j',$sm-($offset-1)*24*3600),date('Y',$sm-($offset-1)*24*3600)).'\')">';

		// Calendar body (days)
		$day=$sm;
		for($i=1;$i<$offset;$i++) echo '<td>&nbsp;</td>';
		$position=$offset-1;
		for($i=1;$i<date('t',$sm)+1;$i++){
			if($position==7) {
				$position=0;
				if($firstDayOfMonth!=$sm || $i<=date('j')) echo '</tr><tr onmouseover="this.style.background=\'#DDD\'" onmouseout="this.style.background=\'\'" onclick="intWeek(\''.mktime(0,0,1,date('n',$sm),$i,date('Y',$sm)).'\')" style="">';
				else echo '</tr><tr style="cursor:default">';
			}
			$position++;
			if($firstDayOfMonth!=$sm || $i<=date('j')){
				echo '<td class="cell">'.$i.'</td>';
			}
			else echo '<td class="dcell">'.$i.'</td>';
		}
		echo '</tr></table>';

		// Current week button
		echo outButton(cfCaption('calCurrentWeek'),'javascript:intWeek(\''.mktime(0,0,1,date('n',time()-6*24*3600),date('j',time()-6*24*3600),date('Y',time()-6*24*3600)).'\')');
		echo '<br/>';
		// Total button
		echo outButton(cfCaption('genAll'),'javascript:intAll(\'\')');
		echo'<br>';
	echo "</center></div>\n";

	/*
	 * Options
	 */
	echo outDivFrame('frame2');
	echo '<div class="frame2Header">'.cfCaption('genOptions')."</div>\n";
	// Localhost connections
	echo '<input type="checkbox" onchange="setLocalhost(this.checked)" onclick="setLocalhost(this.checked)"'.((!$showLocalhost)?'':' checked="checked"').' style="margin-right:1em"><span style="position:absolute;width:150px;">'.cfCaption('calLocalhost').'</span><br/><br/>';
	// log content
	if($displayType=='history' && (cfIsInApp() || cfGGetVar('advancedAdministration'))){
		echo '<input type="checkbox" onchange="setLogViewedContent(this.checked)" onclick="setLogViewedContent(this.checked)"'.((cfGGetVar('historyLogViewedContent'))?' checked="checked"':'').' style="margin-right:1em"><span style="position:absolute;width:150px;">'.cfCaption('historyLogContent').'</span><br/><br/>';
		echo '<input type="checkbox" onchange="setlogViewedContentDisplayThumbnails(this.checked)" onclick="setlogViewedContentDisplayThumbnails(this.checked)"'.((cfGGetVar('logViewedContentDisplayThumbnails'))?' checked="checked"':'').' style="margin-right:1em">'.cfCaption('bookmarksDisplayThumbnails').'<br/>';
	}


	echo "</div>\n";

	// Close button
	if(cfIsInApp()) echo '<center><br/>'.outButton(cfCaption('genClose'),'javascript:wl.UICommand(\'close\')',outIcon('close')).'</center>';

}

// Close left panel
if($displayType=='stats' || $infoType=='list' || cfGGetVar('historyLogViewedContent')){
	echo '</td><td style="width:0.5em">';
	// Main frame td
	echo "<td style=\"vertical-align:top\">\n";
}

/*
 ***************************************************************************************************************************
 * RIGHT PANEL
 ***************************************************************************************************************************
 */
if($displayType=='history' && cfIsInApp())
	echo outDivFrame('frame2', 'id="rightFrame2"', 'height:415px;overflow:auto;white-space:nowrap');
else
	echo outDivFrame('frame2', 'id="rightFrame2"', 'white-space:nowrap');

if($displayType=='stats') echo '<div class="frame2Header">'.cfCaption('calStatsFromTo',cfUTF8Encode(date(cfCaption('_DATE_FORMAT'),$intStart)),cfUTF8Encode(date(cfCaption('_DATE_FORMAT'),$intEnd)))."</div>\n";

/*
 ***************************************************************************************************************************
 * HISTORY
 ***************************************************************************************************************************
 */
if($displayType=='history' && $infoType=='list'){
	require_once(INCLUDE_DIR.'explorerFunctions.php');

	// Get resources and accounts id/name correspondance
	$resources=array(); foreach (dbArrayQuery('SELECT * FROM resource',$db) as $value) $resources[$value['id']]=$value['name'];
	$accounts=array();  foreach (dbArrayQuery('SELECT * FROM account',$db)  as $value) $accounts[$value['id']] =$value['name'];
	$accounts['standalone']=cfUTF8Decode(cfCaption('directResourceAccess'));
	$resourcesConfigs=cfResourcesGetAll();
	$usersConfigs=cfMGetVar('weezoUsers');


	// Filter on date and localhost
	createViewHistory($intStart,$intEnd,$showLocalhost);

	// If content is logged and history is viewed by administrator or from application, include content
	if(cfGGetVar('historyLogViewedContent') && (cfIsInApp() || cfUGetVar('administrator'))){
		$result=dbArrayQuery('SELECT baseView.id as id, baseView.ip as ip, baseView.pseudo as pseudo, baseView.ts as ts, baseView.accountId as accountId, baseView.totalDuration as totalDuration, baseView.resourceId as resourceId, contentAccess.id AS contentId, contentAccess.content AS content FROM baseView LEFT OUTER JOIN contentAccess ON baseView.id=contentAccess.connectionId AND baseView.resourceId=contentAccess.resourceId ORDER BY baseView.id DESC, baseView.resourceId ASC, contentAccess.ts DESC',$db);
		$fullHistory=true;
	}
	// Else, show only basic info
	else{
		$result=dbArrayQuery('SELECT * FROM baseView GROUP BY ip ORDER BY id DESC',$db);
		$fullHistory=false;
	}

	$currentId=false;
	$currentResourceId=false;
	$hasContent=false;
	$firstConnection=true;

	foreach ($result as $k=>$record){
		// New connection
		if($record['id']!=$currentId){
			if($hasContent!==false) echo "</div>\n"; // Close content list
			if($currentResourceId!==false) echo "</div>\n"; // Close resource
			if($currentId!==false) {echo '</div></div><br>'; flush();} // Close connection

			$currentId=$record['id'];
			$currentResourceId=false;
			$hasContent=false;

			// Connection group
			echo outDivFrame('frame3');

			// New connection header and collapse button
			if(isset($usersConfigs[$record['accountId']])) $src='/gfx/icons/'.$usersConfigs[$record['accountId']]['icon'];
			else $src='/gfx/icons/blankIcon.gif';

			$l='&nbsp;<img src="'.$src.'" class="themeIcon" alt="">';
			$l.=cfFixedDate($record['ts'],cfCaption('_FULL_DATE_FORMAT')).' &nbsp; &nbsp; ';
			$l.='<a href="'.cfIPLocatorLink($record['ip']).'" target="_blank" class="link">'.$record['ip'].'</a> &nbsp; &nbsp; ';

			$l.='('.cfUTF8Encode($accounts[$record['accountId']]);
			if($record['pseudo']) $l.=' - '.cfUTF8Encode($record['pseudo']);
			$l.=')';

			if($record['resourceId']!='' || (isset($result[$k+1]) && $result[$k+1]['id']==$record['id']))
				$r=outButtonToggleCollapse('cnxDiv'.$record['id'],false,false,'statsToggleCollapse');
			else
				$r='';
			echo outFrameHeaderTable('frame3Header',$l,$r);

			// Total time
			if(($totalTime=$record['totalDuration'])){
				$sec=$totalTime%60; $totalTime=floor($totalTime/60);
				$min=$totalTime%60; $totalTime=floor($totalTime/60);
				echo '<b>'.cfCaption('calTotalTime').cfCaption('genSeparator').'</b>'.cfCaption('calHourMinSec',cfUTF8Encode($totalTime),cfUTF8Encode($min), cfUTF8Encode($sec)).'<br/><br/>';
			}

			echo '<div id="cnxDiv'.$record['id'].'" style="display:none">';
			$firstConnection=false;
		}

		// New resource access
		if($record['resourceId']!=$currentResourceId){
			if($hasContent!==false) echo ' </div>'; // Close content list
			if($currentResourceId!==false) echo "</div>\n";
			$currentResourceId=$record['resourceId'];
			$hasContent=false;
			echo outDivFrame('frame4','id=resFrame_'.$currentResourceId).'<div class="frame4Header histoItem">';

			// Resource icon
			if(isset($resourcesConfigs[$currentResourceId])) $iconSrc=$resourcesConfigs[$currentResourceId]['definition']['resourceIcon'];
			else $iconSrc=outIcon('broken');
			echo outImage($iconSrc);

			// Resource name
			echo (isset($resources[$currentResourceId]))?cfUTF8Encode($resources[$currentResourceId]):'?';

			echo '</div>';
			if($fullHistory && $record['content']){
				echo '<div class="historyContent">';
				$hasContent=true;
			}
		}

		// New content
		if($fullHistory && $record['content']){
			if(!$hasContent) echo '<div class="historyContent">';
			$hasContent=true;
			echo '<li class="historyContentList"';
			if((cfIsInApp() && strpos($record['content'],'/')!==false)) {
				echo ' onclick="shell(this)" alt="'.$record['contentId'].'" onmouseover="tooltip(this)">';
				if(($ft=efFileType($record['content']))=='image' || $ft=='video')
					echo '<img name="ctn" class="ctn" alt="">';
				else
					echo outImage(efIcon($record['content']),false,' name="ctn" class="ctn" onmouseover="tooltip(this)"');

			}
			else echo '>';
			echo cfUTF8Encode($record['content'])."</li>\n";
		}
	}
	if($hasContent!==false) echo "</div>\n"; // Close content list
	if($currentResourceId) echo "</div>\n"; // Close resource
	if($currentId) echo "</div>\n"; // Close connection
}


/*
 ***************************************************************************************************************************
 * SINGLE CONNECTED USER TRACKING
 ***************************************************************************************************************************
 */
if($displayType=='history' && $infoType=='single') statsSingleUserHistory();


/*
 ***************************************************************************************************************************
 * STATISTICS GRAPHS
 ***************************************************************************************************************************
 */
elseif($infoType=='synth'){
	// Application window display
	if(isset($_ENV['configurationEnvironment']) && cfIsInApp()){
		echo '<center>';
		echo outButton(cfCaption('calBarPlotTitle'),'javascript:displayConnections()');
		echo outButton(cfCaption('calBarPieTitle'),'javascript:displayVisitors()');
		echo outButton(cfCaption('calResourcePieTitle'),'javascript:displayResources()').'<br/><br/>';
		// Graph
		if($displayedGraph=='connections') echo '<img alt="" id="selectedGraph" src="'.$_SERVER['PHP_SELF'].'?graph=Connections&amp;intStart='.$intStart.'&amp;intEnd='.$intEnd.'&amp;intLength='.$intLength.'&amp;showLocalhost='.$showLocalhost.'&amp;noSt=true">';
		if($displayedGraph=='visitors') echo '<img alt="" id="selectedGraph" src="'.$_SERVER['PHP_SELF'].'?graph=Visitors&amp;intStart='.$intStart.'&amp;intEnd='.$intEnd.'&amp;intLength='.$intLength.'&amp;showLocalhost='.$showLocalhost.'&amp;noSt=true">';
		if($displayedGraph=='resources') echo '<img alt="" id="selectedGraph" src="'.$_SERVER['PHP_SELF'].'?graph=Resources&amp;intStart='.$intStart.'&amp;intEnd='.$intEnd.'&amp;intLength='.$intLength.'&amp;showLocalhost='.$showLocalhost.'&amp;noSt=true">';
		echo '</center>';
	}
	// Remote administration display
	else{
		// Connections
		echo '<img alt="" src="'.$_SERVER['PHP_SELF'].'?graph=Connections&amp;intStart='.$intStart.'&amp;intEnd='.$intEnd.'&amp;intLength='.$intLength.'&amp;showLocalhost='.$showLocalhost.'&amp;noSt=true" width="400" height="350" style="float:left;margin:0.5em">';

		// First time / returning Visitors
		echo '<img alt="" src="'.$_SERVER['PHP_SELF'].'?graph=Visitors&amp;intStart='.$intStart.'&amp;intEnd='.$intEnd.'&amp;intLength='.$intLength.'&amp;showLocalhost='.$showLocalhost.'&amp;noSt=true" width="400" height="350" style="margin:0.5em">';
	echo '<br/>';
		// Resource access
		echo '<img alt="" src="'.$_SERVER['PHP_SELF'].'?graph=Resources&amp;intStart='.$intStart.'&amp;intEnd='.$intEnd.'&amp;intLength='.$intLength.'&amp;showLocalhost='.$showLocalhost.'&amp;noSt=true" width="400" height="350" style="float:left; margin:0.5em">';


		// Misc text information
		echo '<div class="frame3" style="position:relative;height:342px;width:392px;margin:0.5em;left:5px;margin-left:0px">'; // (remove arbitrary 8px for padding + WA for left margin)
		createView2($intStart,$intEnd,$showLocalhost);

		echo '<div class="frame3Header">'.cfCaption('genMisc')."</div>\n";
		$nbUniques=count(dbArrayQuery('SELECT ip,count(ip) FROM baseView GROUP BY ip',$db));
		$totalTime=dbSingleQuery('SELECT sum(duration) FROM baseView',$db);

		// Unique visitors
		echo '<b>'.cfCaption('calUnique').cfCaption('genSeparator').'</b>'.cfUTF8Encode($nbUniques).'<br/><br/>';

		// Total connection time
		$sec=$totalTime%60; $totalTime=floor($totalTime/60);
		$min=$totalTime%60; $totalTime=floor($totalTime/60);
		echo '<b>'.cfCaption('calTotalTime').cfCaption('genSeparator').'</b>'.cfCaption('calHourMinSec',cfUTF8Encode($totalTime),cfUTF8Encode($min), cfUTF8Encode($sec)).'<br/>';
		// Debug
		//echo '<a target="_blank" href="http://localhost:8080'.$_SERVER['PHP_SELF'].'?graph=Visitors&intStart='.$intStart.'&intEnd='.$intEnd.'&intLength='.$intLength.'&showLocalhost='.$showLocalhost.'&noSt=true">Lien</a>';
		echo "</div>\n";
	}
}


/*
 ***************************************************************************************************************************
 * STATISTICS DETAILS
 ***************************************************************************************************************************
 */
elseif($infoType=='details'){
	createView1($intStart,$intEnd,$showLocalhost);
	$result=dbArrayQuery('SELECT * FROM baseView GROUP BY ip ORDER BY ip,id DESC, ts ASC',$db);
	$resources=array(); foreach (dbArrayQuery('SELECT * FROM resource',$db) as $value) $resources[$value['id']]=$value['name'];
	$accounts=array(); foreach (dbArrayQuery('SELECT * FROM account',$db) as $value) $accounts[$value['id']]=$value['name'];

	// Group by IP
	$list=array();
	foreach ($result as $value){
		if(!isset($list[$value['ip']])) $list[$value['ip']]=array();
		if(isset($value['hostname']) && strlen($value['hostname'])) $list[$value['ip']]['hostname']=$value['hostname'];
		if(!isset($list[$value['ip']][$value['id']])) $list[$value['ip']][$value['id']]=array('accountId'=>$value['accountId'],'totalDuration'=>$value['totalDuration'],'ts'=>$value['ts'],'res'=>array(),'pseudo'=>$value['pseudo']);
		if(!isset($list[$value['ip']][$value['id']]['res'][$value['resourceId']]) && $value['resourceId']!='') $list[$value['ip']][$value['id']]['res'][$value['resourceId']]=$value['nbPages'];
	}
	// IP group
	if(!count($list)) echo '<br/><div style="text-align:center;padding:10em" id="noConnectionDiv">'.cfCaption('calNoConnection').'</div>';
	else{
		$ipNb=0;
		$resolvedIp=array();
		foreach ($list as $ip=>$ipGroup){
			// Resolve ip host name and store it into db
			if(!isset($ipGroup['hostname'])){
				if(isset($resolvedIp[$ip])){
					$hostName=$resolvedIp[$ip];
				}
				elseif($hostName=@gethostbyaddr($ip)){
					dbArrayQuery('UPDATE connection SET hostname=\''.str_replace('\'',"''",$hostName).'\' WHERE ip=\''.$ip.'\'',$db);
					$resolvedIp[$ip]=$hostName;
				}
				else {
					$hostName='?';
					dbArrayQuery('UPDATE connection SET hostname=\''.str_replace('\'','`',$hostName).'\' WHERE ip=\''.$ip.'\'',$db);
					$resolvedIp[$ip]=$hostName;
				}
			}
			else{
				$hostName=$ipGroup['hostname'];
			}
			echo outDivFrame('frame2');
			echo outFrameHeaderTable('frame2Header','<a href="'.cfIPLocatorLink($ip).'" target="_blank">'.cfUTF8Encode($ip).' ('.cfUTF8Encode($hostName).')</a>',outButtonToggleCollapse('ipDiv'.$ipNb,false,false,'statsToggleCollapse'));
			echo '<div id="ipDiv'.$ipNb.'" style="display:none">';
			// Connection group
			$groupNb=0;
			foreach ($ipGroup as $id=>$connectionGroup) if($id!='hostname'){
				echo outDivFrame('frame3');
				echo outFrameHeaderTable('frame3Header',cfCaption('genGroup').cfCaption('genSeparator').cfUTF8Encode($accounts[$connectionGroup['accountId']]).((strlen($connectionGroup['pseudo']))?' - '.cfUTF8Encode($connectionGroup['pseudo']):''),cfFixedDate($connectionGroup['ts'],cfCaption('_FULL_DATE_FORMAT')).'&nbsp;'.outButtonToggleCollapse('groupDiv'.$ipNb.'-'.$groupNb,false,((count($ipGroup)>3)?false:true),'statsToggleCollapse'));
				echo '<div id="groupDiv'.$ipNb.'-'.$groupNb.'" style="display:'.((count($ipGroup)>3)?'none':'inline').'">';
				$totalTime=$connectionGroup['totalDuration'];
				$sec=$totalTime%60; $totalTime=floor($totalTime/60);
				$min=$totalTime%60; $totalTime=floor($totalTime/60);
				echo '<b>'.cfCaption('calTotalTime').cfCaption('genSeparator').'</b>'.cfCaption('calHourMinSec',cfUTF8Encode($totalTime),cfUTF8Encode($min), cfUTF8Encode($sec)).'<br/><br/>';
				echo '<b>'.cfCaption('calResourceAccess').cfCaption('genSeparator').'</b><br/>';
				// Resource access group
				echo '<ul>';
				if(count($connectionGroup['res'])>0){
					foreach ($connectionGroup['res'] as $resourceId=>$nbPages){
						echo '<li style="padding-left:1em">'.cfUTF8Encode((isset($resources[$resourceId]))?$resources[$resourceId]:'???').' ('.cfCaption('calResourcePieTitle').cfCaption('genSeparator').$nbPages.')</li>';
					}
				}
				else echo '<li style="padding-left:1em">-</li>';
				echo "</ul></div></div>\n";
				$groupNb++;
			}
			echo "</div></div>\n";
			$ipNb++;
			flush();
		}
	}
}
echo "</div>"; // End of rightFrame2
echo "</td></tr></table></div>\n";

// Close DB
dbClose($db);
?>