<?php
/**
 * Wall main file
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
*
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2013 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 3.1.0
 */


//die(wSession_id());
define('W_WALL_MAX_POST_LENGTH',16384);			// Post max length (utf-8 characters)
define('W_WALL_MAX_POSTS_PER_PAGE_CLASSIC',30);	// Max displayed posts per page in classic view
define('W_WALL_MAX_POSTS_PER_PAGE_BOARD',100);	// Max displayed posts per page in board view
define('W_WALL_MAX_POSTS_PER_PAGE_MOBILE',20);	// Max displayed posts per page in mobile view
define('W_WALL_TEXT_DISPLAYS',5);				// Number of displays for text posts
define('W_WALL_IMAGES_WIDTH',320);				// Default width for images, maps and website thumbnails
define('W_WALL_MAX_TN_GENERATION_TIME',20);		// Max thumbnail generation time (seconds)
define('W_WALL_FEED_MIN_UPDATE_INTERVAL',30);	// Min time between two external feeds updates (seconds)
define('W_WALL_FEED_INITIAL_HISTORY_TIME',3600*24*7);	// Initial import time for external feeds
define('W_WALL_REF_POST_WIDTH',300);			// Reference post item width, used for margins calculation

require_once(INCLUDE_DIR.'databaseFunctions.php');
require_once(INCLUDE_DIR.'facebookFunctions.php');
if(!cfIsAsync()) require_once(INCLUDE_DIR.'outputFunctions.php');

// Global database var
$db=false;

$postCommands=array(
	'postCreate'=>array('text','userName','type','link'),
	'postDelete'=>array('id'),
	'commentCreate'=>array('postId','text','userName'),
	'commentDelete'=>array('id'),
	'setInterval'=>array('from','to')
);


/**
 * @desc Initialize wall
 *
 */
function wallInit(){
	global $db;

	$dataBaseFile=cfAppResourceDir().'/wall.db';

	if(!cfRGetVar('resourceInitialized')){
		cfRInitVar('view','board');
		cfRInitVar('mediaWidth',W_WALL_IMAGES_WIDTH);

		// Init user name (pseudo)
		if(WEnv::user()->getVar('accountType')=='anonymous') cfRSetVar('pseudo','');
		elseif (WEnv::user()->getVar('accountType')=='list') cfRSetVar('pseudo',cfUTF8Encode(WEnv::user()->subUser()->name()));
		elseif (WEnv::user()->getVar('accountType')=='singleUser') cfRSetVar('pseudo',cfUTF8Encode(WEnv::user()->getVar('name')));
		else cfRSetVar('pseudo',cfUTF8Encode(WEnv::user()->getVar('pseudo')));

		// Set default view to board for browsers or mobile for... mobiles
		cfRInitVar('view','board');
		cfRSetVar('resourceInitialized',1);
	}

	// Initialize database
	if(!file_exists($dataBaseFile) || filesize($dataBaseFile)==0){
		if($db=dbOpenDB($dataBaseFile)){
			// Create posts table
			dbExec('CREATE TABLE posts ('.
				'id INTEGER PRIMARY KEY, '. // Post id
				'ts TIMESTAMP, '.			// timestamp
				'ip TEXT, '.				// poster's IP address
				'icon TEXT, '.				// user icon, located into www/gfx/icons
				'type TEXT, '.				// type of post: text, image, map...
				'name TEXT, '.				// name of user who posted
				'text TEXT, '.				// main text
				'link TEXT, '.
				'display TEXT'.
			')',$db);

			// Create comments table
			dbExec('CREATE TABLE comments ('.
				'id INTEGER PRIMARY KEY, '.
				'ts TIMESTAMP, '.
				'postId TEXT, '.
				'ip TEXT, '.
				'icon TEXT, '.
				'name TEXT, '.
				'text TEXT'.
			')',$db);

			// Create last external feeds updates table
			dbExec('CREATE TABLE updates ('.
				'type TEXT, '. // Feed type ('general' (for global update), 'twitter', 'facebook'...)
				'value TEXT, '. // Feed value (twitter account, facebook name...)
				'ts TIMESTAMP, '. // Timestamp of last successful update
				'PRIMARY KEY(type, value)'.
			')',$db);

			dbClose($db);
		}
		else outDisplayErrorPage('Error creating/opening database file '.$dataBaseFile);
	}
	// (re) open database
	$db=dbOpenDB($dataBaseFile);
	if(!$db) outDisplayErrorPage('Error creating/opening database file '.$dataBaseFile);

	// Mobile view
	if(/*cfRGetVar('view')!='board' && */cfIsMobile()) {
		cfRSetVar('mobileView',true);
		require_once(INCLUDE_DIR.'mobileFunctions.php');
		require_once(dirname(__FILE__).'/mobileIndex.php');
	}
	else cfRSetVar('mobileView',false);

	// Update post / comment rights
	cfRInitUserRightsVar('postAllowed');
	cfRInitUserRightsVar('commentAllowed');
}

/**
 * Dump database for debug
 *
 */
function wallDumpBase(){
	global $db;
	cfVarDump(dbArrayQuery('SELECT * FROM posts',$db)); // prod
	cfVarDump(dbArrayQuery('SELECT * FROM comments',$db)); // prod
	cfVarDump(dbArrayQuery('SELECT * FROM updates',$db)); // prod
}

function wallCleanLinkTail($link){
	// Remove crap at end of link and concatenate ID and video provider type
	if(strpos($link,'_')) $link=substr($link,0,strpos($link,'_'));
	if(strpos($link,'"')) $link=substr($link,0,strpos($link,'"'));
	if(strpos($link,'?')) $link=substr($link,0,strpos($link,'?'));
	if(strpos($link,'&')) $link=substr($link,0,strpos($link,'&'));
	return $link;
}

/**
 * @desc Create a new post from user-sent POST data
 *
 * @param array $args: POST data
 */
function wallPostCreate($args){
	global $db;

	if(!cfRGetVar('postAllowed')) return;

	$args['text']=str_replace(array("\r\n","\r","\n"),'<br>',trim($args['text']));
	$args['text']=dbEscapeString(cfStrTruncate($args['text'],W_WALL_MAX_POST_LENGTH),$db);
	$args['icon']=WEnv::user()->getVar('icon');
	$link='';

	// User name (pseudo)
	if(WEnv::user()->getVar('accountType')=='anonymous') {
		$args['userName']=dbEscapeString(cfStrTruncate($args['userName'],50),$db);
		cfRSetVar('pseudo',$args['userName']);
	}
	// List type: use subUser name
	elseif (WEnv::user()->getVar('accountType')=='list') $args['userName']=cfUTF8Encode(WEnv::user()->subUser()->name());
	// Single user group: use group name
	elseif (WEnv::user()->getVar('accountType')=='singleUser') $args['userName']=cfUTF8Encode(WEnv::user()->getVar('name'));
	// Group with pseudo: use pseudo entered at login
	else $args['userName']=cfUTF8Encode(WEnv::user()->getVar('pseudo'));

	// Link type is image, and an image has been uploaded
	if($args['type']=='image'){
		if(file_exists(cfAppTempDir().'/temp.wall.'.wSession_id().'.jpg')){
			// Find a name for image
			$imgIndex=0;
			while (file_exists(cfAppResourceDir().'/img.'.$imgIndex.'.jpg')) $imgIndex++;

			// Move uploaded image to resource data directory
			@rename(cfAppTempDir().'/temp.wall.'.wSession_id().'.jpg',cfAppResourceDir().'/img.'.$imgIndex.'.jpg');
			list($w, $h, $type, $attr) = @getimagesize(cfAppResourceDir().'/img.'.$imgIndex.'.jpg');
			// populate link with width & height & img name
			$link='img;'.$w.';'.$h.';img.'.$imgIndex.'.jpg';

			// If a full size image is present
			if(file_exists(cfAppTempDir().'/temp.wall.'.wSession_id().'_f.jpg')){
				@rename(cfAppTempDir().'/temp.wall.'.wSession_id().'_f.jpg',cfAppResourceDir().'/img.'.$imgIndex.'_f.jpg');
			}
		}
		else{
			$args['type']='text';
		}
	}

	// Link type is url: generate thumbnail
	elseif($args['type']=='url'){
		if(1){
			// Find a name for image
			$imgIndex=0;
			while (file_exists(cfAppResourceDir().'/img.'.$imgIndex.'.jpg')) $imgIndex++;

			$tnFile=cfAppResourceDir().'/img.'.$imgIndex.'.jpg';

			// Create thumbnail (sync call to UI)
			cfServerSendCmd('makeWebsiteThumbnail',array(
				'url'=>$args['link'],
				'outputFile'=>$tnFile,
				'size'=>cfRGetVar('mediaWidth'),
				'jpgQuality'=>cfHGetVar('thumbnailJPGQuality',DEFAULT_JPG_QUALITY)),
				false);
			// wailt for tn to be generated
			$startTime=time();
			while (!file_exists($tnFile) && time()-$startTime<W_WALL_MAX_TN_GENERATION_TIME) usleep(50000);

			// If it could not be, just save link
			if(!file_exists($tnFile)) $link='url;'.$args['link'];

			// populate link with width & height & img name
			else{
				list($w, $h, $type, $attr) = @getimagesize($tnFile);
				$link='tn;'.$w.';'.$h.';'.basename($tnFile).';'.$args['link'];
			}
		}
		// No thumbnail: just display url
		else $link='url;'.$args['url'];
	}

	// Link type is url: extract video ID from data
	elseif($args['type']=='video'){
		$link=false;

		$formats=array(
			'player.vimeo.com/video/'=>'vimeo',
			'vimeo.com/'=>'vimeo',
			'www.youtube.com/watch?v='=>'youtube',
			'www.youtube.com/embed/'=>'youtube',
			'youtu.be/'=>'youtube',
			'player.vimeo.com/video/'=>'vimeo',
			'www.dailymotion.com/embed/video/'=>'dailymotion',
			'www.dailymotion.com/video/'=>'dailymotion'
		);

		foreach ($formats as $k=>$v) if(($p=strpos($args['link'],$k))!==false){
			$type=$v;
			$link=substr($args['link'],$p+strlen($k));
			$link=wallCleanLinkTail($link);
			break;
		}
		if($link && $type=='vimeo' && !is_numeric($link)) $link=false; // Ensure vimeo link is numeric

		// Vimeo couch mode format: ID is last part of URL
		if(!$link && strpos($args['link'],'vimeo.com/couchmode/')!==false){
			$link=substr($args['link'],strrpos($args['link'],'/')+1);
			$link=wallCleanLinkTail($link);
			if(is_numeric($link)) $type='vimeo'; else $link=false;
		}

		// URL not recognized: try to autodetect an ID
		if(!$link && strlen($args['link'])) {
			for($i<0;$i<strlen($args['link']);$i++){
				$a=ord(substr($args['link'],$i,1));
				if($a<48 || ($a>57 && $a<65) || ($a>90 && $a<97) || $a>122) break;
			}
			if($i==strlen($args['link'])) {
				$link=$args['link'];
				if(is_numeric($link)) $type='vimeo';
				elseif(strlen($link)>8) $type='youtube';
				else $type='daylimotion';
			}
		}

		$link=$link.';'.$type;
	}

	// Link type is (Google) map
	elseif($args['type']=='map'){
		$params=explode(';',$args['link']);
		if(count($params)!=4) return;
		if(!is_numeric($params[0]) || !is_numeric($params[1]) || !is_numeric($params[2]) || !in_array($params[3],array('roadmap','hybrid','satellite','terrain'))) return;
		$link=$args['link'];
	}

	// display scheme
	$display=rand(0,W_WALL_TEXT_DISPLAYS);

	$ts=time();

	$query="INSERT INTO posts VALUES (NULL, ".							// Autoincrement id
			$ts.", ".													// Timestamp
			"'".$_SERVER['REMOTE_ADDR']."', ".							// IP
			"'".$args['icon']."', ".									// Icon
			"'".$args['type']."', ".									// Post type: 'text', 'image'
			"'".$args['userName']."', ".								// User pseudo
			"'".$args['text']."', ".									// Main text
			"'".$link."', ".											// Link
			"'".$display."'".											// Display scheme
			")";
	$ok=0;

	// Insert into database
	if(dbExec($query,$db)){
		// Retrieve new id
		$id=dbSingleQuery('SELECT max(id) FROM posts WHERE ts='.$ts,$db);

		// Update display with newly created post
		$post=dbArrayQuery('SELECT * from posts WHERE id='.$id,$db);
		unset($post[0]['ip']);
		$post[0]['text']=($post[0]['text']);
		$post[0]['name']=($post[0]['name']);

		echo cfAsyncJSaction('wall.addNewPost('.json_encode($post[0]).')');

		// Log to app
		cfLog(cfCaption('newPostInfo',$args['userName'],cfUTF8Encode(cfRGetVar('name')),false,true),LOG_INF);

		$ok=1;
	}
	echo cfAsyncJSaction('wall.sendPostCompleted('.$ok.')');
}

/**
 * @desc Delete a post
 *
 * @param array $args
 */
function wallPostDelete($args){
	global $db;
	if(is_numeric($args['id']) && WEnv::user()->isAdministrator()){
		// Find post in db
		$post=dbArrayQuery('SELECT id,link FROM posts WHERE id='.$args['id'],$db);
		if(count($post)==1){
			// Delete post
			$resP=dbExec('DELETE FROM posts WHERE id='.$args['id'],$db);
			$resC=dbExec('DELETE FROM comments WHERE postId='.$args['id'],$db);

			// Delete associated media
			if($resP){
				$link=$post[0]['link'];
				if(cfCmpLeft($link,'img;')||cfCmpLeft($link,'tn;')){
					$link=explode(';',$link);
					$img=cfJoinPathFile(cfAppResourceDir(),str_replace('..','.',$link[3]));
					if(file_exists($img)) @unlink($img);
					if(file_exists(str_replace('.jpg','_f.jpg',$img))) @unlink(str_replace('.jpg','_f.jpg',$img));
				}
			}
		}
	}
}

/**
 * @desc Return data for the no-post item
 *
 * @return true
 */
function wallNoPostItem(){
	return array(
		'id'=>'fake',
		'icon'=>'',
		'name'=>'&nbsp;',
		'type'=>'text',
		'text'=>'<center style="padding:80px 0 80px 0;font-size:130%"> '.cfCaption('blogNoPost').' </center>',
		'link'=>'',
		'fake'=>1,
		'display'=>1
	);
}

/**
 * @esc Create a new comment from user-sent POST data
 *
 * @param array $args
 * @return bool result
 */
function wallCommentCreate($args){
	global $db;

	if(!cfRGetVar('commentAllowed')) return;

	$args['text']=str_replace(array("\r\n","\r","\n"),'<br>',trim($args['text']));
	$args['text']=dbEscapeString(cfStrTruncate($args['text'],W_WALL_MAX_POST_LENGTH),$db);
	$args['icon']=WEnv::user()->getVar('icon');
	$type='text';
	$link='';
	$display=0;
	$ts=time();

	// User name (pseudo)
	if(WEnv::user()->getVar('accountType')=='anonymous') $args['userName']=dbEscapeString(cfStrTruncate($args['userName'],50),$db);
	elseif (WEnv::user()->getVar('accountType')=='list') $args['userName']=cfUTF8Encode(WEnv::user()->subUser()->name());
	elseif (WEnv::user()->getVar('accountType')=='singleUser') $args['userName']=cfUTF8Encode(WEnv::user()->getVar('name'));
	else $args['userName']=cfUTF8Encode(WEnv::user()->getVar('pseudo'));

	if(!is_numeric($args['postId'])) return false;

	// 1st check passed post id exists
	$postId=dbSingleQuery('SELECT id FROM posts WHERE id="'.$args['postId'].'"',$db);
	if(!$postId) return false;

	$query="INSERT INTO comments VALUES (NULL, ".						// Autoincrement id
			$ts.", ".													// Timestamp
			"'".$args['postId']."', ".									// database post id
			"'".$_SERVER['REMOTE_ADDR']."', ".							// IP
			"'".$args['icon']."', ".									// Icon
			"'".$args['userName']."', ".								// User pseudo
			"'".$args['text']."'".										// Main text
			")";

	$res=dbExec($query,$db);

	// Send JS code to display newly created comment
	if($res){
		$id=dbSingleQuery('SELECT max(id) FROM comments WHERE ts='.$ts,$db);
		$comment=dbArrayQuery('SELECT * from comments WHERE id='.$id,$db);
		unset($comment[0]['ip']);

		$comment[0]['text']=($comment[0]['text']);
		$comment[0]['name']=($comment[0]['name']);

		echo cfAsyncJSaction('wall.addNewComment('.json_encode($comment[0]).')');

		// Log to app
		cfLog(cfCaption('newCommentInfo',$args['userName'],cfUTF8Encode(cfRGetVar('name')),false,true),LOG_INF);
	}
	return $res;
}

/**
 * @desc Delete a post
 *
 * @param array $args
 */
function wallCommentDelete($args){
	global $db;
	if(is_numeric($args['id']) && WEnv::user()->isAdministrator()){
		$res=dbExec('DELETE FROM comments WHERE id='.$args['id'],$db);
		return $res;
	}
}

/**
 * @desc Return JS object containing posts and comments data
 *
 */
function wallUpdateJSData(){
	global $db;

	// Interval
	$to=cfRGetVar('displayToTimestamp');
	if(!$to) $to=time();
	$to='ts<='.$to;

	$from=cfRGetVar('displayFromTimestamp');
	if($from) $from=' AND ts>='.$from;

	switch (cfRGetVar('view')){
		case 'board':
			$limit=' LIMIT '.W_WALL_MAX_POSTS_PER_PAGE_BOARD;
			break;
		case 'classic':
			$limit=' LIMIT '.W_WALL_MAX_POSTS_PER_PAGE_CLASSIC;
			break;
		case 'mobile':
		default:
			$limit=' LIMIT '.W_WALL_MAX_POSTS_PER_PAGE_MOBILE;
			break;
	}

	$result=array('posts'=>array(),'comments'=>array());

	// Posts
	$tmpPosts=dbArrayQuery('SELECT * FROM posts WHERE '.$to.$from.' ORDER BY ts DESC '.$limit,$db);

	// Compute min/max, remove ip, and use id as array key into $posts
	$min=9999;$max=0;
	foreach ($tmpPosts as $k=>$post){
		$min=min($min,$post['id']);
		$max=max($max,$post['id']);
		$id=$post['id'];
		unset($post['ip']);
		//$post['text']=htmlentities($post['text']);
		//cfDbg($post['text'].strlen($post['text']));
		$post['text']=($post['text']);
		$post['name']=($post['name']);
		$result['posts'][]=$post;
	}

	// Comments
	$tmpComments=dbArrayQuery('SELECT comments.* FROM COMMENTS JOIN posts ON comments.postId=posts.id WHERE posts.id<='.$max.' AND posts.id>='.$min.' ORDER BY comments.ts DESC',$db);
	foreach ($tmpComments as $k=>$comment){
		$id=$comment['id'];
		$comment['text']=($comment['text']);
		$comment['name']=($comment['name']);
		unset($comment['ip']);
		$result['comments'][]=$comment;
	}

	// Return json-encoded array
	return (json_encode($result));
}

/**
 * @desc Process async commands
 *
 */
function wallProcessCommands(){
	global $postCommands;

	// Sync view change
	if(isset($_GET['view']) && !cfIsAsync() && in_array($_GET['view'],array('classic','board'))) cfRSetVar('view',$_GET['view']);

	if(!cfIsAsync()) return;
	cfAsyncHeader();

	// Update right frame folding
	if(isset($_POST['rightFrameFolded'])){
		cfRSetVar('rightFrameFolded',($_POST['rightFrameFolded'])?true:false);
		die(cfAsyncFooter());
	}


	$command=@$_POST['command'];
	if(!isset($postCommands[$command])) die(cfAsyncJSaction('alert("unknown command: '.$command.'")').cfAsyncFooter(false));
	foreach ($postCommands[$command] as $param) if(!isset($_POST[$param])) die(cfAsyncJSaction('alert("missing parameter '.$param.' for command '.$command.'")').cfAsyncFooter(false));

	switch (@$_POST['command']){
		case 'postCreate':
			wallPostCreate($_POST);
			break;
		case 'postDelete':
			wallPostDelete($_POST);
			break;
		case 'commentCreate':
			wallCommentCreate($_POST);
			if(cfIsMobile()) echo cfAsyncJSaction('wall.toggleComments("'.$_POST['postId'].'",1)');
			break;
		case 'commentDelete':
			wallCommentDelete($_POST);
			break;
		case 'setInterval':
			cfRSetVar('displayToTimestamp',@$_POST['to']);
			cfRSetVar('displayFromTimestamp',@$_POST['from']);
			echo cfAsyncJSaction('wall.updateData('.wallUpdateJSData().')');
			echo cfAsyncJSaction('wall.updateDisplay()');
			break;
	}
	die(cfAsyncFooter());
}

/**
 * @desc Display new post form
 *
 */
function wallDisplayNewPostForm(){
	$view=cfRGetVar('view');
	$btWidth=($view=='board')?'150px':'180px';

	if(!cfRGetVar('postAllowed')) return;

?>
<div id="wallPostForm">
	<div class="frame1Header"><?php echo outImageIcon('comments').cfCaption('blogPostNewMessage');?></div><br>
	<textarea id="wallPostText" onkeyup="void(wall.sendPostOk())"></textarea>
	<div id="wallLinkOptions">
	<br>
	<?php

	// Link buttons

	// Image upload button
	echo outBt(cfCaption('blogInsertImage'),outIcon('fi/icoImage'),"javascript:wall.linkTypeSelect('image')",'linkTypeBtImage',false,false,'up',$btWidth,'left').'<br>';

	// Map display button
	echo outBt('Google Map',outIcon('maps'),"javascript:wall.linkTypeSelect('map')",'linkTypeBtMap',false,false,'up',$btWidth,'left').'<br>';

	// Embeded video button
	echo outBt(cfCaption('genVideos'),outIcon('embeddedVideos'),"javascript:wall.linkTypeSelect('video')",'linkTypeBtVideo',false,false,'up',$btWidth,'left').'<br>';

	// Website link button
	echo outBt(cfCaption('blogLink'),outIcon('fi/icoXml'),"javascript:wall.linkTypeSelect('url')",'linkTypeBtUrl',false,false,'up',$btWidth,'left').'<br>';
	?>
	</div>
	<div id="wallLinkType_image" style="display:none">
		<?php echo outUploadForm('12','wall.imageUploadCompleted','none',true,false,array('compact'=>true,'allowedDestDir'=>true,'autoSubmit'=>1,'maxFiles'=>1,'noDragDrop'=>1,'extensions'=>array('jpg'=>1,'jpeg'=>1,'gif'=>1,'tiff'=>1,'png'=>1,'bmp'=>1))); ?>
	</div>
	<div id="wallLinkType_map" style="display:none">
		<div id="wallNewMap"></div>
	</div>
	<div id="wallLinkType_url" style="display:none">
		<?php echo cfCaption('genLink');?><br>
		<input id="wallNewUrl" type="text" value="http://" onchange="wall.sendPostOk()" onkeyup="wall.sendPostOk()">
	</div>

	<div id="wallLinkType_video" style="display:none">
		ID / <?php echo cfCaption('genLink');?> / Code<br>
		<input id="wallNewVideo" type="text" value="" onchange="wall.sendPostOk()" onkeyup="wall.sendPostOk()">
	</div>

	<div id="wallSendBtArea">
	<?php echo outBt(cfCaption('genSend'),outIcon('ok16'),'javascript:wall.sendPost()','sendPostBt',false,false,'off');?>
	</div>
</div>
<?php
}

/**
 * @desc Display calendar
 *
 */
function wallDisplayCalendar(){
	global $db;
	global $postsList;
	$view=cfRGetVar('view');
	$btWidth=($view=='board')?'150px':'180px';

	/**
	 * Check if there is a messsage in given interval
	 * @param timestamp $from
	 * @param timestamp $to
	 * @return boolean result
	 */
	function messageInInterval($from,$to){
		global $postsList;
		foreach ($postsList as $k=>$v){
			if($v['ts']>=$from && $v['ts']<=$to) return true;
			if($v['ts']>$to) return false;
		}
		return false;
	}

	$postsList=dbArrayQuery('SELECT id,ts FROM posts ORDER BY ts ASC',$db);

	echo '<br><br><div class="frame1Header">',outImageIcon('calendar16.png').cfCaption('blogPreviousPosts').'</div>';
	@setlocale(LC_TIME , cfCaption('_ISO_639'));

	// Display calendar of current month

	$firstDayOfMonth=mktime(0,0,1,date('n'),1,date('Y'));
	$day=$firstDayOfMonth; while (date('w',$day)!=1) $day+=3600*24;
	$offset=(date('w',$firstDayOfMonth)==0)?7:date('w',$firstDayOfMonth);
	echo '<center>';
	// Current month
	echo outBt(cfUTF8Encode(ucfirst(strftime('%B %Y',$firstDayOfMonth))),false,'javascript:wall.goMsgs(0,0)',false,false,false,'up',$btWidth);

	// Calendar header
	echo '<table class="frame3" style="text-align:center;table-layout:fixed" cellspacing="0" cellpadding="0"><tr class="frame3header">';
	for($i=0;$i<7;$i++) {echo '<td style="width:20px">'.cfUTF8Encode(strtoupper(substr(strftime('%A',$day),0,1))).'</td>'; $day+=3600*24;}
	echo '</tr><tr class="bsRow">';

	// Calendar body (days)
	$day=$firstDayOfMonth;
	for($i=1;$i<$offset;$i++) echo '<td>&nbsp;</td>';
	$position=$offset-1;
	for($i=1;$i<date('t',$firstDayOfMonth)+1;$i++){
		if($position==7) {$position=0; echo '</tr><tr class="bsRow">';}
		$position++;
		$day=($i<10)?'&nbsp;'.$i:$i;
		if(messageInInterval(
			mktime( 0, 0, 0,date('n',$firstDayOfMonth),$i,date('Y',$firstDayOfMonth)),
			mktime(23,59,59,date('n',$firstDayOfMonth),$i,date('Y',$firstDayOfMonth)))
			){
				if($view=='board')
					echo '<td class="buttonSmall" onclick="wall.goMsgs(\''.mktime(0,0,1,date('n',$firstDayOfMonth),$i,date('Y',$firstDayOfMonth)).'\',\''.mktime(23,59,59,date('n',$firstDayOfMonth),$i,date('Y',$firstDayOfMonth)).'\')">'.$day.'</td>';
				else
					echo '<td>'.outBlockItem('cal',$day,'','margin:1px',array('onclick'=>'wall.goMsgs(\''.mktime(0,0,1,date('n',$firstDayOfMonth),$i,date('Y',$firstDayOfMonth)).'\',\''.mktime(23,59,59,date('n',$firstDayOfMonth),$i,date('Y',$firstDayOfMonth)).'\')')).'</td>';
		}
		else {
			if($view=='board') echo '<td class="buttonSmallDisabled">'.$i.'</td>';
			else echo '<td>'.outBlockItem('cal',$day,false,'margin:1px',false,'disabled').'</td>';
		}

	}
	echo '</tr></table><br/>';

	// Older messages (previous month)
	$lastDisplayedMonth=mktime(0,0,1,date('n',$firstDayOfMonth),1,date('Y',$firstDayOfMonth));
	foreach ($postsList as $key=>$value){
		$currentMonth=mktime(0,0,1,date('n',$value['ts']),1,date('Y',$value['ts']));
		if($currentMonth<$lastDisplayedMonth){
			echo outBt(cfUTF8Encode(ucfirst(strftime('%B %Y',$currentMonth))),false,'javascript:wall.goMsgs('.$currentMonth.','.mktime(23,59,59,date('n',$currentMonth),date('t',$currentMonth),date('Y',$currentMonth)).')',false,false,false,'up',$btWidth).'<br>';
			$lastDisplayedMonth=$currentMonth;
		}
	}
	echo '<br/></center>';
}

/**
 * @desc Send image on an image type post
 *
 * @param string $image: image filename, located into resource data directory
 */
function wallSendPostImage($image){
	if(
		substr($image,0,4)=='img.' && file_exists(cfAppResourceDir().'/'.$image) || // Post image
		$image=='facebookIcon.jpg' ||												// Facebook icon
			(cfCmpLeft($image,'twitterIcon_') && cfCmpRight($image,'.jpg'))			// Twitter account icon
			){
		WHeaders::contentType('jpg');
		echo file_get_contents(cfAppResourceDir().'/'.$image);
	}
	exit;
}

/**
 * Add messages (and their comments) from external feeds to posts (and comments)
 *
 * @param array $messages: array of messages (type, text, time, iconUrl, link)
 * @param array $feed
 */
function wallFeedsAddMessages($messages, $feed){
	global $db;


	$latest=$feed['lastUpdateTime'];

	foreach ($messages as $message){
		$latest=max($latest,$message['time']);

		$message['userName']=dbEscapeString($message['userName'],$db);
		$message['text']=dbEscapeString($message['text'],$db);
		$message['link']=dbEscapeString(@$message['link'],$db);

		$query="INSERT INTO posts VALUES (NULL, ".							// Autoincrement id
				$message['time'].", ".										// Timestamp
				"'0.0.0.0', ".												// IP
				"'".$message['icon']."', ".									// Icon
				"'".$message['type']."', ".									// Post type: 'text', 'image'
				"'".$message['userName']."', ".								// User pseudo
				"'".$message['text']."', ".									// Main text
				"'".$message['link']."', ".									// Link
				"''".														// Display scheme
				")";

		$result=dbExec($query,$db);
		$update=dbExec('INSERT OR REPLACE INTO updates VALUES ("'.$feed['type'].'","'.$feed['id'].'",'.$latest.')',$db);
	}
}

/**
 * @desc Return feeds from Twitter XML result
 *
 * @param string $xml
 */
function wallParseTwitterFeeds($xml,$feedData){
	$iconFile=cfAppResourceDir().'/twitterIcon_'.$feedData['id'].'.jpg';

	$messages=array();
	if(!$xml) return $messages;
	$parsed=cfXmlToArray($xml);
	if(is_array($parsed['status'])){
		foreach($parsed['status'] as $feed){

			// Remove already added feeds
			$time=strtotime(@$feed['created_at']);
			if($feedData['lastUpdateTime']>=$time) continue;

			$text=''; $in=false;
			@$feed['text']=@$feed['text'].' ';
			for($i=0;$i<strlen($feed['text']);$i++){
				$c=substr($feed['text'],$i,1);
				if(!$in){
					if($c=='@' || $c=='#' || substr($feed['text'],$i,7)=='http://' || substr($feed['text'],$i,8)=='https://') {
						$in=$c;
						$pos=$i;
					}
				}
				elseif ($c==' '||$c==','||$c==';'||$c=="\t"||$c=="\r"||$c=="\n"){
					$text.='<a class="wallUrl" target="_blank" href="';
					if($in=='@') $text.='https://twitter.com/#!/'.substr($feed['text'],$pos+1,$i-$pos-1).'">'.substr($feed['text'],$pos,$i-$pos).'</a>';
					elseif($in=='#') $text.='https://twitter.com/#!/search?q=%23'.substr($feed['text'],$pos+1,$i-$pos-1).'">'.substr($feed['text'],$pos,$i-$pos).'</a>';
					elseif($in=='h') $text.=substr($feed['text'],$pos,$i-$pos).'">'.str_replace(array('http://','https://'),'',substr($feed['text'],$pos,$i-$pos)).'</a>';
					$in=false;
				}
				if(!$in) $text.=$c;

			}

			$messages[]=array(
				'type'=>'twitter',
				'userName'=>$feedData['id'],
				'text'=>$text,
				'time'=>$time,
				'icon'=>basename($iconFile)
			);

			// Rerieve account icon
			if(!isset($iconOk)){
				if(!file_exists($iconFile)){
					if(isset($feed['user']['profile_image_url'])){
						if($picture=cfSocketHTTPRequest($feed['user']['profile_image_url'],7,true)){
							file_put_contents($iconFile,$picture);
						}
						$iconOk=1;
					}
				}
				else $iconOk=1;
			}
		}
	}

	return $messages;
}

/**
 * @desc Return feeds from Facebook JSON result
 *
 * @param string $json: feed json encoded
 * @param string $facebookAccountData: facebook account data
 */
function wallParseFacebookFeeds($json,$feedData){

	$messages=array();
	if(!$json) return $messages;
	$parsed=json_decode($json,true);

	if(!isset($parsed['data'])) return;

	$facebookAccountData=facebookAccountGet();

	// Copy facebook icon into resource directory to ensure it will still be there if Facebook profile is disabled
	$facebookIconFile=cfAppResourceDir().'/facebookIcon.jpg';
	if(!file_exists($facebookIconFile)) @cfCreateResizedJPG(facebookProfilePictureFile(),0,0,$facebookIconFile,32,32);

	// Browse result
	foreach ($parsed['data'] as $item){
		// Keep only posts from "me"
		if(@$item['from']['id']==$facebookAccountData['id']){

			// Remove already added feeds
			$time=strtotime(@$item['created_time']);
			if($feedData['lastUpdateTime']>=$time) continue;
//cfDbg('last update: '.date('d/m/Y H:i:s',$facebookAccountData['lastUpdateTs']).' message time:'.date('d/m/Y H:i:s',$time));
			$link='';
			if(@$item['picture']){
				if($picture=cfSocketHTTPRequest($item['picture'],7,true)){
					// Find a name for image
					$imgIndex=0;
					while (file_exists($file=cfAppResourceDir().'/img.'.$imgIndex.'.jpg')) $imgIndex++;

					// Create image into resource data directory
					file_put_contents($file,$picture);
					list($w, $h, $type, $attr) = @getimagesize($file);
					// populate link with width & height & img name
					$link='img;'.$w.';'.$h.';img.'.$imgIndex.'.jpg';
				}
			}

			if(isset($item['message'])) $text=$item['message'];
			elseif (isset($item['caption'])) $text=$item['caption'];
			else $text=@$item['name'];
			$message=array(
				'type'=>'facebook',
				'userName'=>$facebookAccountData['name'],
				'text'=>@$item['message'],
				'time'=>$time,
				'link'=>$link,
				'icon'=>basename($facebookIconFile)
			);
			$messages[]=$message;
		}
	}

	return $messages;
}

/**
 * @desc Update Twitter / Facebook / RSS... messages
 *
 */
function wallUpdateExternalMessages(){
	global $db;

	$feeds=array();
	$urls=array();
	$messages=array();
	$lastUpdate=0;


	// Ensure this function is not called more often than W_WALL_FEED_MIN_UPDATE_INTERVAL seconds
	$lastUpdates=dbArrayQuery('SELECT * FROM updates',$db);
	if(is_array($lastUpdates)) foreach ($lastUpdates as $lu){if($lu['type']=='general') {$lastUpdate=$lu['ts'];break;}}
	if(time()-$lastUpdate<W_WALL_FEED_MIN_UPDATE_INTERVAL) return;
	dbExec('INSERT OR REPLACE INTO updates VALUES ("general","",'.time().')',$db);

	// Generate a list of twitter feeds
	foreach (explode("\t",cfRGetVar('twitterFeeds')) as $feedAccount) if($feedAccount) {
		// Get last successful update time
		$lastUpdateTime=time()-W_WALL_FEED_INITIAL_HISTORY_TIME;
		foreach ($lastUpdates as $lu) if($lu['type']=='twitter' && $lu['value']==$feedAccount) $lastUpdateTime=$lu['ts'];
		$feed=array(
			'id'=>$feedAccount,
			'url'=>'https://twitter.com/statuses/user_timeline/'.$feedAccount.'.xml',
			'type'=>'twitter',
			'lastUpdateTime'=>$lastUpdateTime,
			);
		$feeds[$feedAccount]=$feed;
	}

	// Generate facebook feed
	if(cfRGetVar('facebookWall') && $facebookAccount=facebookAccountGet()){

		// Get last successful update time
		$lastUpdateTime=time()-W_WALL_FEED_INITIAL_HISTORY_TIME;
		foreach ($lastUpdates as $lu) if($lu['type']=='facebook' && $lu['value']==$facebookAccount['id']) $lastUpdateTime=$lu['ts'];
		$feed=array(
			'id'=>$facebookAccount['id'],
			'url'=>facebookAPIRequestURL('me','feed'),
			'type'=>'facebook',
			'lastUpdateTime'=>$lastUpdateTime
			);
		$feeds['facebook']=$feed;
	}
	if(!count($feeds)) return; // No feeds => abort

	foreach ($feeds as $id=>$feed){$urls[$id]=$feed['url'];}

	// Get data
	$res=cfCURLMultiRequests($urls);

	// Browse result
	foreach ($res as $id=>$results) if($results) {
//		cfVarDump($feeds[$id]['type']);
		if($feeds[$id]['type']=='twitter') wallFeedsAddMessages(wallParseTwitterFeeds($results,$feeds[$id]),$feeds[$id]);
		if($feeds[$id]['type']=='facebook') wallFeedsAddMessages(wallParseFacebookFeeds($results,$feeds[$id]),$feeds[$id]);
	}

	/**
	 * Debug
	foreach ($messages as $m){
		echo '<img src="'.$m['icon'].'" style="float:left;width:32px;height:32px"><b>'.$m['text'].'</b><br><span style="font-size:80%">'.$m['time2'].'</span><br clear="all">';
	}
	 */
}

/**
 * Insert posts and comments templates
 *
 */
function wallTemplates(){
	$view=cfRGetVar('view');
?>
<script type="text/javascript">
<?php

	/**
	 * Board templates
	 */
	if($view=='board'){
?>
// Post template
wall.postTemplate='<div class="POSTCLASS" id="pPOSTID" style="STYLE" onmouseover="wall.postHover(this.id,1)">'+
	'<div class="PIN"></div>'+
	'<div class="wallDeleteHandle" onclick="wall.postDelete(this.parentNode.id.substr(1))"></div>'+
	'<div class="wallPostLink">LINK</div>'+
	'<div class="wallCont">'+
		'<div class="wallPostText">TEXT</div>'+
		'<div class="wallPostUser">USER</div>'+
		'NEWCOMMENT'+
		'<div id="commentsPOSTID" class="wallPostComments">'+
			'COMMENTS'+
		'</div>'+
	'</div>'+
'</div>';

// Comments template
wall.commentTemplate='<div class="CLASS" id="cID">'+
		'<div class="wallCommentDeleteHandle" onclick="wall.commentDelete(ID)"></div>'+
		'<img class="wallCommentIcon" src="ICON">TXT'+
		'<br clear="all">'+
		'<div class="wallCommentUser">USER</div>'+
		'</div>';

// New comment form
wall.newCommentTemplate='<div class="wallNewComment" id="ncdPOSTID">'+wl.capt.blogAddComment+'<textarea id="ncPOSTID" class="commentsTA" onkeyup="wall.newCommentKD(\'POSTID\',event)"></textarea></div>';
<?php
	}

	/**
	 * Classic templates
	 */
	else {
?>
// Post template
wall.postTemplate='<div class="frame2 POSTCLASS" id="pPOSTID" style="STYLE" onmouseover="wall.postHover(this.id,1)">'+
	'<div class="wallDeleteHandle" onclick="wall.postDelete(this.parentNode.id.substr(1))" <?php echo str_replace("'","\\'",outToolTip(cfCaption('genDelete'))); ?>></div>'+
	'<div class="wallPostLink">LINK</div>'+
	'<div class="wallCont">'+
		'<div class="wallPostText">TEXT</div>'+
		'<div class="wallPostUser">USER</div>'+
		'<div class="frame3">'+
		'NEWCOMMENT'+
		'</div>'+
		'<div id="commentsPOSTID" class="wallPostComments">'+
			'COMMENTS'+
		'</div>'+
	'</div>'+
'</div>';

// Comments template
wall.commentTemplate='<div class="CLASS" id="cID">'+
		'<div class="wallCommentDeleteHandle" onclick="wall.commentDelete(ID)" <?php echo str_replace("'","\\'",outToolTip(cfCaption('genDelete'))); ?>></div>'+
		'<img class="wallCommentIcon" src="ICON">TXT'+
		'<br clear="all">'+
		'<div class="wallCommentUser">USER</div>'+
		'</div>';

// New comment form
wall.newCommentTemplate='<div class="wallNewComment" id="ncdPOSTID">'+wl.capt.blogAddComment+'<textarea id="ncPOSTID" class="commentsTA" onkeydown="wall.newCommentKD(\'POSTID\',event)"></textarea></div>';
<?php
	}
?>
</script>
<?php
}

/**
 * @desc Sync page display
 *
 */
function wallDisplayPage(){
	global $db;

	$options=array();
	if(cfRGetVar('view')=='board') $options=array('noTheme'=>1);

	cfInsertHEAD(false,$options);

	// Hide delete control for non-admin
	if(!WEnv::user()->isAdministrator()){
		?>
<style type="text/css">
.wallDeleteHandle, .wallCommentDeleteHandle{display:none}
</style>
		<?php
	}
?>
<script type="text/javascript">
var isAdmin=<?php
echo (int)WEnv::user()->isAdministrator().';';
echo cfJSCaptionVar('blogAddComment');
echo cfJSCaptionVar('blogPostNewMessage');
echo cfJSCaptionVar('genPseudo');
echo cfJSCaptionVar('blogNoPost');
?>
commentCreateButton='<?php echo outButton(cfCaption('genSend'),'javascript:wall.commentCreate(POSTID)',outIcon('ok16')); ?>';
</script>
<?php
	echo cfScriptLink('wall.js');
	wallTemplates();
	echo '<link rel="stylesheet" type="text/css" href="'.cfRGetVar('view').'.css?v='.cfGGetVar('appVersion')."\">\n";
?>
</head>
<body class="<?php echo cfRGetVar('view')?>" <?php echo cfBodyRightClick();?>>
<?php echo cfScriptLink('wz_dragdrop.js');

// Pseudo input popup
?>
<div id="pseudoInputPopup" class="popup" style="display:none;position:absolute;margin:0px;text-align:left;vertical-align:top;width:400px">
<?php
echo outFrameHeaderTable("popupHeader",outImageIcon('pseudo').cfCaption('genPseudo'),outButton(false,'javascript:wall.pseudoInputPopupClose()',outIcon('close')),false,'cursor:move');
echo '<br>'.cfCaption('userTypeName').'<br><input type="text" id="wallPseudoInput" style="width:100%" class="textInput">';
echo '<div style="text-align:right">'.outButton(cfCaption('genOK'),'javascript:wall.pseudoInputPopupValidate(1)',outIcon('ok')).'</div>';
?></div><?php


// Main posts area
?>
<table style="table-layout:fixed;width:100% !important;padding:0;margin:0" cellpadding="0" cellspacing="0">
<tr>
	<td id="postAreaTd" class="frame1">
		<div id="postsArea"></div>
	</td>
<?php
// Right frame
?>
	<td id="wallRightFrameTd" class="<?php if(cfRGetVar('rightFrameFolded')) echo 'folded';?>">
	<div style="overflow:hidden;position:relative;width:100%;height:100%">
		<div id="wallRightFrame" class="vBanner">
		<div id="wallRightFrameHandle" onclick="wall.toggleRightFrameFolding()"></div>
<?php

	// New post form (and pseudo input popup)
	wallDisplayNewPostForm();

	// Calendar
	wallDisplayCalendar();

	/**
	 * Wall layout
	 */
	$layouts=array(
		'board'=>'board.png',
		'classic'=>'classic.png',
	);
	echo '<center>';
	foreach ($layouts as $key=>$icon){
		echo outBlockItem('combinedLayoutSwitch','<img src="'.$icon.'" alt="">','id="combinedLayoutSwitch'.$key.'"','margin-right:7px;margin-left:0px;padding:5px;',array('onclick'=>'wl.goURL(\'index.php?view='.$key.'\')'),(($key==cfRGetVar('view'))?'sel':'unsel'));
	}

?>
		<br>&nbsp;
		</center>
		</div>
		</div>
	</td>
</tr>
</table>
<?php echo cfDragRegisterItems();?>
<script type="text/javascript">
wall.view="<?php echo cfRGetVar('view');?>";
wall.postRefWidth="<?php echo W_WALL_REF_POST_WIDTH; ?>";
wall.pseudo="<?php echo str_replace('"',"''",cfRGetVar('pseudo'));?>";
wall.postAllowed=<?php echo (int)cfRGetVar('postAllowed');?>;
wall.commentAllowed=<?php echo (int)cfRGetVar('commentAllowed');?>;
wall.rightFrameFolded=<?php echo (int)cfRGetVar('rightFrameFolded');?>;
wall.language='<?php echo cfGGetVar('language');?>';
wall.postIconWidth=32;
wall.commentIconWidth=24;
wall.noPostItem=<?php echo json_encode(wallNoPostItem());?>;

wall.updateData(<?php echo wallUpdateJSData(); ?>);
wall.updateDisplay();
function winResized(){
	alignBottom(dgi('wallRightFrame'),dgi('wallRightFrameTd'),0)
	wall.recomputePostMargins();
}
wall.init();
</script>
<?php
}

if(isset($included)) return; // Included by upload.php to retrieve defines and functions

if(isset($_GET['wallPostImage'])) wallSendPostImage($_GET['wallPostImage']);

// Initialize wall
wallInit();

//wallDumpBase();exit;
wallProcessCommands();

// Update facebook & twitter feeds
wallUpdateExternalMessages();


// Display page
if(cfRGetVar('mobileView')) wallMobileDisplayPage(); else wallDisplayPage();

?>