<?php
/**
 * Music explorer V1
 *
 * Display an audio files explorer based on files located in a given directory (and subdirs)
 * Audio files list
 * Playlist functions
 * Audio player (streaming)
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

/**
 * @desc return first "cover" image of a given directory
 *
 * @param string $directory
 * @param boolean $includeFilename : if true also return JS prepared image name
 * @return string : img src, img name
 */
function directoryCover($directory,$imgWidth,$includeFilename=false){
	$cfn=efGetCoverFromFolder($directory,$score);

	if($score==-1) return '""'.(($includeFilename)?',false':'');
	return '"'.addslashes(cfExtImage($cfn,$imgWidth,$imgWidth)).'"'.(($includeFilename)?',"'.addslashes(cfProtectFilename(basename($cfn))).'"':'');
}

require_once(INCLUDE_DIR.'explorerFunctions.php');
require_once(INCLUDE_DIR.'outputFunctions.php');

define('DEFAULT_AUDIO_WINDOW_WIDTH',350);
define('DEFAULT_AUDIO_WINDOW_HEIGHT',425);
define('DEFAULT_PLAYLIST_WINDOW_HEIGHT',465);
define('DEFAULT_INLINEPLAYER_WIDTH',250);
define('DEFAULT_INLINEPLAYER_HEIGHT','115px');
define('DEFAULT_INLINE_COVER_WIDTH',80);
define('RIGHT_COLUMN_WIDTH',195);
define('DEFAULT_EXPLORER_COVER_WIDTH',80);

/**
 * @desc return JS code for async updating inline player cover and track info
 *
 * @param string $passedFile
 * @return string JS code
 */
function inlineUpdatePlayerInfoScript($completeFilename){
	$jsAction='inlineSetSongPlayed(';
	if(cfFileExtension($completeFilename)=='m3u')
		$jsAction.='"'.dirname($_SERVER['PHP_SELF']).'/playlist.jpg","'.cfCaption('explorerAudioPlaylist').'","-","-")';
	else{
		$id3=efGetAudioInfo($completeFilename);

		if(!isset($id3['artist']))$id3['artist']=cfCaption('explorerAudioUnknownArtist');
		if(!isset($id3['album'])) $id3['album']=cfCaption('explorerAudioUnknownAlbum');
		if(!isset($id3['title'])) $id3['title']=cfCaption('explorerAudioUnknownTitle');
		if(cfIsMobile())
			$w=max(DEFAULT_INLINE_COVER_WIDTH,min(cfGGetVar('userScreenWidth'),cfGGetVar('userScreenHeight'))/2);
		else
			$w=DEFAULT_INLINE_COVER_WIDTH;

		$jsAction.='"'.cfXMLEncodeProperty(cfExtImage($completeFilename,$w,$w)).'"';
		$jsAction.=',"'.cfStrTruncate(cfUTF8Encode($id3['artist']),20).'","'.cfStrTruncate(cfUTF8Encode($id3['album']),20).'","'.cfStrTruncate(cfUTF8Encode($id3['title']),20).'")';
	}
	return $jsAction;
}

/**
 * @desc Init data on 1st script call
 *
 */
function initAudioExplorerData(){
	if(!cfRGetVar('resourceParametersInitialized')){
		cfRInitVar('extensionFilter',false); // array of extensions of displayed files. Set to false if no filter on file extension
		cfRSetVar('fileTypeFilter',array('audio'));// array of file types of displayed files. Set to false if no filter on file type (see mime_type.php for list of file types)
		cfRSetVar('extensionFilter',array('jpg'=>1,'jpeg'=>1,'gif'=>1,'png'=>1,'tiff'=>1,'itc'=>1,'itc2'=>1)); // Also allow covers files

		cfRSetVar('playlistDisplayed',true); // Show main playlist
		cfRSetVar('needAudioInfo',true); // Indicate efGetDirContent to get id3 info
		if(!cfRGetVar('sortField')) cfRSetVar('sortField','no'); // Files grouping
		cfRSetVarArray('playlist'); // Reset main playlist
		if(cfRGetVar('defaultOutputBitRate')) cfRSetVar('outputBitRate',cfRGetVar('defaultOutputBitRate')); else cfRSetVar('outputBitRate',cfRGetVar('maxOutputBitRate'));
		cfRInitVar('audioPlayer','embeded');
		cfRInitVar('navigationTab',true); // Default show navigation + cover tab
		if(!cfRGetVar('efSortBy')) cfRSetVar('efSortBy','name'); if(!cfRGetVar('efSortOrder')) cfRSetVar('efSortOrder','asc');// sort files by ascending alphabetical order
		cfRSetVar('useHTTPXmlRequest',true); // Use HTTPXmlRequest for communication between browser and server

		cfRSetVar('defaultPlaylistWindowHeight',DEFAULT_PLAYLIST_WINDOW_HEIGHT);
		cfRInitVar('displayMultipleDownload',true); // Unfold multiple download box

		cfRSetVar('mainFileTableToggleAllowed',true); // Allow user to toggle between scroll-view or full-view


		if(!cfBGetVar('drag')) cfRSetVar('maxDragItems',0);

		// Inline player
		cfRInitVar('inlinePlayer',true);// Set inline player for frameless themes
		if(cfBGetVar('standaloneMediaPlayer')) cfRSetVar('inlinePlayer',false);// Unset inline player for devices that doesn't support it
		if(cfBGetVar('name')=='Wii') cfRSetVar('inlinePlayer',true);// ... but set it inline for Wii...

		// iPhone specific
		if(cfBGetVar('name')=='iPhone') {
			cfRSetVar('meta','<meta name="viewport" content="width=device-width, user-scalable=no, maximum-scale=1, minimum-scale=1">');
			cfRSetVar('onload','updateOrientation();scrollTo(0,1)');
		}
		if(cfBGetVar('noScrollDiv')) cfRSetVar('mainFileTableToggleAllowed',false);

		if(cfRGetVar('resourceConfigured')) cfRSetVar('resourceParametersInitialized',true); // indicate that parameters have been initialized
		else $errorMessageCritical=cfCaption('configErrorNotConfigured');
	}

	// Init treeview
	global $tree;
	if(cfRGetVar('treeViewAvailable')){
		cfRSetVar('treeViewDisplayed',true);
		$tree=new treeViewFolder(cfRGetVar('treeViewAvailable'));
		$tree=treeFolderLoad(cfRGetVar('treeViewAvailable'));
	}
	else cfRSetVar('treeViewDisplayed',false);
}

/**
 * @desc Async set audio player source from selected songs
 */
function inlineSendAudioPlaylist(){
	require_once(INCLUDE_DIR.'viewFunctions.php');


	if(isset($_POST['data1'])) $passedDir=cfUnprotectFilename($_POST['data1']); else $passedDir='';
	if(isset($_POST['data2'])) $passedFile=cfUnprotectFilename($_POST['data2']);

	if(@$passedFile=='*playlist*.m3u') $completeFilename=$passedFile;
	elseif(cfSharedMode('list')){
		$completeFilename=cfSharedCompleteFilename(cfJoinPathFile($passedDir,$passedFile));
	}
	else{
		// Get directory
		if(cfRGetVar('path')) $passedDir=str_replace('*resourceBasePath*',cfRGetVar('path'),$passedDir);

		// Get file
		if(cfRGetVar('path')) $passedFile=str_replace('*resourceBasePath*',cfRGetVar('path'),$passedFile);

		if($passedFile=='*playlist*.m3u') $completeFilename=$passedFile;
		else $completeFilename=cfCleanPathName(cfJoinPathFile($passedDir,$passedFile));
	}

	if(efIsPlaylist($completeFilename) && is_file($completeFilename)){
		require_once(INCLUDE_DIR.'fileInfoFunctions.php');
	//return array('completeFileName'=>'*resSpecific*/'.$track['id'].'.mp3', 'name'=>'*resSpecific*/'.$track['id'].'.mp3', 'label'=>cfUTF8Decode(playlistLabelFromTrackInfo($track),true,true,false), 'externalRef'=>$track['id'], 'artist'=>$track['Artist'], 'album'=>$track['Album'], 'track'=>$track['Name'], 'length'=>$track['TotalTime']*1000);

		$tracks=array();
		foreach (fiParsePlaylistFile($completeFilename,true,false) as $k=>$v){
			$tracks[]=array(
				'completeFileName'=>$v['cfn'],
				'name'=>basename($v['cfn'])
			);
		}

		cfRSetVar('playlistTmp',$tracks);

		$completeFilename='*tmp*playlist*.m3u';
	}


	// Generate shuffled playlist if needed (only apply to user-generated playlist)
	elseif(cfRGetVar('shufflePlaylist')){
		$shuffle=cfRGetVar('playlist');
		shuffle($shuffle);
		cfRSetVar('playlistShuffled',$shuffle);
		unset($shuffle);
	}

	// Send async response
	cfAsyncHeader();
	if($completeFilename=='*playlist*.m3u' || $completeFilename=='*tmp*playlist*.m3u' || cfFileRights($completeFilename,'download')){
		$script=false;
		$audioPlayer=vfInsertAudioPlayer($completeFilename,'normal',true,'95%',25,false,$script);
		echo cfAsyncXMLInnerHTMLbyId($audioPlayer,'flashPlayerAsyncOuterDiv'); // Insert new flash audio player
		if($script) echo cfAsyncJSaction($script); // If a script is returned (HTML5 audio), it overrides flash player
		echo cfAsyncXMLJSaction(inlineUpdatePlayerInfoScript($completeFilename));
	}
	cfAsyncFooter();
}

/*
 ***************************************************************************************************************************
 * Init audio explorer data
 ***************************************************************************************************************************
 */

// Define resource-vars that will be saved if user is singleUser (other ones included by efInit())
cfSetSavedRVar('audioPlayer','inlinePlayer','add','displayMultipleDownload');


if(!cfTGetVar('frames')) cfRSetVar('inlinePlayer',false);

// Init common explorer data
efInit();

// Init audio explorer data
initAudioExplorerData();


/*
 ***************************************************************************************************************************
 * Process POST commands
 ***************************************************************************************************************************
 */

// Inline play
if(cfRGetVar('inlinePlayer') && isset($_POST['data3']) && (($_POST['data3']=='view' && cfFileExtension($_POST['data2'])!='jpg' && cfFileExtension($_POST['data2'])!='jpeg' && cfFileExtension($_POST['data2'])!='gif' && cfFileExtension($_POST['data2'])!='bmp') || $_POST['data3']=='inlinePlayAll')){
	inlineSendAudioPlaylist();
}

// Update cover and song info displayed into inline player
if((cfIsMobile() || cfRGetVar('inlinePlayer')) && isset($_POST['data2']) && isset($_POST['data3']) && $_POST['data3']=='inlineUpdatePlayerInfo'){
	$completeFilename=cfFilenameResolve(base64_decode($_POST['data2']));

	cfAsyncHeader();
	if(cfFileRights($completeFilename,'state')=='authorized') echo cfAsyncXMLJSaction(inlineUpdatePlayerInfoScript($completeFilename));
	cfAsyncFooter();
}


/**
 * If mobile device, redirect to mobile explorer
 */
if(cfIsMobile()) {require(INCLUDE_DIR.'mobileExplorerFunctions.php');mefExplorer();exit;}


// Process common explorer commands
efProcessPOST();
cfAsyncXMLChangePhpVarValue('displayMultipleDownload'); // process hide / show playlist
cfAsyncXMLChangePhpVarValue('navigationTab'); // process hide / show navigation tab (1 shown, 0 hidden)
cfAsyncXMLChangePhpVarValue('forceHTML5AudioPlayer'); // Use HTML5 audio player instead of flash player
cfAsyncXMLChangePhpVarValue('outputBitRate',array('32','64','128','original')); // Update reencoded bitrate

/*
 ***************************************************************************************************************************
 * Get files and subfolders list, generate thumbnails if needed
 ***************************************************************************************************************************
 */
if(!cfRGetVar('noFileRefresh')) efDirUpdate();
cfRSetVar('noFileRefresh',false);

/*
 ***************************************************************************************************************************
 * Insert HEAD, JS functions and form
 ***************************************************************************************************************************
 */
efInsertExplorerHeadAndBody();

/*
 ***************************************************************************************************************************
 * Start of resource specific section
 ***************************************************************************************************************************
 */

$displayInlinePlayer=cfRGetVar('inlinePlayer');


// Put in $mdFiles an array : [artist - album name][track # - title]=file name
$mdFiles=array();
$allCaption=cfUTF8Decode(cfCaption('genAll'));
if(isset($_ENV['weezoResEfFiles']))
	foreach ($_ENV['weezoResEfFiles'] as $value) if(isset($value['type']) && $value['type']=='file' && cfFileExtension($value['name'])!='m3u'){
		if(isset($value['artist'])) $mdArtist=$value['artist']; else $mdArtist=' '.cfCaption('explorerAudioUnknownArtist');
		if(isset($value['album'])) $mdAlbum=$value['album']; else $mdAlbum=cfCaption('explorerAudioUnknownAlbum');
		if(isset($value['track'])) $mdTrack=$value['track']; else $mdTrack=false;
		if(isset($value['title'])) $mdTitle=$value['title']; else $mdTitle=cfFileWithoutExtension($value['name']);
		switch (cfRGetVar('sortField')) {
			case 'artist':
				if($mdTrack) $mdFiles[$mdArtist][$mdTrack.' - '.$mdTitle]=$value['name']; else $mdFiles[$mdArtist][$mdTitle]=$value['name'];
				break;
			case 'artistAlbum':
				if($mdTrack) $mdFiles[$mdArtist.' - '.$mdAlbum][$mdTrack.' - '.$mdTitle]=$value['name']; else $mdFiles[$mdArtist.' - '.$mdAlbum][$mdTitle]=$value['name'];
				break;
			case 'album':
				if($mdTrack) $mdFiles[$mdAlbum][$mdTrack.' - '.$mdTitle]=$value['name']; else $mdFiles[$mdAlbum][$mdTitle]=$value['name'];
				break;
			default: // No grouping, display filenames
				$mdFiles[$allCaption][$value['name']]=$value['name'];
				break;
		}
	}


/*
 ***************************************************************************************************************************
 * GENEARTE PLAYLISTS SONG LIST, DIRECTORIES AND COVER
 ***************************************************************************************************************************
 */
$arrayMainPlaylist=efPlayListContentArray();
// Only refresh playlist
if($asyncResponse=='plAdd' || $asyncResponse=='plSupp' || $asyncResponse=='mSuppAll' || $asyncResponse=='mAddAll' || $asyncResponse=='plUp' || $asyncResponse=='plDown'){
	echo cfAsyncXMLJSaction('plRefresh('.$arrayMainPlaylist.')');
	// Update buttons
	echo efMultipleDownloadUpdateBt();
	// Update reloading icon
	echo cfAsyncXMLJSaction('mProcessing(false)');
	die(cfAsyncFooter());
}

/*
 * M3U Playlists
 * Structure : Array(filename1, filename2,...)
 */
$itemsFound=false;
$arrayPlaylists='[';
if($_ENV['weezoResEfFiles']['nbFiles']) foreach($_ENV['weezoResEfFiles'] as $key=>$value) if(isset($value['name']) && efIsPlaylist($value['name'])){
	if($itemsFound) $arrayPlaylists.=',';
	$itemsFound=true;
	$arrayPlaylists.='"'.str_replace('"','\\"',cfProtectFilename($value['name'])).'"';
}
$arrayPlaylists.=']';

/*
 * Audio files
 * Structure : Array(
 *					Array(Artist/album1 label, filename1, label1, filename2, label2,...),
 *					Array(Artist/album2 label, filename1, label1, filename2, label2,...),
 *					...
 *					)
 */
$arrayAudioFiles='[';
$itemsFound=false;
reset($mdFiles);
uksort($mdFiles, "strnatcasecmp");

// Artist - Album
if(count($mdFiles)) foreach($mdFiles as $key=>$value){
	if($itemsFound) $arrayAudioFiles.=','; else $itemsFound=true;
	$arrayAudioFiles.='["'.stripslashes(cfUTF8Encode($key)).'"';
	// Track number - Song title
	uksort($value, "strnatcasecmp");
	foreach($value as $key2=>$value2) $arrayAudioFiles.=',"'.addslashes(cfUTF8Encode($key2)).'","'.str_replace('"','\"',cfProtectFilename($value2)).'"';
	$arrayAudioFiles.=']';
}
$arrayAudioFiles.=']';

// Get cover for current directory view
$cover=directoryCover(cfRGetVar('efCurrentDirectory'),DEFAULT_EXPLORER_COVER_WIDTH,true);


/*
 ***************************************************************************************************************************
 * Process POST
 ***************************************************************************************************************************
*/
if($asyncResponse){
	echo cfAsyncXMLJSaction('clearDraggableItems();');
	echo cfAsyncXMLInnerHTMLbyId(outClickablePath(cfResourceRelativePath(cfRGetVar('efCurrentDirectory')),false),'cPath');
	if(efFilesListIsResource()){
		echo cfAsyncXMLJSaction('refreshPlaylists('. $arrayPlaylists.');');
		echo cfAsyncXMLJSaction('refreshAudioFiles('. $arrayAudioFiles.');');
		echo cfAsyncXMLJSaction('rebuildDraggableItems();');
	}
	else efFilesList('audioFilesFrame');

	echo cfAsyncXMLJSaction('refreshCover('. $cover.');');
	// Set treeView
	if(cfRGetVar('treeViewDisplayed')) echo cfAsyncXMLJSaction($tree->updateAndSelectScript(cfRGetVar('efCurrentDirectory')).';');
	echo cfAsyncXMLJSaction('asyncSetPI(0)'); // Remove progress indicator
	cfAsyncFooter();
}



echo outTableTransparent('frame1',false,'id="mainFileTable"').'<tr><td id="" style="vertical-align:top">'."\n";

/*
 ***************************************************************************************************************************
 * INLINE PLAYER AREA
 ***************************************************************************************************************************
*/
if($displayInlinePlayer) {
	echo '<table class="banner" style="width:100%;margin:0;border-bottom:0"><colgroup><col width="'.DEFAULT_INLINEPLAYER_WIDTH.'"><col width="*"></colgroup><tr valign="top"><td style="padding-right:0.5em">';
	outDisplayInlineAudioPlayer();
	echo '</td><td>';
}

/*
***************************************************************************************************************************
* PLAYLIST AREA
***************************************************************************************************************************
*/
echo outMultipleDownloadList('frame1','folder',true,false,'padding-bottom:0px;'.((cfBGetVar('playlistFormat') || cfBGetVar('download'))?false:'display:none'),false,false,array('m3u'=>1,'inlinePlayerFold'=>1));

?>
<script language="javascript" type="text/javascript">
function updatePlaylistSize(){if(dgi('mDlList')) dgi('mDlList').style.width="100%";}
var mdlInt=W.setInterval("updatePlaylistSize()",1000);
</script>
<?php

if($displayInlinePlayer) echo '</td></tr></table>';


/*
 ***************************************************************************************************************************
 * EXPLORER AREA
 ***************************************************************************************************************************
 */

echo outDivFrame('frame1','id="explorerDiv"');

// Title and navigation links
if(cfTGetVar('frames')) $l=outResourceConfigBt(false); else $l='';
$l.='<span style="margin-right:1em">'.cfCaption('explorerAudioBrowse').'</span><span class="smallFont" id="cPath">'.outClickablePath(cfResourceRelativePath(cfRGetVar('efCurrentDirectory'))).'</span>';

// Files list and thumbnails size selectors
$r=efFilesListSelector().'&nbsp;&nbsp;&nbsp;'.outThumbnailsSizeButtons(true);

// File search
if(cfRGetVar('searchFiles')) $r.='&nbsp;'.outButton(false,'javascript:searchFiles()',outIcon('search'),cfCaption('genSearch'));

// Toggle display mode button
if(cfRGetVar('mainFileTableToggleAllowed')) echo outFrameHeaderTable('frame1Header',$l,$r);
else echo '<div class="frame1Header">'.$l."</div>\n";


// Group by control
if(efFilesListIsResource()){
?>
<span id="explorerGroupBy"><?php echo cfCaption('genGroupBy');
if(cfBGetVar('screenSize')!='monitor') echo '<br>'; else echo '&nbsp;';
?>
<input type="radio" name="groupOpt" id="groupArtist" onclick="changeGrouping(this.id)" <?php echo ((cfRGetVar('sortField')=='artist')?'checked="checked"':''); ?>><?php echo cfCaption('explorerAudioArtist'); ?>

<input type="radio" name="groupOpt" id="groupAlbum" style="margin-left:1em" onclick="changeGrouping(this.id)" <?php echo ((cfRGetVar('sortField')=='album')?'checked="checked"':''); ?>><?php echo cfCaption('explorerAudioAlbum'); ?>

<input type="radio" name="groupOpt" id="groupArtistAlbum" style="margin-left:1em" onclick="changeGrouping(this.id)" <?php echo ((cfRGetVar('sortField')=='artistAlbum')?'checked="checked"':''); ?>><?php echo cfCaption('explorerAudioArtist'); ?> / <?php echo cfCaption('explorerAudioAlbum'); ?>

<input type="radio" name="groupOpt" id="groupNo" style="margin-left:1em" onclick="changeGrouping(this.id)"<?php echo ((cfRGetVar('sortField')=='no')?'checked="checked"':''); ?>><?php echo cfCaption('genNoGroup'); ?>
</span>
<?php
}
?>
<script type="text/javascript">
var treeViewFrame="<?php if(cfRGetVar('treeViewAvailable')) echo $tree->HTMLDivID();?>";
function changeGrouping(gid){
	dgi(gid).checked="checked";
	fAS('','groupBy',gid);
	wl.mask.show(1);
	if(W.winUseWindows) winMe.setReloading();
}
</script>
<?php
// Display explorer table header (files & dirs)


echo '<div id="mainFileTableDiv" style="overflow:auto;position:relative;display:block"'.outEHoverFrameAttr().'>';

//
// DISPLAY PLAYLISTS
//

// Start tag for playlists
echo outDivFrame('frame2','id="m3uPlaylistsOuterFrame"','display:none').'<div class="frame2Header">'.cfCaption('explorerAudioPlaylists')."</div>\n";
echo '<div id="m3uPlaylistsInnerFrame"></div>';
echo "</div>\n\n";


//
// DISPLAY AUDIO FILES
//
echo '<div id="audioFilesFrame"></div>';
echo '</div></td>';



/*
 ***************************************************************************************************************************
 * fold/unfold navigation tab handle
 ***************************************************************************************************************************
 */
echo outCollapsableTD('navigationTab','right',array('preToggleJS'=>'resetDraggableItemsWidth()','postToggleJS'=>'winResized()'));


/*
 ***************************************************************************************************************************
 * Cover / Navigation tab
 ***************************************************************************************************************************
 */
echo '<td id="navigationTab" style="width:'.RIGHT_COLUMN_WIDTH.'px;padding-left:0.5em;vertical-align:top'.((cfRGetVar('navigationTab')?'':';display:none')).'" class="vBanner">';
// COVER
echo outDivFrame('frame2','id="coverDiv"','margin-top:0');
echo '<center><img id="coverImage" src="/gfx/v.gif" alt="Cover" style="padding:'.((RIGHT_COLUMN_WIDTH-DEFAULT_EXPLORER_COVER_WIDTH)/8).'"></center>';
echo '<span id="coverId" style="display:none"></span>';
echo "</div>\n\n";

// Navigation
if(cfRGetVar('treeViewDisplayed')){
	echo $tree->HTML('frame2',false,false,'width:'.(RIGHT_COLUMN_WIDTH).'px');
}
elseif(cfRGetVar('subFoldersIncluded')){
	echo outDivFrame('frame2').'<div class="frame2Header">'.cfCaption('explorerBrowse').'</div>';
	echo '<div style="position:relative; width:260px;height:0px;"></div>'; // = min-width...

	// Up Control
	echo '<div id="upBt" class="eTC" eH=1 onclick="go(\'..\');" style="width:100%;cursor:pointer;display:none">';
	echo outImage(outIcon('icoFolderUpBig'),false,false,'margin-right:1em;vertical-align:middle;');
	echo '<span>'.cfCaption('explorerParentDir').'</span></div>';
	// Subdirs frame
	echo '<div id="subdirsFrame"></div>';
	echo "</div>\n\n";
}
echo "</td>\n";
echo "</tr></table>\n\n";

echo "</div></div>\n\n";

// close topDiv
echo '</div>';

// innerHTML stuff
echo '<div style="display:none">';
echo '<div id="playBt">'.outButtonSmall('','javascript:play(\'ITEMID\',1)',outIcon('play'),cfCaption('explorerAudioPlay')).'</div>';
echo '<div id="plPlayBt">'.outButtonSmall('','javascript:plPlay(\'ITEMID\','.DEFAULT_AUDIO_WINDOW_WIDTH.','.DEFAULT_AUDIO_WINDOW_HEIGHT.')',outIcon('play'),cfCaption('explorerAudioPlay'),false,'style="margin-right:0.1em"').'</div>';
echo '<div id="plSuppBt">'.outButtonSmall('','javascript:plSupp(\'ITEMID\')',outIcon('eraser'),cfCaption('genRemove'),false,'style="margin-right:1em"').'</div>';
echo '<div id="plAddBt">'.outButtonSmall('','javascript:plAdd(\'ITEMID\')',outIcon('add'),cfCaption('explorerAudioPlaylistAdd')).'</div>';
echo '<div id="dlBt">'.outButtonSmall('','javascript:dl(\'ITEMID\')',outIcon('dl'),cfCaption('genDownload')).'</div>';
echo '</div>';


/*
 ***************************************************************************************************************************
 * PAGE CONSTRUCTION JAVASCRIPT
 ***************************************************************************************************************************
 */

?>
<script type="text/javascript">
var bp=dgi("playBt").innerHTML;
var ba=dgi("plAddBt").innerHTML;
var bd=dgi("dlBt").innerHTML;
var nbFiles=0;
var nbPlaylists=0;
var seqId=0;
var savedElements=false;
var dragItems=new Array();
var bodyLoaded=false;

// Window resized
function initFrames(){
	bodyLoaded=1;
	if(phpBrowser=='ie') dgi("<?php echo cfRGetVar('treeViewAvailable');?>_outer").style.width=dgi('coverDiv').offsetWidth+'px';
	refreshCover(<?php echo $cover;?>);
	winResized();
}
function winResized(w,h){
	dgi('mainFileTable').style.width="100%";
	if(!bodyLoaded) return;
	var mh=winMe.getInnerHeight();
	if(dgi("noFilesDiv")) setActualHeight(dgi("noFilesDiv"),Math.max(mh-actualOffsetTop(dgi("noFilesDiv")) - 25,100));
	dgi('coverDiv').style.display=(winMe.getInnerHeight()<600)?'none':'';

	alignBottom(dgi('mainFileTableDiv'),winMe,20)

	if(dgi(treeViewFrame)){
		dgi(treeViewFrame).style.height='100%';
		alignBottom(dgi(treeViewFrame),dgi('mainFileTableDiv'))
		<?php if(cfRGetVar('treeViewAvailable')) echo $tree->JSItem().'.adjustInnerDiv();'; ?>
	}
}
function onCollapseFunction(a,b){winResized()}


// Normal/IE png image
function image(src, extra){
	<?php
	if(cfGetBrowser()=='ie' && stripos(($_SERVER['HTTP_USER_AGENT']), "msie 7.0")==false) echo 'if(src.substr(src.length-4).toLowerCase()==".png") return \'<span style="filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src=\\\'\'+src+\'\\\', sizingMethod=\\\'image\\\'); width:16px; height:16px;" \' + extra + \'></span>\';'."\n";
	?>
	return '<img src="'+src+'" '+extra+' alt=""/>';
}
function inlineUpdatePlayerInfo(cfn){fAS(cfn,'inlineUpdatePlayerInfo')}

// m3u Playlists
function refreshPlaylists(fa){
	var buff='';
	nbPlaylists=fa.length;
	if(fa.length==0) {dgi("m3uPlaylistsOuterFrame").style.display="none"; return;}
	for(var i=0;i<fa.length;i++){
		buff+='<div id="_id'+seqId+'" class="eTC" eH=1 style="width:98%">';
		buff+=bp.replace(/ITEMID/g,'id'+seqId);
		buff+='<span style="margin-left:1em" <?php if(cfHGetVar('tooltipFileInfo')){?> onmouseover="tooltip(this.parentNode)"<?php } ?>>'+fa[i].replace(/.m3u/gi,'')+'</span>';
		buff+='</div>';
		buff+='<span id="id'+seqId+'" style="display:none">'+fa[i]+'</span>';
		seqId++;
	}
	dgi("m3uPlaylistsInnerFrame").innerHTML=buff;
	dgi("m3uPlaylistsOuterFrame").style.display="block";
}
// Play audio file or playlist
function play(id,force){
	if(dragCancelOnclick && !force) return dragCancelOnclick=0;
	if(!force && mouseX < actualOffsetLeft(dgi('_'+id).lastChild)) return;
	W.setTimeout("view('"+id+"',<?php echo DEFAULT_AUDIO_WINDOW_WIDTH.','.DEFAULT_AUDIO_WINDOW_HEIGHT;?>)",100);
}
// Called on search result
function insertFiles(){
	var i, t, f, n;
	if(!fa) t=[];
	else{
		t=[''];
		for(i=0;i<fa.length;i++) {
			f=fa[i];
			if(f[1]!='f') continue;
			n=f[2]; n=n.substr(n.lastIndexOf('/')+1); n=n.substr(0,n.lastIndexOf('.'));
			t.push(n); t.push(f[2]);
		}
		t=[t];
	}
	refreshPlaylists([]);
	refreshAudioFiles(t);
}

// Audiofiles
function refreshAudioFiles(fa){
	var buff='',o,aDI=[],i;
	nbFiles=fa.length;
	if(fa.length==0) {
		buff='<div class="frame2" id="noFilesDiv" style="margin:0px"><div class="frame2Header"><?php echo stripslashes(cfCaption('explorerEmptyDir'));?></div><center style="position:relative;top:40%"><?php echo stripslashes(cfCaption('explorerAudioNoFile'));?></center></div>';
	}
	else for(i=0;i<fa.length;i++){
		group=fa[i];
		for(o=0;o<group.length;){
			if(o==0){
				buff+='<?php echo stripslashes(outDivFrame('frame2'));?><div class="frame2Header">'+group[0]+'</div>';
				o=1;
			}
			else{
				buff+='<div id="_id'+seqId+'" class="eTC'+((seqId%2==1)?' even':'')+'" eH=1 style="width:98%;position:relative;" onclick="play(\'id'+seqId+'\')">';
				buff+=bp.replace(/ITEMID/,'id'+seqId);
				<?php if(cfBGetVar('playlistFormat')){	?>buff+=ba.replace(/ITEMID/,'id'+seqId); <?php } ?>
				<?php if(cfRGetVar('downloadShortcut')){?>buff+=bd.replace(/ITEMID/,'id'+seqId); <?php } ?>
				buff+='<span class="mainCapt" style="margin-left:1em" <?php if(cfHGetVar('tooltipFileInfo')){?> onmouseover="tooltip(this.parentNode)"<?php } ?>>'+group[o]+'</span>';
//cfDbgOnce(fa[i][o])
		//		buff+=wl.ratingItem(i,f[15],0,{'extraStyle':'margin-left:1em;float:right'})+'<'+'/td>';

				buff+='</div>';
				buff+='<span id="id'+seqId+'" style="display:none">'+group[o+1]+'</span>';
				if(phpBrowser!='ie') addDraggableItem("_id"+seqId);
				aDI.push(seqId);
				seqId++;
				o+=2;
			}
		}
		if(o) buff+='</div>';
	}

	dgi("audioFilesFrame").innerHTML=buff;

	winResized();

	// Chrome outside-browser drag & drop (user must press Alt key to prevent default drag):
	// add an ondragstart event to downloadable nodes, that will add DownloadURL to dataTransfer item
	for(i=0; i<aDI.length; i++) dgi('_id'+aDI[i]).onmouseover='';
	<?php if(cfRGetVar('downloadShortcut')){?> for(i=0; i<aDI.length; i++) altDragDownloadSet(dgi('_id'+aDI[i]),dlURL('id'+aDI[i])); <?php } ?>
}

// Subdirectories
function refreshDirFiles(fa){
	if(fa.length==0) return;
	var buff='';
	for(var i=0;i<fa.length;i++){
		group=fa[i];
		if(group=='..') dgi("upBt").style.display="block";
		else {
			if(i==0) dgi("upBt").style.display="none";
			buff+='<div id="p'+seqId+'" class="eTC" '+((seqId%2==1)?'lang="1"':"")+' eH=1 onclick="go(\'id'+seqId+'\');" style="width:98%; cursor:pointer;display:block;overflow:hidden">';
			if(group[0]=='f') buff+='<?php echo str_replace("'","\\'",outImage(outIcon("icoFolderBig"),false,false,"margin-right:1em;vertical-align:middle;float:left"));?><span id="id'+seqId+'" style="display:none">'+group[1]+'</span><span>'+group[1]+'</span>';
			else buff+=image(group[0],'alt="" style="margin-right:1em;vertical-align:middle;"')+'<span>'+group[1]+' '+group[2]+'</span><span id="id'+seqId+'" style="display:none">'+group[1]+'</span>';
			buff+='</div>';
			seqId++;
		}
	}
	dgi("subdirsFrame").innerHTML=buff;
}
// Cover
function refreshCover(src,filename){
	var ci=dgi("coverImage");
	if(!ci) return;
	if(src){
		ci.src=src;
		if(filename) dgi("coverId").innerHTML=filename; else dgi("coverId").innerHTML='#';
	}
	else {
		ci.src='/gfx/cover.png';
		dgi("coverId").innerHTML="#";
	}
	ci.style.width='<?php echo DEFAULT_EXPLORER_COVER_WIDTH;?>px';
	ci.style.height='<?php echo DEFAULT_EXPLORER_COVER_WIDTH;?>px';
	dgi('coverDiv').style.display=(winMe.getInnerHeight()<600)?'none':'';
}
function rebuildDraggableItems(){
	if(dragItems.length><?php echo cfHiGetVar('maxDragItems',80); ?>) return;
	for(var i=0; i<dragItems.length; i++){
		try{ADD_DHTML(dragItems[i]+ELASTIC+CURSOR_HAND+DETACH+DETACHOFFSETX+2+DETACHOFFSETY+2+PICKDELAY+400,1);} catch(err){}
	}
	dd.mkWzDom();
}
function clearDraggableItems(){
	for(var i=dd.elements.length-1;i>=0;i--) {
		if(dd.elements[i].name.substr(0,3)=='_id') dd.elements[i].del(true);
	}
	dragItems=new Array();
}
function resetDraggableItemsWidth(){
	var i,e=dd.elements;
	for(i=e.length-1;i>=0;i--) 	if(e[i].name.substr(0,3)=='_id') e[i].div.style.width='98%';
}
function addDraggableItem(fId){dragItems[dragItems.length]=fId;}
function err(nv,f,l){
	wl.mask.hide();
	return true;
}
function inlineSetSongPlayed(imgSrc,artist,album,title){
	if(imgSrc) dgi('inlinePlayerImg').src=imgSrc; else dgi('inlinePlayerImg').src="<?php echo outIcon('cover'); ?>";
	dgi('inlineArtist').innerHTML=artist;
	dgi('inlineAlbum').innerHTML=album;
	dgi('inlineTitle').innerHTML=title;
}
function tooltipGetContent(nodeId){
	if((W.dragInProgress && dragInProgress)) return false;
	fAS(dgi(nodeId.substr(1)).innerHTML,'getFileInfo',1,'',0);
	return '<img id="tooltipLoading" src="">';
}

plRefresh(<?php echo $arrayMainPlaylist;?>);
<?php
// Resource-specific files display
if(efFilesListIsResource()){
?>
refreshPlaylists(<?php echo $arrayPlaylists;?>);
refreshAudioFiles(<?php echo $arrayAudioFiles;?>);
<?php }
elseif (!cfIsAsync()) efFilesList('audioFilesFrame',array('noScriptTag'=>1));
?>
if(!savedElements) savedElements=dd.elements;
rebuildDraggableItems();
W.onerror=err;
efCaptMAdd="<?php echo stripslashes(cfCaptionJS('explorerAudioPlaylistAdd'));// Replace mAdd caption for context menu?>";
efCaptNoFile="<?php echo stripslashes(cfCaptionJS('explorerAudioNoFile'));// Replace caption for empty dir?>";
<?php echo cfDragRegisterItems('SCROLL',false);?>
initFrames()
</script>
<?php outKeysManager(); ?>

</body></html>