<?php
/**
 * iTunes database - based music explorer configuration file
 *
 * Configuration of music explorer resource
 * this script is included by /res/administration/std/resourceConfig.php
 * this script should never be directly called.
 *
 * resourceConfig.php sets 2 variables :
 * $resourceData (array) which contains resource parameters loaded from user.res file
 * $_ENV['configurationEnvironment'] which indicates if resourceConfig.php has been called from browser ('browser')
 * or from application's embeded browser ('application')
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.5
 */

if(!isset($source)) $source='iTunes';

/*
 ***************************************************************************************************************************
 * Verify access rights
 ***************************************************************************************************************************
 */
require_once(INCLUDE_DIR.'resourceConfigFunctions.php');
rcConfigRights(false);

//#############################################################################################################
// Process received commands
//#############################################################################################################

// Insert Javascript function needed for save button activation, and <form> with hidden fields.
rcInsertScriptAndForm();

// Resource icon & name
rcInsertIconName($resourceData,false,@$options);

// Source
echo '<div class="frame2'.(($source=='files')?' highlighted':'').'">';

switch ($source){
	// iTunesdatabase file
	case 'iTunes':
		if(WEnv::host()->isOSX()) echo '<div style="display:none">';
		echo '<div class="frame2Header">'.cfCaption('musicITunesFile','iTunes').'</div>';

		// Backward compatibility: set iTune file from set folder
		if(!isset($resourceData['musicITunesFile']) && isset($resourceData['musicITunesFolder'])) $resourceData['musicITunesFile']=$resourceData['musicITunesFolder'].'/iTunes Music Library.xml';

		// iTunes xml library file

		rcProcessData('musicITunesFile',dataFilePath,WEnv::host()->specialFolderMusic().'/iTunes/iTunes Music Library.xml','iTunes DB (iTunes Music Library.xml)',true);
		$resourceData['musicITunesFolder']=dirname($resourceData['musicITunesFile']);
		if(WEnv::host()->isOSX()) echo '</div>';

		// Verify that audio files actually exist
		rcProcessData('checkFilesExist',dataBoolean,true,false,false/*do not display control*/);

		// Database update method
		rcProcessData('databaseUpdateMethod',dataList,'automatic',array('automatic','manual'),array(
		array(false,cfCaption('musicRefreshingDatabase')),array(
		array('automatic',cfCaption('genAutomatic')),
		array('manual',cfCaption('configStartManual'))),false),true);
		echo '</div>';

		break;


	// Winamp database file
	case 'winamp':
		echo '<div class="frame2Header">'.cfCaption('musicITunesFile','Winamp').'</div>';

		if(isset($resourceData['musicWinampFolder'])){
			// 1st check in program files location (<=XP)
			if(file_exists(cfGGetVar('specialFolderProgramFiles').'/Winamp/Plugins/ml/main.dat'))
				$resourceData['musicWinampFolder']=cfGGetVar('specialFolderProgramFiles').'/Winamp/Plugins/ml/main.dat';
		}
		rcProcessData('musicWinampDatabaseFile',dataFilePath,cfGGetVar('specialFolderAppData').'/Winamp/Plugins/ml/main.dat','Winamp DB (main.dat)',true);
		echo '<br>';

		// Verify that audio files actually exist
		rcProcessData('checkFilesExist',dataBoolean,true,false,false/*do not display control*/);

		// Database update method
		rcProcessData('databaseUpdateMethod',dataList,'automatic',array('automatic','manual'),array(
		array(false,cfCaption('musicRefreshingDatabase')),array(
		array('automatic',cfCaption('genAutomatic')),
		array('manual',cfCaption('configStartManual'))),false),true);

		rcProcessData('path',dataText,'*resSpecific*',false,false/*do not display control*/); // Set path so it is compatible with /ext/image.php for mp3-embeded covers

		break;

	// Files
	case 'files':
		echo '<div class="frame2Header">'.cfCaption('genFiles').'</div>';

		$defaultDir=(cfGGetVar('specialFolderMyMusic'))?cfGGetVar('specialFolderMyMusic'):cfAppSharedDir();
		rcProcessData('path',dataFolderPath,$defaultDir,'noComputerRoot',true);

		// Subdirectories included
		$_POST['subFoldersIncluded']='true';// Force value to true as disabled checkbox control is treated as a false
		rcProcessData('subFoldersIncluded',dataBoolean,true,false,cfCaption('explorerCfgSubDirectoriesIncluded'));

		// Database update method
		rcProcessData('databaseUpdateMethod',dataText,'manual',false,false); // Do not display control

		break;
}
echo '</div><br>';



/*  Advanced config section
    vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv*/
rcAdvancedConfigStart(array('noTopSeparator'=>($source=='files')));

echo '<br>';

// Download shortcut
rcProcessData('downloadShortcut',dataUserRights,'none',false,cfCaption('explorerDownloadShortcut').'&nbsp;&nbsp;&nbsp;'.outHelpButton('helpSDL'), false);

// Multiple download
rcProcessData('multipleDownloadAllowed',dataUserRights,'none',false,cfCaption('configAllowMultipleDownload').'&nbsp;&nbsp;&nbsp;'.outHelpButton('helpMD'),false);

// Create and store playlist
rcProcessData('storePlaylistAllowed',dataUserRights,'admin',false,cfCaption('storePlaylistAllowed').'&nbsp;&nbsp;&nbsp;');

echo '<br>';

echo '<div class="configSubFrame">';
echo '<div class="configSubFrameHeader">'.cfCaption('genDisplay').'</div>';

// Default items per page
rcProcessData('itemsPerPage',dataList,'100',array('10','20','50','100','200','500','0'),array(
cfCaption('explorerItemsPerPage'),array(
array('10','10'),
array('20','20'),
array('50','50'),
array('100','100'),
array('200','200'),
array('500','500'),
array('0',cfCaption('genAll'))),false), false);

echo '&nbsp;&nbsp;&nbsp;&nbsp;';

// Default items per page
rcProcessData('groupBy',dataList,'Artist',array('Artist','Album','Playlist'),array(
cfCaption('genGroupBy'),array(
array('Artist',cfCaption('explorerAudioArtist')),
array('Album',cfCaption('explorerAudioAlbum')),
array('Playlist',cfCaption('explorerAudioPlaylist'))),false));

// Layout
echo '<br><div class="borderedImage">'.cfCaption('defaultLayout').'&nbsp;&nbsp;&nbsp;&nbsp;';
rcProcessData('displayType',dataList,'displayCovers',array('displayCompact','displayFull','displayCovers','displayCoversExt'),array(
array(outIcon('musicLayoutList'),'displayCompact','&nbsp;'),
array(outIcon('musicLayoutFull'),'displayFull','&nbsp;'),
array(outIcon('musicLayoutCovers'),'displayCovers','&nbsp;'),
array(outIcon('musicLayoutCoversExt'),'displayCoversExt','&nbsp;')
));
echo '</div>';
echo '</div>';


rcProcessData('fullviewCoverWidth',dataNumeric,80,false,false/*do not display control*/);		// Cover view artwork width & height
rcProcessData('fullviewCoverExtWidth',dataNumeric,160,false,false/*do not display control*/);	// Extended cover view artwork width & height


// File view (image, audio,...)
rcProcessData('fileViewAllowed',dataBoolean,true,false,false); // Allow/forbid user to listen to audio files

// File access parameters (apply to files and folders located in "path")
rcProcessData('downloadAllowed',dataBoolean,true,false,false/*do not display control*/); // Allow/forbid read access to files and folders
rcProcessData('writeAllowed',dataBoolean,false,false,false/*do not display control*/); // Allow/forbid write access to files and folders
rcProcessData('modifyAllowed',dataBoolean,false,false,false/*do not display control*/); // Allow/forbid modifcation access to files and folders (need writeAllowed to true)
rcProcessData('executeAllowed',dataBoolean,false,false,false/*do not display control*/); // Allow/forbid execute access to files
rcProcessData('uploadAllowed',dataBoolean,false,false,false/*do not display control*/); // Allow/forbid file upload
rcProcessData('remoteDownloadAllowed',dataBoolean,false,false,false/*do not display control*/); // Allow/forbid remote download of files

rcProcessData('userTempDirAccessAllowed',dataBoolean,false,false,false/*do not display control*/); // Allow/forbid access to user temp directory (data/sessiondata/(session_id)/)
rcProcessData('resourceDataDirAccessAllowed',dataBoolean,true,false,false/*do not display control*/); // Allow/forbid access to resource directory (data/res/(res type)/(res subType)/(res file name)/)

// Audio player position
echo '<div class="configSubFrame">';
echo '<div class="configSubFrameHeader">'.cfCaption('musicCfgMaxBr').'</div>';

// Force inline player
rcProcessData('inlinePlayer',dataBoolean,true,false,false/*do not display control*/);

// Number of songs to put in global library random playlist
rcProcessData('plAddAllShuffleSize',dataNumeric,50,false,false/*do not display control*/);

echo '<br/>';

// Default output bit rate
// Maximum output bit rate
rcProcessData('maxOutputBitRate',dataList,'original',array('original','128','64','32'),array(
array(rcResourceImage('cfgMaxBitrate.png'),cfCaption('musicCfgMaxBr')),array(
array('original',cfCaption('musicCfgNoLimit')),
array('128',cfCaption('explorerAudio128')),
array('64',cfCaption('explorerAudio64')),
array('32',cfCaption('explorerAudio32'))),false), false);
echo outHelpButton('helpAudioMaxBR');
echo '</div>';

rcProcessData('flashPlayerBufferLength',dataNumeric,3,false,false/*do not display control*/); // Default buffer time for flash audio player (seconds)
rcProcessData('flashPlayerDefaultVolume',dataNumeric,70,false,false/*do not display control*/); // Default audio volume for flash audio player (percent)

// Layout
rcProcessData('filesLayout',dataText,'resource',false,false/*do not display control*/); // Default files layout: resource specific */
rcProcessData('filesLayoutLocked',dataBoolean,false,false,false/*do not display control*/); // Do not lock files layout (allow user to change) */

// End of advanced config panel
rcAdvancedConfigEnd();

// Comments panel
echo rcCommentsPanelControl();

// Theme
echo rcThemeSelectionControl();

// Save / cancel buttons
echo '<br/>'.rcButtonSaveCancel();

// If resource need access to resource data dir, create it
rcCreateResourceDataDir();

// Indicate that resource has been initialized
rcProcessData('resourceConfigured',dataBoolean,true,false,false/*do not display control*/);

// Save parameters
rcWriteResourceFile($resourceData, $fileName);
?>
<style type="text/css">
.borderedImage img{border:1px solid #CCC}
</style>
<script language="javascript" type="text/javascript">
function dataInitForm(){
<?php
if($source=='iTunes' && !file_exists(str_replace('*appDir*',cfAppDataRootDir(),$resourceData['musicITunesFolder']).'/iTunes Music Library.xml')) echo 'dgn("musicITunesFile").style.color="red";';
if($source=='winamp' && !file_exists(str_replace('*appDir*',cfAppDataRootDir(),$resourceData['musicWinampDatabaseFile']))) echo 'dgn("musicWinampDatabaseFile").style.color="red";';
if($source=='files') echo "dgn('subFoldersIncluded').disabled=true;";
?>
}
dataInitForm();
</script>