<?php
/**
 * WEBCAM
 *
 * Display a page with webcam image
 * Page refreshes itself to update image
 * webcam image is obtained through cfServerSendCommand function
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */

/**
 * @desc Process POST command
 *
 */
function resWebcamProcessPOST(){

	// Toggle use of flash player as webcam viewer
	if(isset($_POST['toggleFlash'])) cfRSetVar('useFlashPlayer',!cfRGetVar('useFlashPlayer'));

	// Process image-only refresh requested by HTTPXmlRequest
	if(isset($_POST['asyncRefresh']) || isset($_POST['asyncRequest']) || isset($_GET['asyncRefreshXML'])){
		// Flash query
		if(isset($_GET['asyncRefreshXML'])){
			header('P3P: policyref="/w3c/p3p.xml",CP="NON COR CURa OUR NOR NAV"');
			header('Content-Type:text/xml');
			header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
			$output='<response ';
		}
		// Ajax query
		else cfAsyncHeader();

		// Retreive available space (mobiles)
		if(isset($_POST['viewportWidth']) && is_numeric($_POST['viewportWidth']) && isset($_POST['viewportHeight']) && is_numeric($_POST['viewportHeight'])){
			cfRSetVar('userScreenWidth',$_POST['viewportWidth']);
			cfRSetVar('userScreenHeight',$_POST['viewportHeight']);
		}

		// Set output image width & height
		$w=0;$h=0;
		if(cfIsMobile('wii')){
			$w=cfRGetVar('userScreenWidth');
			$h=cfRGetVar('userScreenHeight');
			cfRSetVar('windowSizeSet',1);
		}
		// Webcam image found
		$webcamDetected=false;
		if(($handle = @opendir(cfAppResourceDir()))) {

			// Browse webcam directory for .jpg files
			while (false !== ($file = @readdir($handle)) && !$webcamDetected) {
				if(cfFileExtension($file)=='jpg'){
					if(!cfRGetVar('windowSizeSet') && !cfIsMobile()){
						// Get webcam output size by checking image size
						list($width, $height, $type, $attr) = getimagesize(cfJoinPathFile(cfAppResourceDir(),$file));

						// If image OK
						if($width>1 && $height>1) {
							// Resize window to webcam image size
							if(isset($_GET['asyncRefreshXML'])) $output.='setImageSize="'.($width).','.($height).'" ';
							else echo cfAsyncXMLJSaction('setImageSize('.($width).','.($height).');');
							cfRSetVar('windowSizeSet',1);
						}
						else{
							if(isset($_GET['asyncRefreshXML'])) $output.='noWebcamDetected="1" ';
							echo cfAsyncXMLJSaction('noWebcamDetected();');
						}
					}
					if(cfRGetVar('windowSizeSet')){
						// Mobiles: fit image to screen
						if(cfIsMobile()){
							list($width, $height, $type, $attr) = getimagesize(cfJoinPathFile(cfAppResourceDir(),$file));
							list($w,$h)=cfThumbnailDimensions($width,$height,$w,$h,false);
							echo cfAsyncXMLJSaction('setImageSize('.$w.','.$h.');');
						}

						/**
						 * Update webcam image
						 */

						// Flash
						if(isset($_GET['asyncRefreshXML'])) $output.='setImage="'.cfExtImage(cfJoinPathFile(cfAppResourceDir(),$file),$w,$h,false).'" ';
						// HTML
						else echo cfAsyncXMLJSaction('setImage("'.cfExtImage(cfJoinPathFile(cfAppResourceDir(),$file),$w,$h,false,false,array('rnd'=>rand())).'");');

						// Update time caption
						if(cfIsMobile()) $dateFormat=cfCaption('_HMS_DATE_FORMAT'); else $dateFormat=cfCaption('_FULL_DATE_FORMAT');
						if(isset($_GET['asyncRefreshXML']))
							$output.='setTimeCaption="'.cfUTF8Encode(date($dateFormat, strtotime(cfFileWithoutExtension($file)))).'" ';
						else
							echo cfAsyncXMLJSaction('setTimeCaption("'.cfUTF8Encode(date($dateFormat, strtotime(cfFileWithoutExtension($file)))).'")');
					}
					$webcamDetected=true;
					break;
				}
			}
			closedir($handle);
			if(!$webcamDetected) echo cfAsyncXMLJSaction('noWebcamDetected();');
		}
		if(isset($_GET['asyncRefreshXML'])) die($output.'>OK</response>'); else die(cfAsyncFooter());
	}

}


// Define resource-vars that will be saved if user is singleUser
cfSetSavedRVar('useFlashPlayer');

// Init resource
if(!cfRGetVar('parametersLoaded')) {
	// Remove previous .jpg images (may result of a previous session)
	foreach (cfGlob(cfAppResourceDir().'/*.jpg') as $cfn) @unlink($cfn);

	cfRSetVar('refreshRate',str_replace(',','.',cfRGetVar('refreshRate')));
	if(!is_numeric(cfRGetVar('refreshRate'))) cfRSetVar('refreshRate',3);

	// Use flash version for browser supporting flash
	cfRInitVar('useFlashPlayer',true);
	if(!cfBGetVar('flash')) cfRSetVar('useFlashPlayer',false);

	// Wait for application's response for 1st image
	cfServerSendCommand('webcam dir="'.cfAppResourceDir().'"','sync');

	cfRSetVar('parametersLoaded',true);
}
// request webcam image update
else cfServerSendCommand('webcam dir="'.cfAppResourceDir().'"');


/*
 ***************************************************************************************************************************
 * Process POST commands
 ***************************************************************************************************************************
 */
resWebcamProcessPOST();

/*
 ***************************************************************************************************************************
 * Mobiles
 ***************************************************************************************************************************
 */
if(cfIsMobile('wii')) {require(dirname(__FILE__).'/mobileIndex.php');exit;}



/*
 ***************************************************************************************************************************
 * Insert HEAD, JAVASCRIPT functions and form
 ***************************************************************************************************************************
 */
cfInsertHEAD(false);
require(INCLUDE_DIR.'outputFunctions.php');

// Reset window size so next async update reset image size & position
cfRSetVar('windowSizeSet',0);
?>
<script type="text/javascript">
var refreshRate=<?php echo (1000*cfRGetVar('refreshRate'));?>;
var useFlashPlayer=<?php echo (int)cfRGetVar('useFlashPlayer');?>;
var resName="<?php echo addslashes(cfUTF8Encode(cfRGetVar('name')));?>";
</script>
<script language="VBScript">
On Error Resume Next
Sub playerWebcam_FSCommand(ByVal command, ByVal args)
	Call playerWebcam_DoFSCommand(command, args)
End Sub
</script>
</head>
<?php
/**
 * BODY
 */
?>
<body onorientationchange="updateOrientation()" onload="init()" style="overflow:hidden">
<form name="toggleFlashForm" method="POST" style="display:none"><input type="text" name="toggleFlash" value="1"></form>
<div id="topDiv" style="height:100%;">

<table class="frame1" height="100%" width="100%" border="0" cellspacing=0 cellpadding=0 style="margin:0px;padding:0px;vertical-align:middle"><tr>
<td style="text-align:center">
<div style="position:relative;display:inline;white-space:nowrap">
	<div id="webcamError" style="display:none;text-align:center">
		<div class="frame2" style="width:200px; text-align:center;margin-left:auto;margin-right:auto">
		<br/><div class="warning"><?php echo cfCaption('webcamError');?></div><br/>
		</div>
	</div>
<?php
// Webcam error div
echo ''."\n";

$webcamDetected=false;
// Browse resource directory to find a jpg image
if(($handle = @opendir(cfAppResourceDir()))) {
	while (false !== ($file = @readdir($handle)) && !$webcamDetected) {
		if(cfFileExtension($file)=='jpg'){

				/**
				 * Flash player
				 */
				if(cfRGetVar('useFlashPlayer')){
				$webcamWidth=320;
				$webcamHeight=200;
				$src=$_SERVER['REQUEST_URI'].'&amp;asyncRefreshXML=1';
?>
<object id="playerWebcam" classid="clsid:d27cdb6e-ae6d-11cf-96b8-444553540000" codebase="<?php echo 'http'.((isset($_SERVER['HTTPS']))?'s':'');?>://fpdownload.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=7,0,0,0" style="width:<?php echo $webcamWidth; ?>px; height:<?php echo $webcamHeight; ?>px" align="middle">
	<param name="allowScriptAccess" value="sameDomain" />
	<param name="allowFullScreen" value="true" />
	<param name="fullScreenRect" value="true" />
	<param name="movie" value="<?php echo cfHostName();?>/js/webcam.swf?file=<?php echo $src; ?>&amp;browser=<?php echo cfGetBrowser();?>" />
	<param name="quality" value="high" />
	<param name="bgcolor" value="#000000" />
	<embed name="playerWebcam" id="playerWebcamEmbed"
		style="width:<?php echo $webcamWidth; ?>px; height:<?php echo $webcamHeight; ?>px" align="middle"
		src="<?php echo cfHostName();?>/js/webcam.swf?file=<?php echo $src; ?>&amp;browser=<?php echo cfGetBrowser();?>"
		swLiveConnect="true"
		quality="high"
		allowFullScreen="true"
		fullScreenRect="true"
		align="middle"
		allowScriptAccess="sameDomain"
		type="application/x-shockwave-flash"
		pluginspage="<?php echo 'http'.((isset($_SERVER['HTTPS']))?'s':'');?>://www.macromedia.com/go/getflashplayer"/>
</object>
<?php
			}
			/**
			 * Still image
			 */
			else {
				echo '<img src="'.cfExtImage(cfJoinPathFile(cfAppResourceDir(),$file),false,false,false).'" id="webcamImg0" alt="">'."\n";
				echo '<img src="'.cfExtImage(cfJoinPathFile(cfAppResourceDir(),$file),false,false,false).'" id="webcamImg1" alt="" style="position:absolute;left:0px;'.outCssOpacity(1).'">'."\n";
			}
			$webcamDetected=true;
		}
	}
	closedir($handle);
}

?>
</div>
</td></tr></table>
</div>
<?php

// Flash / HTML toggle button
echo '<div style="position:absolute;top:0em;right:0em">'.outButton('','javascript:toggleFlash()',((cfRGetVar('useFlashPlayer'))?outIcon('noFlash'):outIcon('flash')),false,'toggleFlashBt').'</div>';

// Scripts
echo cfScriptLink('webcam.js');
// Display webcam not detected message if no image could be found
if(!@$webcamDetected) echo '<script type="text/javascript">noWebcamDetected();</script>';
?>
</body></html>